@@ -3,127 +3,117 @@
 """
 Copyright (C) 2010-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import functools
 import types
 
+from typing import List, Any, Optional, Dict, Union, TYPE_CHECKING, Callable, Type, cast
+
 from twisted.internet.defer import Deferred, succeed, fail
 
 from qt import QPropertyAnimation, QParallelAnimationGroup, \
     QAbstractAnimation, QEasingCurve
-from qt import pyqtProperty, QGraphicsObject, QGraphicsItem
+from qt import Property, QGraphicsObject, QGraphicsItem, QPointF, QObject
 
-from common import Internal, Debug, isAlive, StrMixin
-from log import logDebug, logException, id4
+from common import Internal, Debug, isAlive, ReprMixin, id4
+from log import logDebug, logException
 
+if TYPE_CHECKING:
+    from qt import QGraphicsScene
 
-class Animation(QPropertyAnimation, StrMixin):
+PropertyType = Union[QPointF,int,float]
+
+class Animation(QPropertyAnimation, ReprMixin):
 
     """a Qt animation with helper methods"""
 
-    nextAnimations = []
+    nextAnimations : List['Animation'] = []
     clsUid = 0
 
-    def __init__(self, graphicsObject, propName, endValue, parent=None):
+    def __init__(self, graphicsObject:'AnimatedMixin', propName:str,
+        endValue:PropertyType, parent:Optional['QObject']=None) ->None:
+        self.debug = graphicsObject.debug_name() in Debug.animation or Debug.animation == 'all'
+        self.debug |= f'T{id4(graphicsObject)}t' in Debug.animation
         Animation.clsUid += 1
         self.uid = Animation.clsUid
-        pName = propName
-        QPropertyAnimation.__init__(self, graphicsObject, pName.encode(), parent)
+        assert isinstance(graphicsObject, QObject)
+        QPropertyAnimation.__init__(self, graphicsObject, propName.encode(), parent)
         QPropertyAnimation.setEndValue(self, endValue)
+        assert Internal.Preferences
         duration = Internal.Preferences.animationDuration()
         self.setDuration(duration)
         self.setEasingCurve(QEasingCurve.InOutQuad)
         graphicsObject.queuedAnimations.append(self)
         Animation.nextAnimations.append(self)
-        self.debug = graphicsObject.name() in Debug.animation or Debug.animation == 'all'
-        self.debug |= 'T{}t'.format(id4(graphicsObject)) in Debug.animation
         if self.debug:
             oldAnimation = graphicsObject.activeAnimation.get(propName, None)
             if isAlive(oldAnimation):
+                assert isinstance(oldAnimation, Animation)
                 logDebug(
-                    'new Animation(%s) (after %s is done)' %
-                    (self, oldAnimation.ident()))
+                    f'new Animation({self}) (after {oldAnimation.ident()} is done)')
             else:
-                logDebug('Animation(%s)' % self)
+                logDebug(f'Animation({self})')
 
-    def setEndValue(self, endValue):
+    def setEndValue(self, endValue:PropertyType) ->None:
         """wrapper with debugging code"""
         graphicsObject = self.targetObject()
         if not isAlive(graphicsObject):
             # may happen when aborting a game because animations are cancelled first,
             # before the last move from server is executed
             return
-        if graphicsObject.name() in Debug.animation or Debug.animation == 'all':
-            pName = self.pName().decode()
+        if cast('AnimatedMixin', graphicsObject).debug_name() in Debug.animation or Debug.animation == 'all':
             logDebug(
-                '%s: change endValue for %s: %s->%s  %s' % (
-                    self.ident(), pName,
-                    self.formatValue(self.endValue()),
-                    self.formatValue(endValue), graphicsObject))
+                f'{self.ident()}: change endValue for {self.pName()}: '
+                f'{self.formatValue(self.endValue())}->{self.formatValue(endValue)}  {graphicsObject}')
         QPropertyAnimation.setEndValue(self, endValue)
 
-    def ident(self):
+    def ident(self) ->str:
         """the identifier to be used in debug messages"""
         pGroup = self.group() if isAlive(self) else 'notAlive'
         if pGroup or not isAlive(self):
-            return '%s/A%s' % (pGroup, id4(self))
-        else:
-            return 'A%s-%s' % (id4(self), self.targetObject().name())
+            return f'{pGroup}/A_{id4(self)}'
+        return f"A_{id4(self)}-{cast('AnimatedMixin', self.targetObject()).debug_name()}"
 
-    def pName(self):
+    def pName(self) ->str:
         """
         Return self.propertyName() as a python string.
 
         @return: C{str}
         """
-        if isAlive(self):
-            return bytes(self.propertyName()).decode()
-        else:
+        if not isAlive(self):
             return 'notAlive'
+        return bytes(self.propertyName()).decode()
 
-    def formatValue(self, value):
+    def formatValue(self, value:PropertyType) ->str:
         """string format the wanted value from qvariant"""
         pName = self.pName()
         if pName == 'pos':
-            return '%.0f/%.0f' % (value.x(), value.y())
+            assert isinstance(value, QPointF)
+            return f'{value.x():.0f}/{value.y():.0f}'
         if pName == 'rotation':
-            return '%d' % value
+            return f'{int(value)}'
         if pName == 'scale':
-            return '%.2f' % value
+            return f'{value:.2f}'
+        return f'formatValue: unexpected {pName}={value}'
 
-    def __str__(self):
+    def __str__(self) ->str:
         """for debug messages"""
         if isAlive(self) and isAlive(self.targetObject()):
             currentValue = getattr(self.targetObject(), self.pName())
             endValue = self.endValue()
-            targetObject = self.targetObject()
+            targetObject:Union[QObject, str] = self.targetObject()
         else:
             currentValue = 'notAlive'
             endValue = 'notAlive'
             targetObject = 'notAlive'
-        return '%s %s: %s->%s for %s duration=%dms' % (
-            self.ident(), self.pName(),
-            self.formatValue(currentValue),
-            self.formatValue(endValue),
-            targetObject,
-            self.duration())
+        return (f'{self.ident()} {self.pName()}: {self.formatValue(currentValue)}->'
+                f'{self.formatValue(endValue)} for {targetObject} duration={int(self.duration())}ms')
 
     @staticmethod
-    def removeImmediateAnimations():
+    def removeImmediateAnimations() ->None:
         """execute and remove immediate moves from the list
         We do not animate objects if
              - we are in a graphics object drag/drop operation
@@ -133,6 +123,8 @@ class Animation(QPropertyAnimation, StrMixin):
         """
         if Animation.nextAnimations:
             needRefresh = False
+            assert Internal.mainWindow
+            assert Internal.Preferences
             shortcutAll = (Internal.scene is None
                            or Internal.mainWindow.centralView.dragObject
                            or Internal.Preferences.animationSpeed == 99
@@ -141,13 +133,14 @@ class Animation(QPropertyAnimation, StrMixin):
                     # initial deal
             for animation in Animation.nextAnimations[:]:
                 if shortcutAll or animation.duration() == 0:
-                    animation.targetObject().shortcutAnimation(animation)
+                    cast(AnimatedMixin, animation.targetObject()).shortcutAnimation(animation)
                     Animation.nextAnimations.remove(animation)
                     needRefresh = True
             if needRefresh and Internal.scene:
                 Internal.scene.focusRect.refresh()
 
-class ParallelAnimationGroup(QParallelAnimationGroup, StrMixin):
+
+class ParallelAnimationGroup(QParallelAnimationGroup, ReprMixin):
 
     """
     current is the currently executed group
@@ -156,28 +149,25 @@ class ParallelAnimationGroup(QParallelAnimationGroup, StrMixin):
     doAfter to that other group.
     """
 
-    running = []  # we need a reference to active animation groups
+    running : List['ParallelAnimationGroup'] = []  # we need a reference to active animation groups
     current = None
     clsUid = 0
-
-
-    def __init__(self, animations, parent=None):
+    def __init__(self, animations:List[Animation], parent:Optional['QObject']=None) ->None:
         QParallelAnimationGroup.__init__(self, parent)
         self.animations = animations
         self.uid = ParallelAnimationGroup.clsUid
         ParallelAnimationGroup.clsUid += 1
-        self.deferred = Deferred()
+        self.deferred:Deferred = Deferred()
         self.deferred.addErrback(logException)
         self.steps = 0
         self.debug = any(x.debug for x in self.animations)
-        self.debug |= 'G{}g'.format(id4(self)) in Debug.animation
-        self.doAfter = list()
+        self.debug |= f'G{id4(self)}g' in Debug.animation
+        self.doAfter:List[Deferred] = []
         if ParallelAnimationGroup.current:
             if self.debug or ParallelAnimationGroup.current.debug:
-                logDebug('Chaining Animation group G%s to G%s' %
-                         (id4(self), ParallelAnimationGroup.current))
+                logDebug(f'Chaining Animation group G_{id4(self)} to G{ParallelAnimationGroup.current}')
             self.doAfter = ParallelAnimationGroup.current.doAfter
-            ParallelAnimationGroup.current.doAfter = list()
+            ParallelAnimationGroup.current.doAfter = []
             ParallelAnimationGroup.current.deferred.addCallback(self.start).addErrback(logException)
         else:
             self.start()
@@ -186,7 +176,7 @@ class ParallelAnimationGroup(QParallelAnimationGroup, StrMixin):
         self.stateChanged.connect(self.showState)
 
     @staticmethod
-    def cancelAll():
+    def cancelAll() ->None:
         """cancel all animations"""
         if Debug.quit:
             logDebug('Cancelling all animations')
@@ -194,61 +184,63 @@ class ParallelAnimationGroup(QParallelAnimationGroup, StrMixin):
             if isAlive(group):
                 group.clear()
 
-    def showState(self, newState, oldState):
-        """overrides Qt method"""
+    def showState(self, newState:int, oldState:int) ->None:
+        """override Qt method"""
         if self.debug:
-            logDebug('G{}: {} -> {} isAlive:{}'.format(
-                self.uid, self.stateName(oldState), self.stateName(newState), isAlive(self)))
+            logDebug(f'G{self.uid}: {self.stateName(oldState)} -> {self.stateName(newState)} isAlive:{isAlive(self)}')
 
-    def updateCurrentTime(self, value):
+    def updateCurrentTime(self, value:int) ->None:
         """count how many steps an animation does."""
         self.steps += 1
         if self.steps % 50 == 0:
             # periodically check if the board still exists.
             # if not (game end), we do not want to go on
             for animation in self.animations:
-                graphicsObject = animation.targetObject()
+                graphicsObject = cast(AnimatedMixin, animation.targetObject())
                 if hasattr(graphicsObject, 'board') and not isAlive(graphicsObject.board):
                     graphicsObject.clearActiveAnimation(animation)
                     self.removeAnimation(animation)
         QParallelAnimationGroup.updateCurrentTime(self, value)
 
-    def start(self, dummyResults='DIREKT'):
+    def start(self, unusedResults:str='DIREKT') ->Deferred:  # type: ignore[override]
         """start the animation, returning its deferred"""
         if not isAlive(self):
             return fail()
-        assert self.state() != QAbstractAnimation.Running
+        assert self.state() != QAbstractAnimation.State.Running
         for animation in self.animations:
             graphicsObject = animation.targetObject()
+ #            graphicsObject = cast(AnimatedMixin, animation.targetObject())
             if not isAlive(animation) or not isAlive(graphicsObject):
                 return fail()
-            graphicsObject.setActiveAnimation(animation)
+            animatedObject = cast(AnimatedMixin, graphicsObject)
+            animatedObject.setActiveAnimation(animation)
             self.addAnimation(animation)
             propName = animation.pName()
-            animation.setStartValue(graphicsObject.getValue(propName))
+            animation.setStartValue(animatedObject.getValue(propName))
             if propName == 'rotation':
                 # change direction if that makes the difference smaller
                 endValue = animation.endValue()
-                currValue = graphicsObject.rotation
+                currValue = animatedObject.rotation
                 if endValue - currValue > 180:
                     animation.setStartValue(currValue + 360)
                 if currValue - endValue > 180:
                     animation.setStartValue(currValue - 360)
         for animation in self.animations:
-            animation.targetObject().setDrawingOrder()
+            animation.targetObject().setDrawingOrder()  # type:ignore[attr-defined]
         self.finished.connect(self.allFinished)
         scene = Internal.scene
+        assert scene
         scene.focusRect.hide()
         QParallelAnimationGroup.start(
             self,
-            QAbstractAnimation.DeleteWhenStopped)
+            QAbstractAnimation.DeletionPolicy.DeleteWhenStopped)
         if self.debug:
-            logDebug('%s started with speed %d (%s)' % (
-                self, Internal.Preferences.animationSpeed,
-                ','.join('A%s' % id4(x) for x in self.animations)))
+            assert Internal.Preferences
+            _ = ','.join(f'A_{id4(x)}' for x in self.animations)
+            logDebug(f'{self} started with speed {int(Internal.Preferences.animationSpeed)} ({_})')
         return succeed(None).addErrback(logException)
 
-    def allFinished(self):
+    def allFinished(self) ->None:
         """all animations have finished. Cleanup and callback"""
         self.fixAllBoards()
         if self == ParallelAnimationGroup.current:
@@ -257,133 +249,141 @@ class ParallelAnimationGroup(QParallelAnimationGroup, StrMixin):
         if Debug.animationSpeed and self.duration():
             perSecond = self.steps * 1000.0 / self.duration()
             if perSecond < 50:
-                logDebug('%d steps for %d animations, %.1f/sec' %
-                         (self.steps, len(self.children()), perSecond))
+                logDebug(f'{int(self.steps)} steps for {len(self.children())} animations, {perSecond:.1f}/sec')
         # if we have a deferred, callback now
         assert self.deferred
         if self.debug:
-            logDebug('Done: {}'.format(self))
+            logDebug(f'Done: {self}')
         if self.deferred:
             self.deferred.callback(None)
         for after in self.doAfter:
             after.callback(None)
 
-    def fixAllBoards(self):
+    def fixAllBoards(self) ->None:
         """set correct drawing order for all moved graphics objects"""
         for animation in self.children():
-            graphicsObject = animation.targetObject()
+            graphicsObject = animation.targetObject()  # type:ignore[attr-defined]
             if graphicsObject:
                 graphicsObject.clearActiveAnimation(animation)
         if Internal.scene:
             Internal.scene.focusRect.refresh()
-        return
 
-    def stateName(self, state=None):
+    def stateName(self, state:Any=None) ->str:
         """for debug output"""
         if not isAlive(self):
             return 'not alive'
         if state is None:
             state = self.state()
-        if state == QAbstractAnimation.Stopped:
+        if state == QAbstractAnimation.State.Stopped:
             return 'stopped'
-        elif state == QAbstractAnimation.Running:
+        if state == QAbstractAnimation.State.Running:
             return 'running'
-        else:
-            assert False
+        return f'unknown state:{state}'
 
-    def __str__(self):
+    def __str__(self) ->str:
         """for debugging"""
-        return 'G{}({}:{})'.format(self.uid, len(self.animations), self.stateName())
+        return f'G{self.uid}({len(self.animations)}:{self.stateName()})'
+
 
 class AnimatedMixin:
-    """for UITile and PlayerWind"""
+    """for UITile and WindDisc"""
 
-    def __init__(self):
-        super(AnimatedMixin, self).__init__()
-        self.activeAnimation = dict()  # key is the property name
-        self.queuedAnimations = []
+    def __init__(self) ->None:
+        super().__init__()
+        self.activeAnimation:Dict[str, Animation]  = {}  # key is the property name
+        self.queuedAnimations:List[Animation] = []
 
-    def _get_pos(self):
+    def _get_pos(self) ->QPointF:
         """getter for property pos"""
-        return QGraphicsObject.pos(self)
+        return QGraphicsObject.pos(cast(QGraphicsItem, self))
 
-    def _set_pos(self, pos):
+    def _set_pos(self, pos:QPointF) ->None:
         """setter for property pos"""
-        QGraphicsObject.setPos(self, pos)
+        QGraphicsObject.setPos(cast(QGraphicsItem, self), pos)
 
-    pos = pyqtProperty('QPointF', fget=_get_pos, fset=_set_pos)
+    pos = Property(QPointF, fget=_get_pos, fset=_set_pos)
 
-    def _get_scale(self):
+    def _get_scale(self) ->float:
         """getter for property scale"""
-        return QGraphicsObject.scale(self)
+        return QGraphicsObject.scale(cast(QGraphicsItem, self))
 
-    def _set_scale(self, scale):
+    def _set_scale(self, scale:float) ->None:
         """setter for property scale"""
-        QGraphicsObject.setScale(self, scale)
+        QGraphicsObject.setScale(cast(QGraphicsItem, self), scale)
 
-    scale = pyqtProperty(float, fget=_get_scale, fset=_set_scale)
+    scale = Property(float, fget=_get_scale, fset=_set_scale)
 
-    def _get_rotation(self):
+    def _get_rotation(self) ->float:
         """getter for property rotation"""
-        return QGraphicsObject.rotation(self)
+        return QGraphicsObject.rotation(cast(QGraphicsItem, self))
 
-    def _set_rotation(self, rotation):
+    def _set_rotation(self, rotation:float) ->None:
         """setter for property rotation"""
-        QGraphicsObject.setRotation(self, rotation)
+        QGraphicsObject.setRotation(cast(QGraphicsItem, self), rotation)
 
-    rotation = pyqtProperty(float, fget=_get_rotation, fset=_set_rotation)
+    rotation = Property(float, fget=_get_rotation, fset=_set_rotation)
 
-    def queuedAnimation(self, propertyName):
+    def queuedAnimation(self, propertyName:str) ->Optional['Animation']:
         """return the last queued animation for this graphics object and propertyName"""
         for item in reversed(self.queuedAnimations):
             if item.pName() == propertyName:
                 return item
+        return None
 
-    def shortcutAnimation(self, animation):
+    def debug_name(self) ->str:
+        """for mypy"""
+        return ''
+
+#    this results in player names appearing BELOW their walls
+#    def setDrawingOrder(self) ->None:
+#        """for mypy"""
+#
+
+    def shortcutAnimation(self, animation:'Animation') ->None:
         """directly set the end value of the animation"""
         if animation.debug:
-            logDebug('shortcut {}: UTile {}: clear queuedAnimations'.format(animation, self.name()))
+            logDebug(f'shortcut {animation}: UTile {self.debug_name()}: clear queuedAnimations')
         setattr(self, animation.pName(), animation.endValue())
         self.queuedAnimations = []
-        self.setDrawingOrder()
+        self.setDrawingOrder()  # type:ignore[attr-defined] # TODO: mypy protocol?
 
-    def getValue(self, pName):
-        """gets a current property value"""
+    def getValue(self, pName:str) ->PropertyType:
+        """get a current property value"""
         return {'pos': self.pos, 'rotation': self.rotation,
                 'scale': self.scale}[pName]
 
-    def setActiveAnimation(self, animation):
+    def setActiveAnimation(self, animation:'Animation') ->None:
         """the graphics object knows which of its properties are currently animated"""
         self.queuedAnimations = []
         propName = animation.pName()
-        if self.name() in Debug.animation:
+        if self.debug_name() in Debug.animation:
             oldAnimation = self.activeAnimation.get(propName, None)
             if not isAlive(oldAnimation):
                 oldAnimation = None
             if oldAnimation:
-                logDebug('**** setActiveAnimation {} {}: {} OVERRIDES {}'.format(
-                    self.name(), propName, animation, oldAnimation))
+                logDebug(f'**** setActiveAnimation {self.debug_name()} {propName}: '
+                         f'{animation} OVERRIDES {oldAnimation}')
             else:
-                logDebug('setActiveAnimation {} {}: set {}'.format(self.name(), propName, animation))
+                logDebug(f'setActiveAnimation {self.debug_name()} {propName}: set {animation}')
         self.activeAnimation[propName] = animation
-        self.setCacheMode(QGraphicsItem.ItemCoordinateCache)
+        self.setCacheMode(QGraphicsItem.CacheMode.ItemCoordinateCache)  # type: ignore[attr-defined]
 
-    def clearActiveAnimation(self, animation):
+    def clearActiveAnimation(self, animation:'Animation') ->None:
         """an animation for this graphics object has ended.
         Finalize graphics object in its new position"""
         del self.activeAnimation[animation.pName()]
-        if self.name() in Debug.animation:
-            logDebug('UITile {}: clear activeAnimation[{}]'.format(self.name(), animation.pName()))
-        self.setDrawingOrder()
-        if not len(self.activeAnimation):
-            self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)
-            self.update()
+        if self.debug_name() in Debug.animation:
+            logDebug(f'UITile {self.debug_name()}: clear activeAnimation_{animation.pName()}')
+        self.setDrawingOrder()  # type:ignore[attr-defined] # TODO: mypy protocol?
+        if not self.activeAnimation:
+            self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)  # type: ignore[attr-defined]
+            self.update()  # type: ignore[attr-defined]
 
-    def setupAnimations(self):
+    def setupAnimations(self) ->None:
         """move the item to its new place. This puts new Animation
         objects into the queue to be animated by calling animate()"""
-        for pName, newValue in self.moveDict().items():
-            if self.scene() != Internal.scene:
+        for pName, newValue in self.moveDict().items():  # type: ignore[attr-defined]
+            if self.scene() != Internal.scene:  # type: ignore[attr-defined]
                 # not part of the playing scene, like tiles in tilesetselector
                 setattr(self, pName, newValue)
                 continue
@@ -392,13 +392,14 @@ class AnimatedMixin:
                 curValue = animation.endValue()
                 if curValue != newValue:
                     # change a queued animation
-                    if self.name() in Debug.animation:
-                        logDebug('setEndValue for {}: {}: {}->{}'.format(
-                            animation, pName, animation.formatValue(curValue), animation.formatValue(newValue)))
+                    if self.debug_name() in Debug.animation:
+                        logDebug(f'setEndValue for {animation}: {pName}: '
+                                 f'{animation.formatValue(curValue)}->{animation.formatValue(newValue)}')
                     animation.setEndValue(newValue)
             else:
                 animation = self.activeAnimation.get(pName, None)
                 if isAlive(animation):
+                    assert isinstance(animation, Animation)
                     curValue = animation.endValue()
                 else:
                     curValue = self.getValue(pName)
@@ -412,7 +413,7 @@ class AnimationSpeed:
 
     """a helper class for moving graphics with a given speed. 99=immediate."""
 
-    def __init__(self, speed=None):
+    def __init__(self, speed:Optional[int]=None) ->None:
         if speed is None:
             speed = 99
         if Internal.Preferences:
@@ -421,41 +422,40 @@ class AnimationSpeed:
             if Internal.Preferences.animationSpeed != speed:
                 Internal.Preferences.animationSpeed = speed
                 if Debug.animationSpeed:
-                    logDebug('AnimationSpeed sets speed %d' % speed)
+                    logDebug(f'AnimationSpeed sets speed {int(speed)}')
 
-    def __enter__(self):
+    def __enter__(self) ->'AnimationSpeed':
         return self
 
-    def __exit__(self, exc_type, exc_value, trback):
+    def __exit__(self, exc_type:Type[Exception], exc_value:Exception, trback:Any) ->None:
         """reset previous animation speed"""
         if Internal.Preferences:
             if self.__speed < 99:
                 animate()
             if Internal.Preferences.animationSpeed != self.prevAnimationSpeed:
                 if Debug.animationSpeed:
-                    logDebug('AnimationSpeed restores speed %d to %d' % (
-                        Internal.Preferences.animationSpeed, self.prevAnimationSpeed))
+                    logDebug(f'AnimationSpeed restores speed '
+                             f'{Internal.Preferences.animationSpeed} to {self.prevAnimationSpeed}')
                 Internal.Preferences.animationSpeed = self.prevAnimationSpeed
 
 
-def afterQueuedAnimations(doAfter):
+def afterQueuedAnimations(doAfter:Deferred) ->Callable:
     """A decorator"""
 
-    @functools.wraps(doAfter)
-    def doAfterQueuedAnimations(*args, **kwargs):
+    @functools.wraps(doAfter)  # type:ignore[arg-type]
+    def doAfterQueuedAnimations(*args:Any, **kwargs:Any) ->None:
         """do this after all queued animations have finished"""
-        method = types.MethodType(doAfter, args[0])
+        method = types.MethodType(doAfter, args[0])  # type:ignore[arg-type]
         args = args[1:]
-        varnames = doAfter.__code__.co_varnames
-        assert varnames[1] in ('deferredResult', 'dummyDeferredResult'), \
-            '{} passed {} instead of deferredResult'.format(
-                doAfter.__qualname__, varnames[1])
+        varnames = doAfter.__code__.co_varnames  # type:ignore[attr-defined]
+        assert varnames[1] in ('deferredResult', 'unusedDeferredResult'), \
+            f'{doAfter.__qualname__} passed {varnames[1]} instead of deferredResult'  # type:ignore[attr-defined]
         animateAndDo(method, *args, **kwargs)
 
     return doAfterQueuedAnimations
 
 
-def animate():
+def animate() ->Deferred:
     """now run all prepared animations. Returns a Deferred
         so callers can attach callbacks to be executed when
         animation is over.
@@ -470,12 +470,14 @@ def animate():
         return ParallelAnimationGroup.current.deferred
     return succeed(None).addErrback(logException)
 
-def doCallbackWithSpeed(result, speed, callback, *args, **kwargs):
+
+def doCallbackWithSpeed(result:Any, speed:int, callback:Callable, *args:Any, **kwargs:Any) ->None:
     """as the name says"""
     with AnimationSpeed(speed):
         callback(result, *args, **kwargs)
 
-def animateAndDo(callback, *args, **kwargs):
+
+def animateAndDo(callback:Callable, *args:Any, **kwargs:Any) ->Deferred:
     """if we want the next animations to have the same speed as the current group,
     do not use animate().addCallback() because speed would not be kept"""
     result = animate()
