@@ -18,7 +18,7 @@ class BackgroundSelector(QWidget):
 
     """presents all available backgrounds with previews"""
 
-    def __init__(self, parent):
+    def __init__(self, parent:QWidget) ->None:
         super().__init__(parent)
         loadUi(self)
         self.kcfg_backgroundName = QLineEdit(self)
@@ -26,7 +26,7 @@ class BackgroundSelector(QWidget):
         self.kcfg_backgroundName.setObjectName('kcfg_backgroundName')
         self.setUp()
 
-    def setUp(self):
+    def setUp(self) ->None:
         """fill the selector"""
 
         # The lineEdit widget holds our background path, but the user does
@@ -40,9 +40,10 @@ class BackgroundSelector(QWidget):
         self.backgroundList = Background.available()
         for aset in self.backgroundList:
             self.backgroundNameList.addItem(aset.name)
-        self.kcfg_backgroundName.setText(Internal.Preferences.backgroundName)
+        assert Internal.Preferences
+        self.kcfg_backgroundName.setText(str(Internal.Preferences.backgroundName))
 
-    def backgroundNameChanged(self, name):
+    def backgroundNameChanged(self, name:str) ->None:
         """the name changed: update the current row"""
         igrindex = 0
         for idx, aset in enumerate(self.backgroundList):
@@ -51,7 +52,7 @@ class BackgroundSelector(QWidget):
                 break
         self.backgroundNameList.setCurrentRow(igrindex)
 
-    def backgroundRowChanged(self):
+    def backgroundRowChanged(self) ->None:
         """user selected a new background, update our information about it and paint preview"""
         selBackground = self.backgroundList[
             self.backgroundNameList.currentRow()]
