@@ -15,17 +15,25 @@ import shutil
 import logging
 import logging.handlers
 import socket
+import string
 from signal import signal, SIGABRT, SIGINT, SIGTERM
+from typing import Optional, Any, Union, List, Sequence, Mapping
+from typing import TYPE_CHECKING, Iterable, Generator, Literal, cast
 
-from qt import QStandardPaths, QObject, QGraphicsItem, QSize
-from qtpy import PYSIDE_VERSION, QT5, QT6, compat
 from qtpy.compat import isalive as qtpy_isalive
+from qt import QStandardPaths, QObject, QSize
+
+if TYPE_CHECKING:
+    from tile import Tile
+    from mainwindow import MainWindow
+    from twisted.internet.interfaces import IReactorCore
+    from config import SetupPreferences
+    from qt import QGraphicsItem
+    from scene import GameScene
 
 # pylint: disable=invalid-name
 
-Internal = None
-
-if os.name == 'nt':
+if sys.platform == 'win32':
     # This is only needed for manual execution, and
     # we expect python to be the python3 interpreter.
     # The windows installer will use kajongg.exe and kajonggserver.exe
@@ -33,9 +41,11 @@ if os.name == 'nt':
 else:
     interpreterName = 'python3'
 
-LIGHTSOURCES = ['NE', 'NW', 'SW', 'SE']
+# TODO: enumeration
+LIGHTSOURCES = cast(Union[Literal['NE'], Literal['NW'], Literal['SW'], Literal['SE']], ['NE', 'NW', 'SW', 'SE'])
 
-def isAlive(qobj: QObject) ->bool:
+
+def isAlive(qobj: Union[QObject, 'QGraphicsItem', None]) ->bool:
     """check if the underlying C++ object still exists"""
     if qobj is None:
         return False
@@ -44,7 +54,7 @@ def isAlive(qobj: QObject) ->bool:
         print('NOT alive:', repr(qobj))
     return result
 
-def serverAppdataDir():
+def serverAppdataDir() ->str:
     """
     The per user directory with kajongg application information like the database.
 
@@ -58,11 +68,10 @@ def serverAppdataDir():
     # but only if .kajonggserver does not yet exist
     kdehome = os.environ.get('KDEHOME', '~/.kde')
     oldPath = os.path.expanduser(
-        kdehome +
-        '/share/apps/kajongg/kajonggserver.db')
+        f"{kdehome}/share/apps/kajongg/kajonggserver.db")
     if not os.path.exists(oldPath):
         oldPath = os.path.expanduser(
-            '~/.kde' +'4/share/apps/kajongg/kajonggserver.db')
+            '~/.kde4/share/apps/kajongg/kajonggserver.db')
     if os.path.exists(oldPath) and not os.path.exists(serverDir):
         # upgrading an old kajonggserver installation
         os.makedirs(serverDir)
@@ -75,7 +84,7 @@ def serverAppdataDir():
     return serverDir
 
 
-def clientAppdataDir():
+def clientAppdataDir() ->str:
     """
     The per user directory with kajongg application information like the database.
 
@@ -98,7 +107,7 @@ def clientAppdataDir():
     return result
 
 
-def appdataDir():
+def appdataDir() ->str:
     """
     The per user directory with kajongg application information like the database.
 
@@ -108,7 +117,7 @@ def appdataDir():
     return serverAppdataDir() if Internal.isServer else clientAppdataDir()
 
 
-def cacheDir():
+def cacheDir() ->str:
     """the cache directory for this user"""
     result = os.path.join(appdataDir(), '.cache')
     if not os.path.exists(result):
@@ -116,7 +125,7 @@ def cacheDir():
     return result
 
 
-def socketName():
+def socketName() ->str:
     """client and server process use this socket to talk to each other"""
     serverDir = os.path.expanduser('~/.kajonggserver')
     if not os.path.exists(serverDir):
@@ -125,17 +134,17 @@ def socketName():
                    # there
     if Options.socket:
         return Options.socket
-    return os.path.normpath('{}/socket{}'.format(serverDir, Internal.defaultPort))
+    return os.path.normpath(f'{serverDir}/socket{Internal.defaultPort}')
 
 
-def handleSignals(handler):
+def handleSignals(handler: Any) ->None:
 
     """set up signal handling"""
 
     signal(SIGABRT, handler)
     signal(SIGINT, handler)
     signal(SIGTERM, handler)
-    if os.name != 'nt':
+    if sys.platform != 'win32':
         from signal import SIGHUP, SIGQUIT
         signal(SIGHUP, handler)
         signal(SIGQUIT, handler)
@@ -150,6 +159,7 @@ class Debug:
     traffic = False
     process = False
     time = False
+    timestamp:Optional[datetime.datetime] = None
     sql = False
     animation = ''  # 'yeysywynG87gfefsfwfn' for tiles and G#g for groups where # is the uid
     animationSpeed = False
@@ -186,13 +196,13 @@ class Debug:
     i18n = False
     isalive = False
 
-    def __init__(self):
-        raise Exception('Debug is not meant to be instantiated')
+    def __init__(self) ->None:
+        raise TypeError('Debug is not meant to be instantiated')
 
     @staticmethod
-    def help():
+    def help() ->'str':
         """a string for help texts about debug options"""
-        def optYielder(options):
+        def optYielder(options: List[str]) -> Generator[str, None, None]:
             """yields options with markers for line separation"""
             for idx, opt in enumerate(options):
                 yield opt
@@ -203,54 +213,51 @@ class Debug:
                              if isinstance(Debug.__dict__[x], bool))
         stringOptions = sorted(x for x in options
                                if isinstance(Debug.__dict__[x], str))
-        stringExample = '%s:%s' % (stringOptions[0], 's3s4')
+        stringExample = f'{stringOptions[0]}:s3s4'
         allOptions = sorted(boolOptions + stringOptions)
         opt = '\n'.join(
             ', '.join(optYielder(allOptions)).split(' SEPARATOR, '))
         # TODO: i18n for this string. First move i18n out of kde so we can import it here
-        return """set debug options. Pass a comma separated list of options.
+        return f"""set debug options. Pass a comma separated list of options.
 Options are: {opt}.
-Options {stropt} take a string argument like {example}.
+Options {', '.join(stringOptions)} take a string argument like {stringExample}.
 --debug=events can get suboptions like in --debug=events:Mouse:Hide
-     showing all event messages with 'Mouse' or 'Hide' in them""".format(
-         opt=opt,
-         stropt=', '.join(stringOptions), example=stringExample)
+     showing all event messages with 'Mouse' or 'Hide' in them"""
 
     @staticmethod
-    def setOptions(args):
+    def setOptions(args: 'str') ->'str':
         """args comes from the command line. Put this in the Debug class.
         If something goes wrong, return an error message."""
         if not args:
-            return None
+            return ''
         Debug.argString = args
         for arg in args.split(','):
             parts = arg.split(':')
             option = parts[0]
+            value: Union[bool, str]
             if len(parts) == 1:
                 value = True
             else:
                 value = ':'.join(parts[1:])
             if option not in Debug.__dict__:
-                return '--debug: unknown option %s' % option
+                return f'--debug: unknown option {option}'
             if not isinstance(Debug.__dict__[option], type(value)):
-                return ('--debug: wrong type for option %s: '
-                        'given %s/%s, should be %s') % (
-                            option, value, type(value),
-                            type(Debug.__dict__[option]))
+                return (f"--debug: wrong type for option {option}: "
+                        f"given {value}/{type(value)}, should be {type(Debug.__dict__[option])}")
             if option != 'scores' or not Internal.isServer:
                 type.__setattr__(Debug, option, value)
         if Debug.time:
-            Debug.time = datetime.datetime.now()
+            Debug.timestamp = datetime.datetime.now()
         if Debug.modelTest and not Debug.modeltest_is_supported():
             print('--debug=modelTest is not yet supported for pyside, use pyqt')
             sys.exit(2)
-        return None
+        return ''
 
     @staticmethod
-    def modeltest_is_supported():
+    def modeltest_is_supported() ->bool:
         """Is the QT binding supported."""
         try:
-            import sip
+            import sip  # type:ignore[import]
         except ImportError:
             return False
         try:
@@ -260,26 +267,26 @@ Options {stropt} take a string argument like {example}.
             return False
 
     @staticmethod
-    def str():
+    def str() ->str:
         """__str__ does not work with class objects"""
         result = []
         for option in Debug.__dict__:
             if not option.startswith('_'):
-                result.append('{}={}'.format(option, getattr(Debug, option)))
+                result.append(f'{option}={getattr(Debug, option)}')
         return ' '.join(result)
 
+
 class FixedClass(type):
 
     """Metaclass: after the class variable fixed is set to True,
     all class variables become immutable"""
-    def __setattr__(cls, key, value):
-        if cls.fixed:
-            raise SystemExit('{cls}.{key} may not be changed'.format(
-                cls=cls.__name__, key=key))
+    def __setattr__(cls, key: str, value: object) ->None:
+        if cls.fixed:  # type: ignore
+            raise SystemExit(f'{cls.__name__}.{key} may not be changed')
         type.__setattr__(cls, key, value)
 
 
-class StrMixin:
+class ReprMixin:
 
     """
     A mixin defining a default for __repr__,
@@ -288,12 +295,12 @@ class StrMixin:
     runtime overhead to check for this beforehand.
     """
 
-    def __repr__(self):
+    def __repr__(self) ->str:
         clsName = self.__class__.__name__
         content = str(self)
         if content.startswith(clsName):
             return content
-        return '{cls}({content})'.format(cls=clsName, content=content)
+        return f'{clsName}_{id4(self)}({content})'
 
 
 class Options(metaclass=FixedClass):
@@ -308,7 +315,7 @@ class Options(metaclass=FixedClass):
     host = None
     player = None
     dbPath = None
-    socket = None
+    socket : Optional[str] = None
     port = None
     playOpen = False
     gui = False
@@ -317,18 +324,18 @@ class Options(metaclass=FixedClass):
     continueServer = False
     fixed = False
 
-    def __init__(self):
-        raise Exception('Options is not meant to be instantiated')
+    def __init__(self) ->None:
+        raise TypeError('Options is not meant to be instantiated')
 
     @staticmethod
-    def str():
+    def str() -> str:
         """__str__ does not work with class objects"""
         result = []
         for option in Options.__dict__:
             if not option.startswith('_'):
                 value = getattr(Options, option)
                 if isinstance(value, (bool, int, str)):
-                    result.append('{}={}'.format(option, value))
+                    result.append(f'{option}={value}')
         return ' '.join(result)
 
 class SingleshotOptions:
@@ -352,41 +359,35 @@ class __Internal:
     @type logPrefix: C{str}
     @cvar isServer: True if this is the server process.
     @type isServer: C{bool}
-    @cvar scaleScene: Defines if the scene is scaled.
-        Disable for debugging only.
-    @type scaleScene: C{bool}
     @cvar reactor: The twisted reactor instance.
     @type reactor: L{twisted.internet.reactor}
     @cvar app: The Qt or KDE app instance
     @type app: L{KApplication}
     @cvar db: The sqlite3 data base
     @type db: L{DBHandle}
-    @cvar scene: The QGraphicsScene.
-    @type scene: L{PlayingScene} or L{ScoringScene}
+    @cvar scene: The game scene.
+    @type scene: L{GameScene}: L{PlayingScene} or L{ScoringScene}
     """
-    # pylint: disable=too-many-instance-attributes
-    Preferences = None
+    Preferences:Optional['SetupPreferences'] = None
     defaultPort = 8301
     logPrefix = 'C'
     isServer = False
-    scaleScene = True
-    reactor = None
-    app = None
-    db = None
-    scene = None
-    mainWindow = None
+    reactor:'IReactorCore'
+    app : Any = None
+    db : Any = None
+    scene:Optional['GameScene'] = None
+    mainWindow : Optional['MainWindow'] = None
     game = None
     autoPlay = False
-    logger = None
-    kajonggrc = None
+    logger : Any = None
+    kajonggrc : Any = None
 
-    def __init__(self):
+    def __init__(self) ->None:
         """init the loggers"""
-        global Internal # pylint: disable=global-statement
-        Internal = self
-        logName = os.path.basename(sys.argv[0]).replace('.py', '').replace('.exe', '')  + '.log'
+        handler: Any
+        logName = f"{os.path.basename(sys.argv[0]).replace('.py', '').replace('.exe', '')}.log"
         self.logger = logging.getLogger(logName)
-        if os.name == 'nt':
+        if sys.platform == 'win32':
             haveDevLog = False
         else:
             try:
@@ -404,10 +405,10 @@ class __Internal:
         formatter = logging.Formatter("%(name)s: %(levelname)s %(message)s")
         handler.setFormatter(formatter)
 
-__Internal()
+Internal = __Internal()
 
 
-class IntDict(defaultdict, StrMixin):
+class IntDict(defaultdict, ReprMixin):
 
     """a dict where the values are expected to be numeric, so
     we can add dicts.If parent is given, parent is expected to
@@ -415,11 +416,11 @@ class IntDict(defaultdict, StrMixin):
     This allows us to have a tree of IntDicts, and we only have
     to update the leaves, getting the sums for free"""
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional['IntDict']=None) ->None:
         defaultdict.__init__(self, int)
         self.parent = parent
 
-    def copy(self):
+    def copy(self) -> 'IntDict':
         """need to reimplement this because the __init__ signature of
         IntDict is not identical to that of defaultdict"""
         result = IntDict(self.parent)
@@ -427,20 +428,22 @@ class IntDict(defaultdict, StrMixin):
         # see https://www.logilab.org/ticket/23986
         return result
 
-    def __add__(self, other):
+    def __add__(self, other: object) -> 'IntDict':
         """add two IntDicts"""
+        assert isinstance(other, IntDict), other
         result = self.copy()
         for key, value in other.items():
             result[key] += value
         return result
 
-    def __radd__(self, other):
+    def __radd__(self, other: object) -> 'IntDict':
         """we want sum to work (no start value)"""
         assert other == 0
         return self.copy()
 
-    def __sub__(self, other):
+    def __sub__(self, other: object) -> 'IntDict':
         """self - other"""
+        assert isinstance(other, IntDict), other
         result = self.copy()
         for key, value in other.items():
             result[key] -= value
@@ -449,57 +452,58 @@ class IntDict(defaultdict, StrMixin):
                 del result[key]
         return result
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) ->bool:
+        assert isinstance(other, IntDict), other
         return self.all() == other.all()
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) ->bool:
+        assert isinstance(other, IntDict), other
         return self.all() != other.all()
 
-    def count(self, countFilter=None):
+    def count(self, countFilter: Optional[Iterable]=None) ->int:
         """how many tiles defined by countFilter do we hold?
         countFilter is an iterator of element names. No countFilter: Take all
         So count(['we', 'ws']) should return 8"""
-        return sum((defaultdict.get(self, x) or 0)
+        return sum((defaultdict.get(self, x) or 0)  # type: ignore
                    for x in countFilter or self)
 
-    def all(self, countFilter=None):
+    def all(self, countFilter: Optional[Iterable]=None) ->List['Tile']:
         """return a list of all tiles defined by countFilter,
         each tile multiplied by its occurrence.
         countFilter is an iterator of element names. No countFilter: take all
         So all(['we', 'fs']) should return ['we', 'we', 'we', 'we', 'fs']"""
-        result = []
+        result : List['Tile'] = []
         for element in countFilter or self:
             result.extend([element] * self[element])
         return sorted(result)
 
-    def __contains__(self, tile):
+    def __contains__(self, tile: object) ->bool:
         """does not contain tiles with count 0"""
         return defaultdict.__contains__(self, tile) and self[tile] > 0
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: Any, value: Any) ->None:
         """also update parent if given"""
         if self.parent is not None:
-            self.parent[key] += value - defaultdict.get(self, key, 0)
+            self.parent[key] += value - defaultdict.get(self, key, cast(Any, 0))
         defaultdict.__setitem__(self, key, value)
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: Any) ->None:
         """also update parent if given"""
         if self.parent is not None:
-            self.parent[key] -= defaultdict.get(self, key, 0)
+            self.parent[key] -= defaultdict.get(self, key, cast(Any, 0))
         defaultdict.__delitem__(self, key)
 
-    def clear(self):
+    def clear(self) ->None:
         """also update parent if given"""
         if self.parent is not None:
             for key, value in defaultdict.items(self):
                 self.parent[key] -= value
         defaultdict.clear(self)
 
-    def __str__(self):
+    def __str__(self) ->str:
         """sort the result for better log comparison"""
         keys = sorted(self.keys())
-        return ', '.join('{}:{}'.format(
-            str(x), str(self[x])) for x in keys)
+        return ', '.join(f'{str(x)}:{str(self[x])}' for x in keys)
 
 
 class ZValues:
@@ -514,7 +518,7 @@ class ZValues:
 
 class Speeds:
     """some fixed animation speeds"""
-    windMarker = 20
+    windDisc = 20
     sideText = 60
 
 
@@ -522,10 +526,89 @@ class DrawOnTopMixin:
 
     """The inheriting QGraphicsObject will draw itself above all non moving tiles"""
 
-    def setDrawingOrder(self):
+    def setDrawingOrder(self) ->None:
         """we want us above all non moving tiles"""
-        if self.activeAnimation.get('pos'):
+        if self.activeAnimation.get('pos'):  # type: ignore
             movingZ = ZValues.movingZ
         else:
             movingZ = 0
-        self.setZValue(ZValues.markerZ + movingZ)
+        self.setZValue(ZValues.markerZ + movingZ)  # type: ignore
+
+
+def id4(obj: object) ->str:
+    """object id for debug messages"""
+    if obj is None:
+        return 'NONE'
+    try:
+        if hasattr(obj, 'uid'):
+            return obj.uid
+    except Exception:  # pylint: disable=broad-except
+        pass
+    return '.' if Debug.neutral else Fmt.num_encode(id(obj))
+
+
+class Fmt(string.Formatter):
+
+    """this formatter can parse {id(x)} and output a short ascii form for id"""
+    alphabet = string.ascii_uppercase + string.ascii_lowercase
+    base = len(alphabet)
+    formatter : Optional['Fmt'] = None
+
+    @staticmethod
+    def num_encode(number: int, length: int=4) ->str:
+        """make a short unique ascii string out of number, truncate to length"""
+        result : List[str] = []
+        while number and len(result) < length:
+            number, remainder = divmod(number, Fmt.base)
+            result.append(Fmt.alphabet[remainder])
+        return ''.join(reversed(result))
+
+    def get_value(self, key : Union[int, str], args: Sequence[Any], kwargs: Mapping[str, Any]) ->str:
+        assert isinstance(key, str), key
+        if key.startswith('id(') and key.endswith(')'):
+            idpar = key[3:-1]
+            if idpar == 'self':
+                idpar = 'SELF'
+            if kwargs[idpar] is None:
+                return 'None'
+            if Debug.neutral:
+                return '....'
+            return Fmt.num_encode(id(kwargs[idpar]))
+        if key == 'self':
+            return kwargs['SELF']
+        return kwargs[key]
+
+Fmt.formatter = Fmt()
+
+
+def fmt(text: str, **kwargs: Any) ->str:
+    """use the context dict for finding arguments.
+    For something like {self} output 'self:selfValue'"""
+    if '}' in text:
+        parts = []
+        for part in text.split('}'):
+            if '{' not in part:
+                parts.append(part)
+            else:
+                part2 = part.split('{')
+                if part2[1] == 'callers':
+                    if part2[0]:
+                        parts.append(f'{part2[0]}:{{{part2[1]}}}')
+                    else:
+                        parts.append(f'{{{part2[1]}}}')
+                else:
+                    showName = f"{part2[1]}:"
+                    if showName.startswith('_hide'):
+                        showName = ''
+                    if showName.startswith('self.'):
+                        showName = showName[5:]
+                    parts.append(f'{part2[0]}{showName}{{{part2[1]}}}')
+        text = ''.join(parts)
+    argdict = sys._getframe(1).f_locals  # pylint: disable=protected-access
+    argdict.update(kwargs)
+    if 'self' in argdict:
+        # formatter.format will not accept 'self' as keyword
+        argdict['SELF'] = argdict['self']
+        del argdict['self']
+    assert Fmt.formatter is not None
+    return Fmt.formatter.format(text, **argdict)
