@@ -10,6 +10,8 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
+from typing import TYPE_CHECKING
+
 from common import Internal
 
 from qt import Qt, QWidget, QSlider, QHBoxLayout, QLabel
@@ -22,6 +24,9 @@ from statesaver import StateSaver
 from tilesetselector import TilesetSelector
 from backgroundselector import BackgroundSelector
 
+if TYPE_CHECKING:
+    from qt import QKeyEvent
+
 __all__ = ['ConfigDialog']
 
 
@@ -29,11 +34,11 @@ class PlayConfigTab(QWidget):
 
     """Display Config tab"""
 
-    def __init__(self, parent):
+    def __init__(self, parent:QWidget) ->None:
         super().__init__(parent)
         self.setupUi()
 
-    def setupUi(self):
+    def setupUi(self) ->None:
         """layout the window"""
         self.setContentsMargins(0, 0, 0, 0)
         vlayout = QVBoxLayout(self)
@@ -52,7 +57,7 @@ class PlayConfigTab(QWidget):
         self.kcfg_propose.setObjectName('kcfg_propose')
         self.kcfg_animationSpeed = QSlider(self)
         self.kcfg_animationSpeed.setObjectName('kcfg_animationSpeed')
-        self.kcfg_animationSpeed.setOrientation(Qt.Horizontal)
+        self.kcfg_animationSpeed.setOrientation(Qt.Orientation.Horizontal)
         self.kcfg_animationSpeed.setSingleStep(1)
         lblSpeed = QLabel(i18n('Animation speed:'))
         lblSpeed.setBuddy(self.kcfg_animationSpeed)
@@ -79,16 +84,16 @@ class PlayConfigTab(QWidget):
         self.setSizePolicy(pol)
         self.retranslateUi()
 
-    def retranslateUi(self):
+    def retranslateUi(self) ->None:
         """translate to current language"""
 
 
-class ConfigDialog(KConfigDialog):  # pylint: disable=too-many-public-methods
+class ConfigDialog(KConfigDialog):
 
     """configuration dialog with several pages"""
 
-    def __init__(self, parent, name):
-        # pylint: disable=super-init-not-called
+    def __init__(self, parent:QWidget, name:str) ->None:
+        assert Internal.Preferences
         KConfigDialog.__init__(
             self, parent, name, Internal.Preferences)
         StateSaver(self)
@@ -103,11 +108,11 @@ class ConfigDialog(KConfigDialog):  # pylint: disable=too-many-public-methods
                 BackgroundSelector(self),
                 i18n("Backgrounds"), "games-config-background")]
 
-    def keyPressEvent(self, event):
+    def keyPressEvent(self, event:'QKeyEvent') ->None:
         """The four tabs can be selected with CTRL-1 .. CTRL-4"""
         mod = event.modifiers()
         key = chr(event.key() % 128)
-        if Qt.ControlModifier | mod and key in '123456789'[:len(self.pages)]:
+        if Qt.KeyboardModifier.ControlModifier | mod and key in '123456789'[:len(self.pages)]:
             self.setCurrentPage(self.pages[int(key) - 1])
             return
         KConfigDialog.keyPressEvent(self, event)
