@@ -7,6 +7,8 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
+from typing import List, Tuple, TYPE_CHECKING, Optional, Union
+
 from qt import Qt, QAbstractTableModel, QModelIndex
 from qt import QLabel, QDialog, QHBoxLayout, QVBoxLayout, QDialogButtonBox
 
@@ -17,29 +19,32 @@ from guiutil import BlockSignals
 from common import Debug
 from modeltest import ModelTest
 
+if TYPE_CHECKING:
+    from qt import QWidget
+    from rule import Ruleset
+
 
 class DifferModel(QAbstractTableModel):
 
     """a model for our ruleset differ"""
 
-    def __init__(self, diffs, view):
+    def __init__(self, diffs:List[Tuple[str,str,str]], view:'RulesetDiffer') ->None:
         super().__init__()
         self.diffs = diffs
         self.view = view
 
-    def columnCount(self, unusedIndex=QModelIndex()):
-        # pylint: disable=no-self-use
+    def columnCount(self, unusedIndex:QModelIndex=QModelIndex()) ->int:
         """how many columns does this node have?"""
         return 3  # rule name, left values, right values
 
-    def rowCount(self, parent):
+    def rowCount(self, parent:QModelIndex=QModelIndex()) ->int:
         """how many items?"""
         if parent.isValid():
             # we have only top level items
             return 0
         return len(self.diffs)
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index:QModelIndex, role:int=Qt.ItemDataRole.DisplayRole) ->Union[int,str,None]:
         """get from model"""
         if not index.isValid():
             return None
@@ -47,20 +52,20 @@ class DifferModel(QAbstractTableModel):
             return None
         diff = self.diffs[index.row()]
         column = index.column()
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return diff[column]
-        if role == Qt.TextAlignmentRole:
-            return int(Qt.AlignLeft | Qt.AlignVCenter)
+        if role == Qt.ItemDataRole.TextAlignmentRole:
+            return int(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
         return None
 
-    def headerData(self, section, orientation, role):
+    def headerData(self, section:int, orientation:int, role:int=Qt.ItemDataRole.DisplayRole) ->Union[int,str,None]:
         """tell the view about the wanted headers"""
-        if role == Qt.TextAlignmentRole:
-            if orientation == Qt.Horizontal:
-                return int(Qt.AlignLeft | Qt.AlignVCenter)
-        if role != Qt.DisplayRole:
+        if role == Qt.ItemDataRole.TextAlignmentRole:
+            if orientation == Qt.Orientation.Horizontal:
+                return int(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
-        if orientation == Qt.Horizontal:
+        if orientation == Qt.Orientation.Horizontal:
             if section == 0:
                 return i18nc('Kajongg', 'Rule')
             if section == 1:
@@ -74,12 +79,9 @@ class RulesetDiffer(QDialog):
 
     """Shows differences between rulesets"""
 
-    def __init__(self, leftRulesets, rightRulesets, parent=None):
+    def __init__(self, leftRulesets:List['Ruleset'], rightRulesets:List['Ruleset'],
+        parent:Optional['QWidget']=None) ->None:
         QDialog.__init__(self, parent)
-        if not isinstance(leftRulesets, list):
-            leftRulesets = list([leftRulesets])
-        if not isinstance(rightRulesets, list):
-            rightRulesets = list([rightRulesets])
         leftRulesets, rightRulesets = leftRulesets[:], rightRulesets[:]
         # remove rulesets from right which are also on the left side
         for left in leftRulesets:
@@ -95,8 +97,8 @@ class RulesetDiffer(QDialog):
                         x for x in rightRulesets if id(x) != id(right)]
         self.leftRulesets = leftRulesets
         self.rightRulesets = rightRulesets
-        self.model = None
-        self.modelTest = None
+        self.model:DifferModel
+        self.modelTest:ModelTest
         self.view = MJTableView(self)
         self.buttonBox = QDialogButtonBox()
         self.buttonBox.setStandardButtons(QDialogButtonBox.Ok)
@@ -128,31 +130,31 @@ class RulesetDiffer(QDialog):
         self.leftRulesetChanged()
         StateSaver(self)
 
-    def leftRulesetChanged(self):
+    def leftRulesetChanged(self) ->None:
         """slot to be called if the left ruleset changes"""
         if len(self.leftRulesets) == 1:
             self.orderRight()
         self.rulesetChanged()
 
-    def rulesetChanged(self):
+    def rulesetChanged(self) ->None:
         """slot to be called if the right ruleset changes"""
         self.model = DifferModel(self.formattedDiffs(), self)
         if Debug.modelTest:
             self.modelTest = ModelTest(self.model, self)
         self.view.setModel(self.model)
 
-    def orderRight(self):
+    def orderRight(self) ->None:
         """order the right rulesets by similarity to current left ruleset.
         Similarity is defined by the length of the diff list."""
         leftRuleset = self.cbRuleset1.current
         diffPairs = sorted((len(x.diff(leftRuleset)), x)
                            for x in self.rightRulesets)
         combo = self.cbRuleset2
-        with BlockSignals(combo):
+        with BlockSignals([combo]):
             combo.items = [x[1] for x in diffPairs]
         combo.setCurrentIndex(0)
 
-    def formattedDiffs(self):
+    def formattedDiffs(self) ->List[Tuple[str, str, str]]:
         """a list of tuples with 3 values: name, left, right"""
         formatted = []
         leftRuleset = self.cbRuleset1.current
