@@ -10,23 +10,22 @@ SPDX-License-Identifier: GPL-2.0
 
 # pylint: disable=wrong-import-position
 
-# keyboardinterrupt should simply terminate
-# import signal
-# signal.signal(signal.SIGINT, signal.SIG_DFL)
 import sys
 import os
 import logging
+from typing import Tuple, List, Optional, Type, Any
 
-from qt import QObject, QCommandLineParser, QCommandLineOption, Qt, QGuiApplication
 from qtpy import QT5
+from qt import QObject, QCommandLineParser, QCommandLineOption, Qt, QGuiApplication
 from kde import KApplication
 from mi18n import i18n, MLocale
 
 from common import Options, SingleshotOptions, Internal, Debug
 # do not import modules using twisted before our reactor is running
 
-def initRulesets():
-    """exits if user only wanted to see available rulesets"""
+def __initRulesetsOrExit() ->None:
+    """exits if user only wanted to see available rulesets
+    or if the given ruleset does not exist"""
     import predefined
     predefined.load()
     if Options.showRulesets or Options.rulesetName:
@@ -44,11 +43,9 @@ def initRulesets():
             matches = [x for x in rulesets if Options.rulesetName in x]
             if len(matches) != 1:
                 if not matches:
-                    msg = 'Ruleset %s is unknown' % Options.rulesetName
+                    msg = f'Ruleset {Options.rulesetName} is unknown'
                 else:
-                    msg = 'Ruleset %s is ambiguous: %s' % (
-                        Options.rulesetName,
-                        ', '.join(matches))
+                    msg = f"Ruleset {Options.rulesetName} is ambiguous: {', '.join(matches)}"
                 Internal.db.close()
                 raise SystemExit(msg)
             Options.ruleset = rulesets[matches[0]]
@@ -56,9 +53,10 @@ def initRulesets():
 
 class CommandLineOption(QCommandLineOption):
     """add some helping attributes"""
-    def __init__(self, name, description, valueName=None,
-        defaultValue=None, optName=None, argType=None, singleshot=False):
-        QCommandLineOption.__init__(self, [name], description, valueName, defaultValue)
+    def __init__(self, name : str, description : str, valueName : Optional[str] =None,
+        defaultValue : Optional[str]=None, optName : Optional[str]=None,
+        argType : Optional[Type]=None, singleshot:bool=False) ->None:
+        QCommandLineOption.__init__(self, [name], description, valueName or '', defaultValue or '')
         if argType is None:
             if valueName is None:
                 argType = bool
@@ -68,13 +66,15 @@ class CommandLineOption(QCommandLineOption):
         self.optName = optName or name
         self.singleshot = singleshot
 
-def defineOptions():
+def defineOptions() -> Tuple[QCommandLineParser, List[CommandLineOption]]:
     """define command line options"""
     parser = QCommandLineParser()
     options = []
-    def option(*args, optName=None, argType=None, singleshot=False):
+    def option(name:str, description:str, valueName:Optional[str]=None, defaultValue:Optional[str]=None,
+        optName:Optional[str]=None, argType:Optional[Type]=None, singleshot:bool=False) ->None:
         """helper"""
-        opt = CommandLineOption(*args, optName=optName, argType=argType, singleshot=singleshot)
+        opt = CommandLineOption(name, description, valueName, defaultValue,
+            optName=optName, argType=argType, singleshot=singleshot)
         options.append(opt)
         parser.addOption(opt)
 
@@ -88,7 +88,6 @@ def defineOptions():
     option('table', i18n('start new TABLE'), 'TABLE', '1', argType=int, singleshot=True)
     option('join', i18n('join TABLE'), 'TABLE', '1', argType=int, singleshot=True)
     option('ruleset', i18n('use RULESET without asking'), 'RULESET', '', optName='rulesetName')
-    option('rounds', i18n('play only ROUNDS rounds per game. Only for debugging!'), 'ROUNDS', '4', argType=int)
     option('player', i18n('prefer PLAYER for next login'), 'PLAYER', '')
     option('ai', i18n('use AI variant for human player in demo mode'), 'AI', '', optName='AI')
     option('csv', i18n('write statistics to CSV'), 'CSV', '')
@@ -96,12 +95,14 @@ def defineOptions():
     option('game', i18n('for testing purposes: Initializes the random generator'),
            'seed(/firsthand)(..(lasthand))', '0')
     option('nogui', i18n('show no graphical user interface. Intended only for testing'), optName='gui')
-    option('socket', i18n('use a dedicated server listening on SOCKET. Intended only for testing'), 'SOCKET', '')
-    option('port', i18n('use a dedicated server listening on PORT. Intended only for testing'), 'PORT', '')
+    option('socket', i18n(
+        'use a dedicated server already running and listening on SOCKET. Intended only for testing'), 'SOCKET', '')
+    option('port', i18n(
+        'use a dedicated server already running and listening on PORT. Intended only for testing'), 'PORT', '')
     option('debug', Debug.help(), 'DEBUG', '')
     return parser, options
 
-def parseOptions():
+def parseOptions() ->None:
     """parse command line options and save the values"""
     Options.gui = True
     parser, options = defineOptions()
@@ -116,6 +117,7 @@ def parseOptions():
                     logging.shutdown()
                     sys.exit(2)
                 continue
+            target: Type
             if option.optName in SingleshotOptions.__dict__:
                 target = SingleshotOptions
             else:
@@ -133,7 +135,7 @@ def parseOptions():
     from query import initDb
     if not initDb():
         raise SystemExit('Cannot initialize database')
-    initRulesets()
+    __initRulesetsOrExit()
     Options.fixed = True  # may not be changed anymore
 
 
@@ -141,7 +143,7 @@ class EvHandler(QObject):
 
     """an application wide event handler"""
 
-    def eventFilter(self, receiver, event):
+    def eventFilter(self, receiver: Any, event: Any) ->bool:
         """will be called for all events"""
         from log import EventData
         EventData(receiver, event)
@@ -149,13 +151,14 @@ class EvHandler(QObject):
 
 from util import gitHead
 
-if os.name == 'nt':
+if sys.platform == 'win32':
     _ = os.path.dirname(os.path.realpath(__file__))
     if _.endswith('.zip'):
         # cx_freeze
         os.chdir(os.path.dirname(_))
 
-QGuiApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)
+QGuiApplication.setAttribute(Qt.ApplicationAttribute.AA_ShareOpenGLContexts, True)
+
 Internal.app = KApplication()
 parseOptions()
 
@@ -182,8 +185,9 @@ if Options.csv:
             'You cannot write to %s with changes uncommitted to git',
             Options.csv)
         sys.exit(2)
+
 from mainwindow import MainWindow
 if QT5:
-    QGuiApplication.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
+    QGuiApplication.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps, True)
 MainWindow()
 Internal.app.exec_()
