@@ -3,19 +3,8 @@
 """
 Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import socket
@@ -24,6 +13,7 @@ import datetime
 import os
 import sys
 from itertools import chain
+from typing import List, Tuple, Union, Optional, TYPE_CHECKING, Any
 
 from twisted.spread import pb
 from twisted.cred import credentials
@@ -44,7 +34,7 @@ from log import logWarning, logException, logInfo, logDebug, SERVERMARK
 from util import removeIfExists, which
 from common import Internal, Options, SingleshotOptions, Debug, isAlive
 from common import interpreterName
-from common import StrMixin
+from common import ReprMixin
 from common import appdataDir, socketName
 from game import Players
 from query import Query
@@ -53,18 +43,28 @@ from statesaver import StateSaver
 from guiutil import ListComboBox, decorateWindow
 from rule import Ruleset
 
+if TYPE_CHECKING:
+    from twisted.internet.defer import Deferred
+    from twisted.internet.unix import Connector
+    from humanclient import HumanClient
+    from deferredutil import Request
 
 class LoginAborted(Exception):
 
     """the user aborted the login"""
-    pass
 
 
-class Url(str, StrMixin):
+class Url(str, ReprMixin):
 
     """holds connection related attributes: host, port, socketname"""
-    # pylint: disable=too-many-public-methods
-    def __new__(cls, url):
+
+    def __init__(self, url:str) ->None:
+        assert url
+        super().__init__()
+        self.host: str
+        self.port:Optional[int]
+
+    def __new__(cls, url:str) ->'Url':
         assert url
         host = None
         port = None
@@ -88,32 +88,29 @@ class Url(str, StrMixin):
 
         return obj
 
-    def __str__(self):
+    def __str__(self) ->str:
         """show all info"""
-        if self.useSocket:
-            return socketName()
-        else:
-            return '{}:{}'.format(self.host, self.port)
+        return socketName() if self.useSocket else f'{self.host}:{self.port}'
 
     @property
-    def useSocket(self):
+    def useSocket(self) ->bool:
         """do we use socket for current host?"""
         return (
             self.host == '127.0.0.1'
-            and os.name != 'nt'
+            and sys.platform != 'win32'
             and not Options.port)
 
     @property
-    def isLocalGame(self):
+    def isLocalGame(self) ->bool:
         """Are we playing a local game not needing the network?"""
         return self.host == '127.0.0.1'
 
     @property
-    def isLocalHost(self):
+    def isLocalHost(self) ->bool:
         """do server and client run on the same host?"""
         return self.host in ('127.0.0.1', 'localhost')
 
-    def findFreePort(self):
+    def findFreePort(self) ->Optional[int]:
         """find an unused port on the current system.
         used when we want to start a local server on windows"""
         assert self.isLocalHost
@@ -126,42 +123,40 @@ class Url(str, StrMixin):
             except socket.error:
                 return port
         logException('cannot find a free port')
+        return None
 
-    def startServer(self, result, waiting=0):
+    def startServer(self, result:Any, waiting:int=0) ->'Deferred':
         """make sure we have a running local server or network connectivity"""
         if self.isLocalHost:
             # just wait for that server to appear
             if self.__serverListening():
                 return result
-            else:
-                if waiting == 0:
-                    self.__startLocalServer()
-                elif waiting > 30:
-                    logDebug('Game %s: Server %s not available after 30 seconds, aborting' % (
-                        SingleshotOptions.game, self))
-                    raise CancelledError
-                return deferLater(Internal.reactor, 1, self.startServer, result, waiting + 1)
-        elif which('qdbus'):
+            if waiting == 0:
+                self.__startLocalServer()
+            elif waiting > 30:
+                logDebug(f'Game {SingleshotOptions.game}: Server {self} not available after 30 seconds, aborting')
+                raise CancelledError
+            return deferLater(Internal.reactor, 1, self.startServer, result, waiting + 1)
+        if which('qdbus'):
             try:
-                stdoutdata, stderrdata = subprocess.Popen(
+                stdoutdata, stderrdata = subprocess.Popen(  # pylint:disable=consider-using-with
                     ['qdbus',
                      'org.kde.kded',
                      '/modules/networkstatus',
                      'org.kde.Solid.Networking.status'],
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate(timeout=1)
-            except subprocess.TimeoutExpired:
-                raise twisted.internet.error.ConnectError()
+            except subprocess.TimeoutExpired as _:
+                raise twisted.internet.error.ConnectError() from _
             stdoutdata = stdoutdata.strip()
             stderrdata = stderrdata.strip()
             if stderrdata == '' and stdoutdata != '4':
-                # pylint: disable=nonstandard-exception
                 raise twisted.internet.error.ConnectError()
             # if we have stderrdata, qdbus probably does not provide the
             # service we want, so ignore it
         return result
 
     @staticmethod
-    def __findServerProgram():
+    def __findServerProgram() ->List[str]:
         """how should we start the server?"""
         result = []
         if sys.argv[0].endswith('kajongg.py'):
@@ -182,32 +177,31 @@ class Url(str, StrMixin):
             logDebug(i18n('trying to start local server %1', result))
         return result
 
-    def __startLocalServer(self):
+    def __startLocalServer(self) ->None:
         """start a local server"""
         try:
             args = self.__findServerProgram()
-            if self.useSocket or os.name == 'nt':  # for nt --socket tells the server to bind to 127.0.0.1
-                args.append('--socket=%s' % socketName())
+            if self.useSocket or sys.platform == 'win32':  # for win32 --socket tells the server to bind to 127.0.0.1
+                args.append(f'--socket={socketName()}')
                 if removeIfExists(socketName()):
                     logInfo(
                         i18n('removed stale socket <filename>%1</filename>', socketName()))
             if not self.useSocket:
-                args.append('--port=%d' % self.port)
+                assert self.port
+                args.append(f'--port={int(self.port)}')
             if self.isLocalGame:
                 args.append(
-                    '--db={}'.format(
-                        os.path.normpath(
-                            os.path.join(appdataDir(), 'local3.db'))))
+                    f"--db={os.path.normpath(os.path.join(appdataDir(), 'local3.db'))}")
             if Debug.argString:
-                args.append('--debug=%s' % Debug.argString)
-            if os.name == 'nt':
-                startupinfo = subprocess.STARTUPINFO()
-                startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+                args.append(f'--debug={Debug.argString}')
+            if sys.platform == 'win32':
+                startupinfo = subprocess.STARTUPINFO()  # type: ignore
+                startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW  # type: ignore
             else:
                 startupinfo = None
-            process = subprocess.Popen(
+            process = subprocess.Popen(  # pylint:disable=consider-using-with
                 args,
-                startupinfo=startupinfo)  # , shell=os.name == 'nt')
+                startupinfo=startupinfo)  # , shell=sys.platform == 'win32')
             if Debug.connections:
                 logDebug(
                     i18n(
@@ -217,16 +211,9 @@ class Url(str, StrMixin):
             exc.filename = ' '.join(args)
             logException(exc)
 
-    def __serverListening(self):
-        """is the expected server listening?"""
-        if self.useSocket:
-            proto = socket.AF_UNIX
-            param = socketName()
-            if not os.path.exists(param):
-                return False
-        else:
-            proto = socket.AF_INET
-            param = (self.host, self.port)
+    @staticmethod
+    def __check_socket(proto:int, param:Union[str, Tuple[str, Optional[int]]]) ->bool:
+        """check connection on socket"""
         sock = socket.socket(proto, socket.SOCK_STREAM)
         sock.settimeout(1)
         try:
@@ -236,20 +223,29 @@ class Url(str, StrMixin):
         except socket.error:
             return False
 
-    def connect(self, factory):
-        """returns a twisted connector"""
+    def __serverListening(self) ->bool:
+        """is the expected server listening?"""
+        if self.useSocket:
+            socket_path = socketName()
+            if not os.path.exists(socket_path):
+                return False
+            return self.__check_socket(socket.AF_UNIX, socket_path)
+        return self.__check_socket(socket.AF_INET, (self.host, self.port))
+
+    def connect(self, factory:pb.PBClientFactory) ->'Connector':
+        """return a twisted connector"""
+        assert Internal.reactor
         if self.useSocket:
             return Internal.reactor.connectUNIX(socketName(), factory, timeout=5)
-        else:
-            host = self.host
-            return Internal.reactor.connectTCP(host, self.port, factory, timeout=5)
+        host = self.host
+        return Internal.reactor.connectTCP(host, self.port, factory, timeout=5)
 
 
 class LoginDlg(QDialog):
 
     """login dialog for server"""
 
-    def __init__(self):
+    def __init__(self) ->None:
         """self.servers is a list of tuples containing server and last playername"""
         QDialog.__init__(self, None)
         decorateWindow(self, i18nc('kajongg', 'Login'))
@@ -258,7 +254,7 @@ class LoginDlg(QDialog):
         localName = i18nc('kajongg name for local game server', Query.localServerName)
         self.servers = Query(
             'select url,lastname from server order by lasttime desc').records
-        servers = list(x[0] for x in self.servers if x[0] != Query.localServerName)
+        servers = [x[0] for x in self.servers if x[0] != Query.localServerName]
         # the first server combobox item should be default: either the last used server
         # or localName for autoPlay
         if localName not in servers:
@@ -281,11 +277,11 @@ class LoginDlg(QDialog):
         self.serverChanged()
         StateSaver(self)
 
-    def returns(self, dummyButton=None):
+    def returns(self, unusedButton:Optional[str]=None) ->Tuple[Url, str, str, 'Ruleset']:
         """login data returned by this dialog"""
         return (Url(self.url), self.username, self.password, self.__defineRuleset())
 
-    def setupUi(self):
+    def setupUi(self) ->None:
         """create all Ui elements but do not fill them"""
         self.buttonBox = KDialogButtonBox(self)
         self.buttonBox.setStandardButtons(
@@ -293,7 +289,7 @@ class LoginDlg(QDialog):
         # Ubuntu 11.10 unity is a bit strange - without this, it sets focus on
         # the cancel button (which it shows on the left). I found no obvious
         # way to use setDefault and setAutoDefault for fixing this.
-        self.buttonBox.button(QDialogButtonBox.Ok).setFocus(True)
+        self.buttonBox.button(QDialogButtonBox.Ok).setFocus()
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
         vbox = QVBoxLayout(self)
@@ -307,20 +303,19 @@ class LoginDlg(QDialog):
         self.edPassword = QLineEdit()
         self.edPassword.setEchoMode(QLineEdit.PasswordEchoOnEdit)
         self.grid.addRow(i18n('Password:'), self.edPassword)
-        self.cbRuleset = ListComboBox()
+        self.cbRuleset = ListComboBox([])
         self.grid.addRow(i18nc('kajongg', 'Ruleset:'), self.cbRuleset)
         vbox.addLayout(self.grid)
         vbox.addWidget(self.buttonBox)
         pol = QSizePolicy()
         pol.setHorizontalPolicy(QSizePolicy.Expanding)
         self.cbUser.setSizePolicy(pol)
-        self.__port = None
 
-    def serverChanged(self, dummyText=None):
+    def serverChanged(self, unusedText:Optional[str]=None) ->None:
         """the user selected a different server"""
         records = Query('select player.name from player, passwords '
                         'where passwords.url=? and passwords.player = player.id', (self.url,)).records
-        players = list(x[0] for x in records)
+        players = [x[0] for x in records]
         preferPlayer = Options.player
         if preferPlayer:
             if preferPlayer in players:
@@ -329,7 +324,7 @@ class LoginDlg(QDialog):
         self.cbUser.clear()
         self.cbUser.addItems(players)
         if not self.cbUser.count():
-            user = KUser() if os.name == 'nt' else KUser(os.geteuid())
+            user = KUser() if sys.platform == 'win32' else KUser(os.geteuid())
             self.cbUser.addItem(user.fullName() or user.loginName())
         if not preferPlayer:
             userNames = [x[1] for x in self.servers if x[0] == self.url]
@@ -352,16 +347,15 @@ class LoginDlg(QDialog):
                 self.cbRuleset.items = Ruleset.selectableRulesets(self.url)
         self.buttonBox.button(QDialogButtonBox.Ok).setEnabled(bool(self.url))
 
-    def __defineRuleset(self):
+    def __defineRuleset(self) ->'Ruleset':
         """find out what ruleset to use"""
         if Options.ruleset:
             return Options.ruleset
-        elif Internal.autoPlay or bool(Options.host):
+        if Internal.autoPlay or bool(Options.host):
             return Ruleset.selectableRulesets()[0]
-        else:
-            return self.cbRuleset.current
+        return self.cbRuleset.current
 
-    def userChanged(self, text):
+    def userChanged(self, text:str) ->None:
         """the username has been changed, lookup password"""
         if text == '':
             self.edPassword.clear()
@@ -376,22 +370,22 @@ class LoginDlg(QDialog):
             self.edPassword.clear()
 
     @property
-    def url(self):
+    def url(self) ->str:
         """abstracts the url of the dialog"""
         return english(self.cbServer.currentText())
 
     @property
-    def username(self):
+    def username(self) ->str:
         """abstracts the username of the dialog"""
         return self.cbUser.currentText()
 
     @property
-    def password(self):
+    def password(self) ->str:
         """abstracts the password of the dialog"""
         return self.edPassword.text()
 
     @password.setter
-    def password(self, password):
+    def password(self, password:str) ->None:
         """abstracts the password of the dialog"""
         self.edPassword.setText(password)
 
@@ -399,12 +393,11 @@ class LoginDlg(QDialog):
 class AddUserDialog(KDialog):
 
     """add a user account on a server: This dialog asks for the needed attributes"""
-    # pylint: disable=too-many-instance-attributes
 
-    def __init__(self, url, username, password):
+    def __init__(self, url:str, username:str, password:str):
         KDialog.__init__(self)
         decorateWindow(self, i18nc("@title:window", "Create User Account"))
-        self.setButtons(KDialog.ButtonCode(KDialog.Ok | KDialog.Cancel))
+        self.setButtons(KDialog.Ok | KDialog.Cancel)
         vbox = QVBoxLayout()
         grid = QFormLayout()
         self.lbServer = QLabel()
@@ -434,11 +427,11 @@ class AddUserDialog(KDialog):
         self.passwordChanged()
         self.edPassword2.setFocus()
 
-    def passwordChanged(self, dummyText=None):
+    def passwordChanged(self, unusedText:Optional[str]=None) ->None:
         """password changed"""
         self.validate()
 
-    def validate(self):
+    def validate(self) ->None:
         """does the dialog hold valid data?"""
         equal = self.edPassword.size(
         ) and self.edPassword.text(
@@ -447,22 +440,22 @@ class AddUserDialog(KDialog):
         self.button(KDialog.Ok).setEnabled(equal)
 
     @property
-    def username(self):
+    def username(self) ->str:
         """abstracts the username of the dialog"""
         return self.lbUser.text()
 
     @username.setter
-    def username(self, username):
+    def username(self, username:str) ->None:
         """abstracts the username of the dialog"""
         self.lbUser.setText(username)
 
     @property
-    def password(self):
+    def password(self) ->str:
         """abstracts the password of the dialog"""
         return self.edPassword.text()
 
     @password.setter
-    def password(self, password):
+    def password(self, password:str) ->None:
         """abstracts the password of the dialog"""
         self.edPassword.setText(password)
 
@@ -471,28 +464,28 @@ class Connection:
 
     """creates a connection to server"""
 
-    def __init__(self, client):
+    def __init__(self, client:'HumanClient'):
         self.client = client
-        self.perspective = None
-        self.connector = None
-        self.url = None
-        self.username = None
-        self.password = None
-        self.__ruleset = None
+        self.perspective:pb.RemoteReference
+        self.connector: 'Connector'
+        self.url: Url
+        self.username: str
+        self.password: str
+        self.__ruleset: 'Ruleset'
         self.dlg = LoginDlg()
 
     @property
-    def ruleset(self):
+    def ruleset(self) ->'Ruleset':
         """reader"""
         return self.__ruleset
 
     @ruleset.setter
-    def ruleset(self, value):
+    def ruleset(self, value:'Ruleset') ->None:
         """save changed ruleset as last used ruleset for this server"""
-        if self.__ruleset != value:
+        if not hasattr(self, '__ruleset'):
             self.__ruleset = value
             if value:
-                def write():
+                def write() ->Query:
                     """write to database, returns 1 for success"""
                     return Query('update server set lastruleset=? where url=?', (value.rulesetId, self.url))
                 value.save()
@@ -502,9 +495,10 @@ class Connection:
                     self.__updateServerInfoInDatabase()
                     write()
 
-    def login(self):
+    def login(self) ->DeferredDialog:
         """to be called from HumanClient"""
-        result = DeferredDialog(self.dlg).addCallback(self.__haveLoginData)
+        result = DeferredDialog(self.dlg)
+        result.addCallback(self.__haveLoginData)
         result.addCallback(self.__checkExistingConnections)
         result.addCallback(self.__startServer)
         result.addCallback(self.__loginToServer)
@@ -514,33 +508,35 @@ class Connection:
             result.clicked()
         return result
 
-    def __haveLoginData(self, arguments):
+    def __haveLoginData(self, arguments:Tuple[Url, str, str, 'Ruleset']) ->None:
         """user entered login data, now try to login to server"""
+        if not Internal.autoPlay and self.dlg.result() == 0:
+            self._loginReallyFailed(Failure(CancelledError()))
         self.url, self.username, self.password, self.ruleset = arguments
         if self.url.isLocalHost:
             # we have localhost if we play a Local Game: client and server are identical,
             # we have no security concerns about creating a new account
             Players.createIfUnknown(self.dlg.cbUser.currentText())
 
-    def __startServer(self, result):
+    def __startServer(self, result:List['Request']) ->'Deferred':
         """if needed"""
         return self.url.startServer(result)
 
-    def __loginToServer(self, dummy=None):
+    def __loginToServer(self, unused:List['Request']) ->'Deferred':
         """login to server"""
         return self.loginCommand(self.username).addErrback(self._loginFailed)
 
-    def loggedIn(self, perspective):
+    def loggedIn(self, perspective:pb.RemoteReference) ->'Connection':
         """successful login on server"""
-        assert perspective
+        assert perspective, type(perspective)
         self.perspective = perspective
         self.perspective.notifyOnDisconnect(self.client.serverDisconnected)
         self.__updateServerInfoInDatabase()
-        self.dlg = None
+        self.dlg = None  # type: ignore
         self.pingLater()  # not right now, client.connection is still None
         return self
 
-    def __updateServerInfoInDatabase(self):
+    def __updateServerInfoInDatabase(self) ->None:
         """we are online. Update table server."""
         lasttime = datetime.datetime.now().replace(microsecond=0).isoformat()
         with Internal.db:
@@ -560,28 +556,30 @@ class Connection:
                 Query('insert into passwords(url,player,password) values(?,?,?)',
                       (self.url, playerId, self.password))
 
-    def __checkExistingConnections(self, dummy=None):
+    def __checkExistingConnections(self, unused:Optional[str]=None) ->None:
         """do we already have a connection to the wanted URL?"""
         for client in self.client.humanClients:
             if client.connection and client.connection.url == self.url:
                 logWarning(
                     i18n('You are already connected to server %1', self.url))
-                client.tableList.activateWindow()
+                if client.tableList:
+                    client.tableList.activateWindow()
                 raise CancelledError
 
-    def loginCommand(self, username):
+    def loginCommand(self, username:str) ->'Deferred':
         """send a login command to server. That might be a normal login
         or adduser/deluser/change passwd encoded in the username"""
         factory = pb.PBClientFactory(unsafeTracebacks=True)
         self.connector = self.url.connect(factory)
+        assert self.dlg
         utf8Password = self.dlg.password.encode('utf-8')
         utf8Username = username.encode('utf-8')
         cred = credentials.UsernamePassword(utf8Username, utf8Password)
         return factory.login(cred, client=self.client)
 
-    def __adduser(self):
+    def __adduser(self) ->'Deferred':
         """create a user account"""
-        assert self.url is not None
+        assert self.dlg
         if not self.url.isLocalHost:
             if not AddUserDialog(self.url,
                                  self.dlg.username,
@@ -592,26 +590,23 @@ class Connection:
             ['adduser', self.dlg.username, self.dlg.password])
         return self.loginCommand(adduserCmd)
 
-    def _loginFailed(self, failure):
+    def _loginFailed(self, failure:Failure) ->Union['Deferred', Failure]:
         """login failed"""
-        def answered(result):
+        def answered(result:bool) ->Union['Deferred', Failure]:
             """user finally answered our question"""
-            if result:
-                return self.__adduser()
-            else:
-                return Failure(CancelledError())
+            return self.__adduser() if result else Failure(CancelledError())
         message = failure.getErrorMessage()
         if 'Wrong username' in message:
             if self.url.isLocalHost:
                 return answered(True)
-            else:
-                msg = i18nc('USER is not known on SERVER',
-                            '%1 is not known on %2, do you want to open an account?', self.dlg.username, self.url.host)
-                return QuestionYesNo(msg).addCallback(answered)
-        else:
-            return self._loginReallyFailed(failure)
+            assert self.dlg
+            msg = i18nc('USER is not known on SERVER',
+                        '%1 is not known on %2, do you want to open an account?', self.dlg.username, self.url.host)
+            return QuestionYesNo(msg).addCallback(answered)
+        self._loginReallyFailed(failure)
+        return Failure() # only for mypy
 
-    def _loginReallyFailed(self, failure):
+    def _loginReallyFailed(self, failure:Failure) ->None:
         """login failed, not fixable by adding missing user"""
         msg = None
         if not isAlive(Internal.mainWindow):
@@ -631,26 +626,25 @@ class Connection:
                 'Login to server %1 failed: You have no network connection',
                 self.url)
         else:
-            msg = 'Login to server {} failed: {}/{} Callstack:{}'.format(
-                self.url, failure.value.__class__.__name__, failure.getErrorMessage(
-                ),
-                failure.getTraceback())
+            msg = (f'Login to server {self.url} failed: '
+                   f'{failure.value.__class__.__name__}/{failure.getErrorMessage()} Callstack:{failure.getTraceback()}')
+        assert msg
         # Maybe the server is running but something is wrong with it
         if self.url and self.url.useSocket:
             if removeIfExists(socketName()):
                 logInfo(
                     i18n('removed stale socket <filename>%1</filename>', socketName()))
             msg += '\n\n\n' + i18n('Please try again')
-        self.dlg = None
+        self.dlg = None  # type: ignore
         if msg:
             logWarning(msg)
         raise CancelledError
 
-    def pingLater(self, dummyResult=None):
+    def pingLater(self, unusedResult:Optional[str]=None) ->None:
         """ping the server every 5 seconds"""
         Internal.reactor.callLater(5, self.ping)
 
-    def ping(self):
+    def ping(self) ->None:
         """regularly check if server is still there"""
         if self.client.connection:
             # when pinging starts, we do have a connection and when the
@@ -659,3 +653,6 @@ class Connection:
                 'ping').addCallback(
                     self.pingLater).addErrback(
                         self.client.remote_serverDisconnects)
+
+    def __str__(self) ->str:
+        return f'{self.username}@{self.url}'
