@@ -7,36 +7,36 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
-# pylint: disable=wrong-import-order, wrong-import-position
+# pylint: disable=wrong-import-order
 
 import sys
 import codecs
 from itertools import chain
 
-import cgitb
+import cgitb  # pylint:disable=deprecated-module
 import tempfile
 import webbrowser
 import logging
+from typing import Any, TYPE_CHECKING, Optional, Union, Tuple, Type, cast
 
 from log import logError, logDebug
 from common import Options, Internal, isAlive, Debug, handleSignals
 
-
 class MyHook(cgitb.Hook):
 
     """override the standard cgitb hook: invoke the browser"""
 
-    def __init__(self):
+    def __init__(self) ->None:
         self.tmpFileName = tempfile.mkstemp(
             suffix='.html',
             prefix='bt_',
             text=True)[1]
         # cgitb can only handle ascii, work around that.
         # See https://bugs.python.org/issue22746
-        cgitb.Hook.__init__(self, file=codecs.open(self.tmpFileName, 'w',
+        cgitb.Hook.__init__(self, file=codecs.open(self.tmpFileName, 'w',  # pylint:disable=consider-using-with
                                                    encoding='latin-1', errors='xmlcharrefreplace'))
 
-    def handle(self, info=None):
+    def handle(self, info:Optional[Union[Tuple[Type[BaseException], Any, Any], Tuple[None, None, None]]]=None) ->None:
         """handling the exception: show backtrace in browser"""
         if getattr(cgitb, 'Hook', None):
             # if we cannot import twisted (syntax error), Hook is not yet known
@@ -49,16 +49,16 @@ class MyHook(cgitb.Hook):
 NOTFOUND = []
 
 try:
-    from qt import Qt, QEvent, QMetaObject, QTimer, QKeySequence
+    from qt import Qt, QEvent, QMetaObject, QTimer
     from qt import QWidget, QGridLayout, QAction
 except ImportError as importError:
-    NOTFOUND.append('Please install PyQt5: %s' % importError)
+    NOTFOUND.append(f'Please install PyQt5: {importError}')
 
 try:
     from twisted.spread import pb # pylint: disable=unused-import
     from twisted.internet.error import ReactorNotRunning
 except ImportError as importError:
-    NOTFOUND.append('Package python3-twisted is missing or too old (I need 16.6.0): %s' % importError)
+    NOTFOUND.append(f'Package python3-twisted is missing or too old (I need 16.6.0): {importError}')
 
 
 try:
@@ -82,19 +82,24 @@ try:
     from kdestub import Action, KApplication
 
 except ImportError as importError:
-    NOTFOUND.append('Kajongg is not correctly installed: modules: %s' %
-                    importError)
+    NOTFOUND.append(f'Kajongg is not correctly installed: modules: {importError}')
 
 if NOTFOUND:
-    logError("\n".join(" * %s" % s for s in NOTFOUND), showStack=False)
+    logError("\n".join(f" * {s}" for s in NOTFOUND), showStack=False)
     sys.exit(3)
 
+if TYPE_CHECKING:
+    from deferredutil import Request
+    from twisted.internet.defer import Deferred
+    from qt import QSize, QKeySequence
+    from scene import GameScene
 
-def cleanExit(*unusedArgs): # pylint: disable=unused-argument
+def cleanExit(*unusedArgs:Any) ->None:
     """close sqlite3 files before quitting"""
     if isAlive(Internal.mainWindow):
         if Debug.quit:
             logDebug('cleanExit calling mainWindow.close')
+        assert Internal.mainWindow  # for mypy
         Internal.mainWindow.close()
     else:
         # this must be very early or very late
@@ -111,20 +116,20 @@ class MainWindow(KXmlGuiWindow):
     """the main window"""
     # pylint: disable=too-many-instance-attributes
 
-    def __init__(self):
+    def __init__(self) ->None:
         # see https://marc.info/?l=kde-games-devel&m=120071267328984&w=2
         super().__init__()
         Internal.app.aboutToQuit.connect(self.aboutToQuit)
-        self.exitConfirmed = None
-        self.exitReady = None
-        self.exitWaitTime = None
+        self.exitConfirmed:Optional[bool] = None
+        self.exitReady:Optional[bool] = None
+        self.exitWaitTime:Optional[int] = None
         Internal.mainWindow = self
-        self._scene = None
-        self.centralView = None
-        self.background = None
-        self.playerWindow = None
-        self.rulesetWindow = None
-        self.confDialog = None
+        self._scene:Optional['GameScene'] = None
+        self.centralView: FittingView
+        self.background:Background = Background()
+        self.playerWindow:Optional[PlayerList] = None
+        self.rulesetWindow:Optional[RulesetSelector] = None
+        self.confDialog:Optional[ConfigDialog] = None
         self.__installReactor()
         if Options.gui:
             KStandardAction.preferences(
@@ -132,6 +137,7 @@ class MainWindow(KXmlGuiWindow):
                 self.actionCollection())
             self.setupUi()
             self.setupGUI()
+            assert Internal.Preferences
             Internal.Preferences.addWatch(
                 'tilesetName',
                 self.tilesetNameChanged)
@@ -141,7 +147,7 @@ class MainWindow(KXmlGuiWindow):
             self.retranslateUi()
             for action in self.toolBar().actions():
                 if 'onfigure' in action.text():
-                    action.setPriority(QAction.LowPriority)
+                    action.setPriority(QAction.Priority.LowPriority)
             if Options.host and not Options.demo:
                 self.scene = PlayingScene(self)
                 HumanClient()
@@ -151,24 +157,26 @@ class MainWindow(KXmlGuiWindow):
             HumanClient()
 
     @staticmethod
-    def __installReactor():
+    def __installReactor() ->None:
         """install the twisted reactor"""
-        if Internal.reactor is None:
+        if not hasattr(Internal, 'reactor'):
             import qtreactor
             qtreactor.install()
-            from twisted.internet import reactor
+            from twisted.internet.interfaces import IReactorCore
+            from twisted.internet import reactor as reactor_module
+            reactor = cast(IReactorCore, reactor_module)
             reactor.runReturn(installSignalHandlers=False)
             Internal.reactor = reactor
             if Debug.quit:
                 logDebug('Installed qtreactor')
 
     @property
-    def scene(self):
+    def scene(self) ->Optional['GameScene']:
         """a proxy"""
         return self._scene
 
     @scene.setter
-    def scene(self, value):
+    def scene(self, value:Optional['GameScene']) ->None:
         """if changing, updateGUI"""
         if not isAlive(self):
             return
@@ -181,7 +189,8 @@ class MainWindow(KXmlGuiWindow):
             self.actionExplain.setData(ExplainView)
             self.actionScoreTable.setData(ScoreTable)
         self._scene = value
-        self.centralView.setScene(value)
+        # contrary to what the Qt doc says, scene can be set to None
+        self.centralView.setScene(value)  # type:ignore[arg-type]
         self.adjustMainView()
         self.updateGUI()
         canDemo = not value or isinstance(value, PlayingScene)
@@ -190,7 +199,7 @@ class MainWindow(KXmlGuiWindow):
         self.actionExplain.setEnabled(value is not None)
         self.actionScoreTable.setEnabled(value is not None)
 
-    def sizeHint(self):
+    def sizeHint(self) ->'QSize':
         """give the main window a sensible default size"""
         result = KXmlGuiWindow.sizeHint(self)
         result.setWidth(result.height() * 3 // 2)
@@ -204,12 +213,8 @@ class MainWindow(KXmlGuiWindow):
         result.setWidth(width)
         return result
 
-    def showEvent(self, event):
-        """force a resize which calculates the correct background image size"""
-        self.centralView.resizeEvent(True)
-        KXmlGuiWindow.showEvent(self, event)
-
-    def _kajonggToggleAction(self, name, icon, shortcut=None, actionData=None):
+    def _kajonggToggleAction(self, name:str, icon:str, shortcut:Optional['Qt.Key']=None,
+        actionData:Optional[Union[str, Type['QWidget']]]=None) ->Action:
         """a checkable action"""
         res = Action(self,
             name,
@@ -221,7 +226,7 @@ class MainWindow(KXmlGuiWindow):
             res.toggled.connect(self._toggleWidget)
         return res
 
-    def setupUi(self):
+    def setupUi(self) ->None:
         """create all other widgets
         we could make the scene view the central widget but I did
         not figure out how to correctly draw the background with
@@ -229,7 +234,6 @@ class MainWindow(KXmlGuiWindow):
         QGraphicsView.drawBackground always wants a pixmap
         for a huge rect like 4000x3000 where my screen only has
         1920x1200"""
-        # pylint: disable=too-many-statements
         self.setObjectName("MainWindow")
         centralWidget = QWidget()
         self.centralView = FittingView()
@@ -237,35 +241,37 @@ class MainWindow(KXmlGuiWindow):
         layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(self.centralView)
         self.setCentralWidget(centralWidget)
-        self.centralView.setFocusPolicy(Qt.StrongFocus)
-        self.background = None  # just for pylint
-        self.windTileset = Tileset(Internal.Preferences.windTilesetName)
+        self.centralView.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
+        assert Internal.Preferences
+        _ = Internal.Preferences.windTilesetName
+        assert isinstance(_, str)
+        self.windTileset = Tileset(_)
         self.adjustMainView()
         self.actionScoreGame = Action(
             self,
             "scoreGame",
             "draw-freehand",
-            self.scoringScene,
-            Qt.Key_C)
+            self.startScoringGame,
+            Qt.Key.Key_C)
         self.actionPlayGame = Action(
             self,
             "play",
             "arrow-right",
             self.playGame,
-            Qt.Key_N)
+            Qt.Key.Key_N)
         self.actionAbortGame = Action(
             self,
             "abort",
             "dialog-close",
             self.abortAction,
-            Qt.Key_W)
+            Qt.Key.Key_W)
         self.actionAbortGame.setEnabled(False)
         self.actionQuit = Action(
             self,
             "quit",
             "application-exit",
             self.close,
-            Qt.Key_Q)
+            Qt.Key.Key_Q)
         self.actionPlayers = Action(
             self,
             "players", "im-user", self.slotPlayers)
@@ -275,50 +281,50 @@ class MainWindow(KXmlGuiWindow):
             "games-kajongg-law",
             self.slotRulesets)
         self.actionChat = self._kajonggToggleAction("chat", "call-start",
-                                                    shortcut=Qt.Key_H, actionData=ChatWindow)
+                                                    shortcut=Qt.Key.Key_H, actionData=ChatWindow)
         self.actionChat.setEnabled(False)
         self.actionAngle = Action(
             self,
             "angle",
             "object-rotate-left",
             self.changeAngle,
-            Qt.Key_G)
+            Qt.Key.Key_G)
         self.actionAngle.setEnabled(False)
         self.actionScoreTable = self._kajonggToggleAction(
             "scoreTable", "format-list-ordered",
-            Qt.Key_T, actionData=ScoreTable)
+            Qt.Key.Key_T, actionData=ScoreTable)
         self.actionScoreTable.setEnabled(False)
         self.actionExplain = self._kajonggToggleAction(
             "explain", "applications-education",
-            Qt.Key_E, actionData=ExplainView)
+            Qt.Key.Key_E, actionData=ExplainView)
         self.actionExplain.setEnabled(False)
         self.actionFullscreen = self._kajonggToggleAction(
-            "fullscreen", "view-fullscreen", shortcut=Qt.Key_F | Qt.ShiftModifier)
+            "fullscreen", "view-fullscreen", shortcut=cast(Qt.Key, Qt.Key.Key_F | Qt.KeyboardModifier.ShiftModifier))
         self.actionFullscreen.toggled.connect(self.fullScreen)
         self.actionAutoPlay = Action(
             self,
             "demoMode",
             "arrow-right-double",
             None,
-            Qt.Key_D)
+            Qt.Key.Key_D)
         self.actionAutoPlay.setCheckable(True)
         self.actionAutoPlay.setEnabled(True)
         self.actionAutoPlay.toggled.connect(self._toggleDemoMode)
         self.actionAutoPlay.setChecked(Internal.autoPlay)
         QMetaObject.connectSlotsByName(self)
 
-    def playGame(self):
+    def playGame(self) ->None:
         """manual wish for a new game"""
         if not Internal.autoPlay:
             # only if no demo game is running
-            self.playingScene()
+            self.startPlayingGame()
 
-    def playingScene(self):
+    def startPlayingGame(self) ->None:
         """play a computer game: log into a server and show its tables"""
         self.scene = PlayingScene(self)
         HumanClient()
 
-    def scoringScene(self):
+    def startScoringGame(self) ->None:
         """start a scoring scene"""
         scene = ScoringScene(self)
         game = scoreGame()
@@ -328,25 +334,31 @@ class MainWindow(KXmlGuiWindow):
             game.throwDices()
             self.updateGUI()
 
-    def fullScreen(self, toggle):
+    def fullScreen(self, toggle:bool) ->None:
         """toggle between full screen and normal view"""
+        _ = self.windowState()
         if toggle:
-            self.setWindowState(self.windowState() | Qt.WindowFullScreen)
+            new_state = _ | Qt.WindowState.WindowFullScreen
         else:
-            self.setWindowState(self.windowState() & ~Qt.WindowFullScreen)
+            new_state = _ & ~Qt.WindowState.WindowFullScreen
+        self.setWindowState(cast(Qt.WindowState, new_state))
 
-    def close(self, unusedResult=None):
+    def closeMe(self) ->None:
+        """for QTimer"""
+        self.close()
+
+    def close(self, unusedResult:Any=None) ->bool:
         """wrap close() because we call it with a QTimer"""
         if isAlive(self):
             return KXmlGuiWindow.close(self)
         return True  # is closed
 
-    def closeEvent(self, event):
+    def closeEvent(self, event:QEvent) ->None:
         KXmlGuiWindow.closeEvent(self, event)
         if event.isAccepted() and self.exitReady:
             QTimer.singleShot(5000, self.aboutToQuit)
 
-    def queryClose(self):
+    def queryClose(self) ->bool:
         """queryClose, queryExit and aboutToQuit are no
         ideal match for the async Deferred approach.
 
@@ -377,8 +389,7 @@ class MainWindow(KXmlGuiWindow):
         and we really end the program.
         """
 
-        # pylint: disable=too-many-branches
-        def confirmed(result):
+        def confirmed(result:Any) ->None:
             """quit if the active game has been aborted"""
             self.exitConfirmed = bool(result)
             if Debug.quit:
@@ -394,10 +405,10 @@ class MainWindow(KXmlGuiWindow):
             else:
                 self.exitConfirmed = None
 
-        def cancelled(result):
+        def cancelled(result:Any) ->None:
             """just do nothing"""
             if Debug.quit:
-                logDebug('mainWindow.queryClose.cancelled: {}'.format(result))
+                logDebug(f'mainWindow.queryClose.cancelled: {result}')
             self.exitConfirmed = None
         if self.exitConfirmed is False:
             # user is currently being asked
@@ -413,12 +424,12 @@ class MainWindow(KXmlGuiWindow):
                         'MainWindow.queryClose not asking, exitConfirmed=True')
         return True
 
-    def queryExit(self):
+    def queryExit(self) ->bool:
         """see queryClose"""
-        def quitDebug(*args, **kwargs):
+        def quitDebug(msg:str) ->None:
             """reducing branches in queryExit"""
             if Debug.quit:
-                logDebug(*args, **kwargs)
+                logDebug(msg)
 
         if self.exitReady:
             quitDebug('MainWindow.queryExit returns True because exitReady is set')
@@ -431,20 +442,20 @@ class MainWindow(KXmlGuiWindow):
                         self.confDialog,
                         self.rulesetWindow, self.playerWindow]):
                 if isAlive(widget):
+                    assert widget
                     widget.hide()
             if self.exitWaitTime is None:
                 self.exitWaitTime = 0
-            if Internal.reactor and Internal.reactor.running:
+            if hasattr(Internal, 'reactor') and Internal.reactor.running:
                 self.exitWaitTime += 10
                 if self.exitWaitTime % 1000 == 0:
                     logDebug(
-                        'waiting since %d seconds for reactor to stop' %
-                        (self.exitWaitTime // 1000))
+                        f'waiting since {int(self.exitWaitTime // 1000)} seconds for reactor to stop')
                 try:
                     quitDebug('now stopping reactor')
-                    Internal.reactor.stop()
+                    Internal.reactor.stop()  # type:ignore[misc]
                     assert isAlive(self)
-                    QTimer.singleShot(10, self.close)
+                    QTimer.singleShot(10, self.closeMe)
                 except ReactorNotRunning:
                     self.exitReady = True
                     quitDebug(
@@ -455,17 +466,17 @@ class MainWindow(KXmlGuiWindow):
         return bool(self.exitReady)
 
     @staticmethod
-    def aboutToQuit():
+    def aboutToQuit() ->None:
         """now all connections to servers are cleanly closed"""
         mainWindow = Internal.mainWindow
         Internal.mainWindow = None
         if mainWindow:
             if Debug.quit:
                 logDebug('aboutToQuit starting')
-            if mainWindow.exitWaitTime is not None and mainWindow.exitWaitTime > 1000.0 or Debug.quit:
-                logDebug(
-                    'reactor stopped after %d ms' %
-                    (mainWindow.exitWaitTime))
+            if mainWindow.exitWaitTime is not None:
+                if mainWindow.exitWaitTime > 1000.0 or Debug.quit:
+                    logDebug(
+                        f'reactor stopped after {int(mainWindow.exitWaitTime)} ms')
         else:
             if Debug.quit:
                 logDebug('aboutToQuit: mainWindow is already None')
@@ -486,13 +497,14 @@ class MainWindow(KXmlGuiWindow):
         if Debug.quit:
             logDebug('aboutToQuit ending')
 
-    def abortAction(self):
+    def abortAction(self) ->'Deferred':
         """abort current game"""
         if Debug.quit:
             logDebug('mainWindow.abortAction invoked')
+        assert self.scene
         return self.scene.abort()
 
-    def retranslateUi(self):
+    def retranslateUi(self) ->None:
         """retranslate"""
         self.actionScoreGame.setText(
             i18nc('@action:inmenu', "&Score Manual Game"))
@@ -503,18 +515,18 @@ class MainWindow(KXmlGuiWindow):
                   '&Score a manual game.'))
 
         self.actionPlayGame.setText(i18nc('@action:intoolbar', "&Play"))
-        self.actionPlayGame.setPriority(QAction.LowPriority)
+        self.actionPlayGame.setPriority(QAction.Priority.LowPriority)
         self.actionPlayGame.setWhatsThis(
             i18nc('kajongg @info:tooltip', 'Start a new game.'))
 
         self.actionAbortGame.setText(i18nc('@action:inmenu', "&Abort Game"))
-        self.actionAbortGame.setPriority(QAction.LowPriority)
+        self.actionAbortGame.setPriority(QAction.Priority.LowPriority)
         self.actionAbortGame.setWhatsThis(
             i18nc('kajongg @info:tooltip',
                   'Abort the current game.'))
 
         self.actionQuit.setText(i18nc('@action:inmenu', "&Quit Kajongg"))
-        self.actionQuit.setPriority(QAction.LowPriority)
+        self.actionQuit.setPriority(QAction.Priority.LowPriority)
 
         self.actionPlayers.setText(i18nc('@action:intoolbar', "&Players"))
         self.actionPlayers.setWhatsThis(
@@ -551,7 +563,7 @@ class MainWindow(KXmlGuiWindow):
                                               'Explain the scoring for all players in the current game.'))
 
         self.actionAutoPlay.setText(i18nc('@action:inmenu', "&Demo Mode"))
-        self.actionAutoPlay.setPriority(QAction.LowPriority)
+        self.actionAutoPlay.setPriority(QAction.Priority.LowPriority)
         self.actionAutoPlay.setWhatsThis(i18nc('kajongg @info:tooltip',
                                                'Let the computer take over for you. Start a new local game if needed.'))
 
@@ -560,36 +572,36 @@ class MainWindow(KXmlGuiWindow):
             i18nc('kajongg @info:tooltip',
                   'Chat with the other players.'))
 
-    def changeEvent(self, event):
+    def changeEvent(self, event:QEvent) ->None:
         """when the applicationwide language changes, recreate GUI"""
-        if event.type() == QEvent.LanguageChange:
+        if event.type() == QEvent.Type.LanguageChange:
             self.setupGUI()
             self.retranslateUi()
 
-    def slotPlayers(self):
+    def slotPlayers(self) ->None:
         """show the player list"""
         if not self.playerWindow:
             self.playerWindow = PlayerList(self)
         self.playerWindow.show()
 
-    def slotRulesets(self):
+    def slotRulesets(self) ->None:
         """show the player list"""
         if not self.rulesetWindow:
             self.rulesetWindow = RulesetSelector()
         self.rulesetWindow.show()
 
-    def adjustMainView(self):
+    def adjustMainView(self) ->None:
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
-        if not Internal.scaleScene or not isAlive(self):
+        if not isAlive(self):
             return
         view, scene = self.centralView, self.scene
         if scene:
             scene.adjustSceneView()
-            view.fitInView(scene.itemsBoundingRect(), Qt.KeepAspectRatio)
+            view.fitInView(scene.itemsBoundingRect(), Qt.AspectRatioMode.KeepAspectRatio)
 
-    @afterQueuedAnimations
-    def backgroundChanged(self, unusedDeferredResult, unusedOldName, newName):
+    @afterQueuedAnimations  # type:ignore[arg-type]
+    def backgroundChanged(self, unusedDeferredResult:'Deferred', unusedOldName:str, newName:str) ->None:
         """if the wanted background changed, apply the change now"""
         centralWidget = self.centralWidget()
         if centralWidget:
@@ -597,10 +609,10 @@ class MainWindow(KXmlGuiWindow):
             self.background.setPalette(centralWidget)
             centralWidget.setAutoFillBackground(True)
 
-    @afterQueuedAnimations
+    @afterQueuedAnimations  # type:ignore[arg-type]
     def tilesetNameChanged(
-            self, unusedDeferredResult, unusedOldValue=None,
-            unusedNewValue=None):
+            self, unusedDeferredResult:'Deferred', unusedOldValue:Optional[str]=None,
+            unusedNewValue:Optional[str]=None) ->None:
         """if the wanted tileset changed, apply the change now"""
         if self.centralView:
             with AnimationSpeed():
@@ -608,8 +620,8 @@ class MainWindow(KXmlGuiWindow):
                     self.scene.applySettings()
             self.adjustMainView()
 
-    @afterQueuedAnimations
-    def showSettings(self, unusedDeferredResult, unusedChecked=None):
+    @afterQueuedAnimations  # type:ignore[arg-type]
+    def showSettings(self, unusedDeferredResult:'Request', unusedChecked:Optional[bool]=None) ->None:
         """show preferences dialog. If it already is visible, do nothing"""
         # This is called by the triggered() signal. So why does KDE
         # not return the bool checked?
@@ -621,10 +633,11 @@ class MainWindow(KXmlGuiWindow):
         self.confDialog = ConfigDialog(self, "settings")
         self.confDialog.show()
 
-    def _toggleWidget(self, checked):
+    def _toggleWidget(self, checked:bool) ->None:
         """user has toggled widget visibility with an action"""
         assert self.scene
         action = self.sender()
+        assert isinstance(action, Action), action
         actionData = action.data()
         if checked:
             if isinstance(actionData, type):
@@ -641,16 +654,17 @@ class MainWindow(KXmlGuiWindow):
             assert actionData
             actionData.hide()
 
-    def _toggleDemoMode(self, checked):
+    def _toggleDemoMode(self, checked:bool) ->None:
         """switch on / off for autoPlay"""
         if self.scene:
+            assert isinstance(self.scene, PlayingScene)
             self.scene.toggleDemoMode(checked)
         else:
             Internal.autoPlay = checked
             if checked and Internal.db:
-                self.playingScene()
+                self.startPlayingGame()
 
-    def updateGUI(self):
+    def updateGUI(self) ->None:
         """update some actions, all auxiliary windows and the statusbar"""
         if not isAlive(self):
             return
@@ -660,10 +674,11 @@ class MainWindow(KXmlGuiWindow):
         self.actionAbortGame.setEnabled(bool(self.scene))
         scene = self.scene
         if isAlive(scene):
+            assert scene
             scene.updateSceneGUI()
 
-    @afterQueuedAnimations
-    def changeAngle(self, deferredResult, unusedButtons=None, unusedModifiers=None): # pylint: disable=unused-argument
+    @afterQueuedAnimations  # type:ignore[arg-type]
+    def changeAngle(self, deferredResult:'Request', unusedButtons:Any=None, unusedModifiers:Any=None) ->None: # pylint: disable=unused-argument
         """change the lightSource"""
         if self.scene:
             with AnimationSpeed():
