@@ -26,17 +26,19 @@ There are several variations of python ports for this script floating
 around, it seemed easier to me to maintain one for myself
 """
 
+from typing import List, Dict, Any, Type, cast
+
 try:
     from PyQt5 import sip
 except ImportError:
-    import sip
+    import sip  # type:ignore[no-redef,import]
 from qt import QObject, Qt, QAbstractItemModel, QModelIndex
 from qt import QPersistentModelIndex, QFont, QColor, QSize
 from common import isAlive
 
 # pylint: skip-file
 
-def isValid(variant):
+def isValid(variant:Any) ->bool:
     return (variant is not None)
 
 
@@ -44,16 +46,16 @@ class ModelTest(QObject):
 
     """tests a model"""
 
-    def __init__(self, _model, parent):
+    def __init__(self, _model:QAbstractItemModel, parent:QObject) ->None:
         """
         Connect to all of the models signals, Whenever anything happens recheck everything.
         """
         QObject.__init__(self, parent)
         self._model = _model
-        self.model = sip.cast(_model, QAbstractItemModel)
-        self.insert = []
-        self.remove = []
-        self.changing = []
+        self.model = cast(QAbstractItemModel, sip.cast(_model, QAbstractItemModel))  # type:ignore[arg-type]
+        self.insert:List[Dict] = []
+        self.remove:List[Dict] = []
+        self.changing:List[QPersistentModelIndex] = []
         self.fetchingMore = False
         assert(self.model)
 
@@ -80,7 +82,7 @@ class ModelTest(QObject):
         self.model.rowsRemoved.connect(self.rowsRemoved)
         self.runAllTests()
 
-    def nonDestructiveBasicTest(self):
+    def nonDestructiveBasicTest(self) ->None:
         """
         nonDestructiveBasicTest tries to call a number of the basic functions (not all)
         to make sure the model doesn't outright segfault, testing the functions that makes sense.
@@ -88,16 +90,16 @@ class ModelTest(QObject):
         assert(self.model.buddy(QModelIndex()) == QModelIndex())
         self.model.canFetchMore(QModelIndex())
         assert(self.model.columnCount(QModelIndex()) >= 0)
-        assert(self.model.data(QModelIndex(), Qt.DisplayRole) is None)
+        assert(self.model.data(QModelIndex(), Qt.ItemDataRole.DisplayRole) is None)
         self.fetchingMore = True
         self.model.fetchMore(QModelIndex())
         self.fetchingMore = False
         flags = self.model.flags(QModelIndex())
-        assert(int(flags & Qt.ItemIsEnabled) == Qt.ItemIsEnabled or
-               int(flags & Qt.ItemIsEnabled) == 0)
+        assert(int(flags & Qt.ItemFlag.ItemIsEnabled) == Qt.ItemFlag.ItemIsEnabled or
+               int(flags & Qt.ItemFlag.ItemIsEnabled) == 0)
         self.model.hasChildren(QModelIndex())
         self.model.hasIndex(0, 0)
-        self.model.headerData(0, Qt.Horizontal, Qt.DisplayRole)
+        self.model.headerData(0, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole)
         self.model.index(0, 0, QModelIndex())
         self.model.itemData(QModelIndex())
         cache = None
@@ -107,14 +109,14 @@ class ModelTest(QObject):
         assert(self.model.rowCount(QModelIndex()) >= 0)
         variant = None
         self.model.setData(QModelIndex(), variant, -1)
-        self.model.setHeaderData(-1, Qt.Horizontal, None)
-        self.model.setHeaderData(0, Qt.Horizontal, None)
-        self.model.setHeaderData(999999, Qt.Horizontal, None)
+        self.model.setHeaderData(-1, Qt.Orientation.Horizontal, None)
+        self.model.setHeaderData(0, Qt.Orientation.Horizontal, None)
+        self.model.setHeaderData(999999, Qt.Orientation.Horizontal, None)
         self.model.sibling(0, 0, QModelIndex())
         self.model.span(QModelIndex())
         self.model.supportedDropActions()
 
-    def rowCount(self):
+    def rowCount(self) ->None:
         """
         Tests self.model's implementation of QAbstractItemModel::rowCount() and hasChildren()
 
@@ -138,7 +140,7 @@ class ModelTest(QObject):
         # The self.models rowCount() is tested more extensively in checkChildren,
         # but this catches the big mistakes
 
-    def columnCount(self):
+    def columnCount(self) ->None:
         """
         Tests self.model's implementation of QAbstractItemModel::columnCount() and hasChildren()
         """
@@ -154,7 +156,7 @@ class ModelTest(QObject):
         # columnCount() is tested more extensively in checkChildren,
         # but this catches the big mistakes
 
-    def hasIndex(self):
+    def hasIndex(self) ->None:
         """
         Tests self.model's implementation of QAbstractItemModel::hasIndex()
         """
@@ -176,7 +178,7 @@ class ModelTest(QObject):
         # hasIndex() is tested more extensively in checkChildren()
         # but this catches the big mistakes
 
-    def index(self):
+    def index(self) ->None:
         """
         Tests self.model's implementation of QAbstractItemModel::index()
         """
@@ -203,7 +205,7 @@ class ModelTest(QObject):
         # index() is tested more extensively in checkChildren()
         # but this catches the big mistakes
 
-    def parent(self):
+    def test_parent(self) ->None:
         """
         Tests self.model's implementation of QAbstractItemModel::parent()
         """
@@ -243,15 +245,14 @@ class ModelTest(QObject):
         # parent's children correctly specify their parent
         self.checkChildren(QModelIndex())
 
-    def testRoleDataType(self, role, expectedType):
+    def testRoleDataType(self, role:Qt.ItemDataRole, expectedType:Type[Any]) ->None:
         """Tests implementation if model.data() for role"""
         model = self.model
         result = model.data(model.index(0, 0, QModelIndex()), role)
         if result is not None and not isinstance(result, expectedType):
-            raise Exception('returned data type is {}, should be {}'.format(
-                type(result), expectedType))
+            raise Exception(f'returned data type is {type(result)}, should be {expectedType}')
 
-    def testRoleDataValues(self, role, asserter):
+    def testRoleDataValues(self, role:Qt.ItemDataRole, asserter:Any) ->None:
         """Tests implementation if model.data() for role.
         asserter is a function returning True or False"""
         model = self.model
@@ -259,12 +260,12 @@ class ModelTest(QObject):
         if result is not None and not asserter(result):
             raise Exception('returned value {} is wrong')
 
-    def data(self):
+    def data(self) ->None:
         """
         Tests self.model's implementation of QAbstractItemModel::data()
         """
         # Invalid index should return an invalid qvariant
-        assert not isValid(self.model.data(QModelIndex(), Qt.DisplayRole))
+        assert not isValid(self.model.data(QModelIndex(), Qt.ItemDataRole.DisplayRole))
 
         if self.model.rowCount(QModelIndex()) == 0:
             return
@@ -276,30 +277,30 @@ class ModelTest(QObject):
         assert(
             self.model.setData(QModelIndex(),
                                "foo",
-                               Qt.DisplayRole) == False)
+                               Qt.ItemDataRole.DisplayRole) == False)
 
         # General Purpose roles that should return a QString
-        self.testRoleDataType(Qt.ToolTipRole, str)
-        self.testRoleDataType(Qt.StatusTipRole, str)
-        self.testRoleDataType(Qt.WhatsThisRole, str)
-        self.testRoleDataType(Qt.SizeHintRole, QSize)
-        self.testRoleDataType(Qt.FontRole, QFont)
-        self.testRoleDataType(Qt.ForegroundRole, QColor)
-        self.testRoleDataType(Qt.BackgroundColorRole, QColor)
-        self.testRoleDataType(Qt.TextColorRole, QColor)
+        self.testRoleDataType(Qt.ItemDataRole.ToolTipRole, str)
+        self.testRoleDataType(Qt.ItemDataRole.StatusTipRole, str)
+        self.testRoleDataType(Qt.ItemDataRole.WhatsThisRole, str)
+        self.testRoleDataType(Qt.ItemDataRole.SizeHintRole, QSize)
+        self.testRoleDataType(Qt.ItemDataRole.FontRole, QFont)
+        self.testRoleDataType(Qt.ItemDataRole.ForegroundRole, QColor)
+        self.testRoleDataType(Qt.ItemDataRole.BackgroundColorRole, QColor)
+        self.testRoleDataType(Qt.ItemDataRole.TextColorRole, QColor)
 
         # Check that the alignment is one we know about
         self.testRoleDataValues(
-            Qt.TextAlignmentRole,
-            lambda x: x == (x & int(Qt.AlignHorizontal_Mask | Qt.AlignVertical_Mask)))
+            Qt.ItemDataRole.TextAlignmentRole,
+            lambda x: x == (x & int(Qt.AlignmentFlag.AlignHorizontal_Mask | Qt.AlignmentFlag.AlignVertical_Mask)))
 
         # General Purpose roles that should return a QColor
         # Check that the "check state" is one we know about.
         self.testRoleDataValues(
-            Qt.CheckStateRole,
-            lambda x: x in (Qt.Unchecked, Qt.PartiallyChecked, Qt.Checked))
+            Qt.ItemDataRole.CheckStateRole,
+            lambda x: x in (Qt.CheckState.Unchecked, Qt.CheckState.PartiallyChecked, Qt.CheckState.Checked))
 
-    def runAllTests(self):
+    def runAllTests(self) ->None:
         """run all tests after the model changed"""
         if not isAlive(self):
             return
@@ -310,21 +311,21 @@ class ModelTest(QObject):
         self.columnCount()
         self.hasIndex()
         self.index()
-        self.parent()
+        self.test_parent()
         self.data()
 
-    def rowsAboutToBeInserted(self, parent, start, dummyEnd):
+    def rowsAboutToBeInserted(self, parent:QModelIndex, start:int, unusedEnd:int) ->None:
         """
         Store what is about to be inserted to make sure it actually happens
         """
-        item = {}
+        item:Dict[str, Any] = {}
         item['parent'] = parent
         item['oldSize'] = self.model.rowCount(parent)
         item['last'] = self.model.data(self.model.index(start - 1, 0, parent))
         item['next'] = self.model.data(self.model.index(start, 0, parent))
         self.insert.append(item)
 
-    def rowsInserted(self, parent, start, end):
+    def rowsInserted(self, parent:QModelIndex, start:int, end:int) ->None:
         """
         Confirm that what was said was going to happen actually did
         """
@@ -345,18 +346,18 @@ class ModelTest(QObject):
         assert(item['next'] == self.model.data(
             self.model.index(end + 1, 0, item['parent'])))
 
-    def rowsAboutToBeRemoved(self, parent, start, end):
+    def rowsAboutToBeRemoved(self, parent:QModelIndex, start:int, end:int) ->None:
         """
         Store what is about to be inserted to make sure it actually happens
         """
-        item = {}
+        item:Dict[str, Any] = {}
         item['parent'] = parent
         item['oldSize'] = self.model.rowCount(parent)
         item['last'] = self.model.data(self.model.index(start - 1, 0, parent))
         item['next'] = self.model.data(self.model.index(end + 1, 0, parent))
         self.remove.append(item)
 
-    def rowsRemoved(self, parent, start, end):
+    def rowsRemoved(self, parent:QModelIndex, start:int, end:int) ->None:
         """
         Confirm that what was said was going to happen actually did
         """
@@ -369,14 +370,14 @@ class ModelTest(QObject):
         assert(item['next'] == self.model.data(
             self.model.index(start, 0, item['parent'])))
 
-    def layoutAboutToBeChanged(self):
+    def layoutAboutToBeChanged(self) ->None:
         """
         Store what is about to be changed
         """
         for i in range(0, max(0, min(self.model.rowCount(), 100))):
             self.changing.append(QPersistentModelIndex(self.model.index(i, 0)))
 
-    def layoutChanged(self):
+    def layoutChanged(self) ->None:
         """
         Confirm that what was said was going to happen actually did
         """
@@ -387,7 +388,7 @@ class ModelTest(QObject):
                                            change.parent()))
         self.changing = []
 
-    def checkChildren(self, parent, depth=0):
+    def checkChildren(self, parent:QModelIndex, depth:int=0) ->None:
         """
         Called from parent() test.
 
@@ -464,7 +465,7 @@ class ModelTest(QObject):
                 # While you can technically return a QVariant usually this is a sign
                 # if an bug in data() Disable if this really is ok in your
                 # self.model
-                assert isValid(self.model.data(index, Qt.DisplayRole))
+                assert isValid(self.model.data(index, Qt.ItemDataRole.DisplayRole))
 
                 # if the next test fails here is some somewhat useful debug you play with
                 # if self.model.parent(index) != parent:
