@@ -7,27 +7,32 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
+from typing import Optional, TYPE_CHECKING, Dict
+import sqlite3
+
 from mi18n import i18n, i18nc
 from kde import KIcon
 from dialogs import Sorry, QuestionYesNo
 from qt import Qt
 from qt import QDialog, QHBoxLayout, QVBoxLayout, QDialogButtonBox, \
-    QTableWidget, QTableWidgetItem
+    QTableWidget, QTableWidgetItem, QWidget
 
 from common import Internal
 from query import Query
 from guiutil import decorateWindow
 from statesaver import StateSaver
 
+if TYPE_CHECKING:
+    from qt import QEvent, QKeyEvent
+
 
 class PlayerList(QDialog):
 
     """QtSQL Model view of the players"""
 
-    def __init__(self, parent):
-        QDialog.__init__(self)
-        self.parent = parent
-        self._data = {}
+    def __init__(self, parent:Optional['QWidget']=None):
+        QDialog.__init__(self, parent)
+        self._data:Dict[str, int] = {}
         self.table = QTableWidget(self)
         self.table.horizontalHeader().setStretchLastSection(True)
         self.table.verticalHeader().setVisible(False)
@@ -58,16 +63,16 @@ class PlayerList(QDialog):
         decorateWindow(self, i18n("Players"))
         self.setObjectName('Players')
 
-    def showEvent(self, unusedEvent):
+    def showEvent(self, unusedEvent:'QEvent') ->None:
         """adapt view to content"""
         StateSaver(self, self.table)
 
     @staticmethod
-    def sortKey(text):
+    def sortKey(text:str) ->str:
         """display order in Table"""
         return text.upper() if text else 'zzzzzzzzzzzz'
 
-    def updateTable(self, data=None, currentName=None):
+    def updateTable(self, data:Optional[Dict[str, int]]=None, currentName:Optional[str]=None) ->None:
         """fills self.table from DB"""
         self.table.itemChanged.disconnect(self.itemChanged)
         table = self.table
@@ -94,7 +99,7 @@ class PlayerList(QDialog):
             table.scrollToItem(selectedItem)
         self.table.itemChanged.connect(self.itemChanged)
 
-    def itemChanged(self, item):
+    def itemChanged(self, item:QTableWidgetItem) ->None:
         """this must be new because editing is disabled for others"""
         currentName = item.text()
         if currentName in self._data:
@@ -103,16 +108,17 @@ class PlayerList(QDialog):
             del self._data[self.table.item(self.table.currentRow(), 0).text()]
             self.updateTable(currentName=currentName)
             return
-        query = Query('insert into player(name) values(?)', (currentName, ))
-        if query.failure:
+        try:
+            Query('insert into player(name) values(?)', (currentName, ))
+        except sqlite3.Error as exc:
             Sorry(
                 i18n(
                     'Error while adding player %1: %2',
                     currentName,
-                    query.failure.message))
+                    str(exc)))
         self.updateTable(currentName=currentName)
 
-    def slotInsert(self):
+    def slotInsert(self) ->None:
         """insert a record"""
         self._data[''] = 0
         self.updateTable(data=self._data, currentName='')
@@ -122,21 +128,21 @@ class PlayerList(QDialog):
                 self.table.editItem(item)
 
     @staticmethod
-    def __deletePlayer(playerId):
+    def __deletePlayer(playerId:int) ->None:
         """delete this player and all associated games"""
         with Internal.db: # transaction
             Query("delete from score where player=?", (playerId, ))
             Query("delete from game where p0=? or p1=? or p2=? or p3=?", (playerId, ) * 4)
             Query("delete from player where id=?", (playerId,))
 
-    def delete(self):
+    def delete(self) ->None:
         """delete selected entry"""
-        def answered(result):
+        def answered(result:bool) ->None:
             """coming from QuestionYesNo"""
             if result is True:
                 self.__deletePlayer(playerId)
             cleanup()
-        def cleanup():
+        def cleanup() ->None:
             """update table view"""
             self.updateTable()
             self.table.setCurrentCell(min(currentRow, len(self._data) - 1), 0)
@@ -166,12 +172,12 @@ class PlayerList(QDialog):
                 '  This will also delete all games played by %3!',
                 finishedCount, fullCount - finishedCount, name)).addBoth(answered)
 
-    def keyPressEvent(self, event):
+    def keyPressEvent(self, event:'QKeyEvent') ->None:
         """use insert/delete keys for insert/delete"""
         key = event.key()
-        if key == Qt.Key_Insert:
+        if key == Qt.Key.Key_Insert:
             self.slotInsert()
-        elif key == Qt.Key_Delete:
+        elif key == Qt.Key.Key_Delete:
             self.delete()
         else:
             QDialog.keyPressEvent(self, event)
