@@ -9,17 +9,19 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
+from typing import TYPE_CHECKING, Tuple, List, Optional, Any, Union, Iterable, cast
+
 from qt import Qt, QSize
 from qt import QWidget, QHBoxLayout, QVBoxLayout, \
     QPushButton, QSpacerItem, QSizePolicy, \
     QTreeView, QFont, QAbstractItemView, QHeaderView
 from qt import QModelIndex
 from kdestub import KApplication
-from rule import Ruleset, PredefinedRuleset, RuleBase, ParameterRule, BoolRule
+from rule import Rule, Ruleset, PredefinedRuleset, ParameterRule, BoolRule
 from util import uniqueList
 from mi18n import i18n, i18nc, i18ncE, english
 from differ import RulesetDiffer
-from common import Debug, Internal
+from common import Debug
 from tree import TreeItem, RootItem, TreeModel
 from dialogs import Sorry
 from modeltest import ModelTest
@@ -27,13 +29,17 @@ from genericdelegates import RightAlignedCheckboxDelegate, ZeroEmptyColumnDelega
 from statesaver import StateSaver
 from guiutil import decorateWindow
 
+if TYPE_CHECKING:
+    from qt import QShowEvent, QHideEvent, QItemSelection, QObject, QLayout
+    from rule import RuleList
+
 
 class RuleRootItem(RootItem):
 
     """the root item for the ruleset tree"""
 
-    def columnCount(self):
-        return len(self.rawContent)
+    def columnCount(self) ->int:
+        return len(self.raw)
 
 
 class RuleTreeItem(TreeItem):
@@ -42,93 +48,106 @@ class RuleTreeItem(TreeItem):
     # pylint: disable=abstract-method
     # we know content() is abstract, this class is too
 
-    def columnCount(self):
+    def columnCount(self) ->int:
         """can be different for every rule"""
         if hasattr(self, 'colCount'):
-            return self.colCount  # pylint: disable=no-member
-        return len(self.rawContent)
+            return self.colCount
+        return len(self.raw)
 
-    def ruleset(self):
+    def ruleset(self) ->Ruleset:
         """return the ruleset containing this item"""
-        item = self
-        while not isinstance(item.rawContent, Ruleset):
+        item:TreeItem = self
+        while not isinstance(item.raw, Ruleset):
+            assert item.parent
             item = item.parent
-        return item.rawContent
+        return item.raw
 
 
 class RulesetItem(RuleTreeItem):
 
     """represents a ruleset in the tree"""
 
-    def __init__(self, content):
+    def __init__(self, content:Ruleset) ->None:
         RuleTreeItem.__init__(self, content)
 
-    def content(self, column):
+    @property
+    def raw(self) ->Ruleset:
+        return cast(Ruleset, super().raw)
+
+    def content(self, column:int) ->str:
         """return content stored in this item"""
         if column == 0:
-            return self.rawContent.name
+            return self.raw.name
         return ''
 
-    def columnCount(self):
+    def columnCount(self) ->int:
         return 1
 
-    def remove(self):
+    def remove(self) ->None:
         """remove this ruleset from the model and the database"""
-        self.rawContent.remove()
+        self.raw.remove()
 
-    def tooltip(self):
+    def tooltip(self) ->str:
         """the tooltip for a ruleset"""
-        return self.rawContent.description
+        return self.raw.description
 
 
 class RuleListItem(RuleTreeItem):
 
     """represents a list of rules in the tree"""
 
-    def __init__(self, content):
+    def __init__(self, content:List[ParameterRule]) ->None:
         RuleTreeItem.__init__(self, content)
 
-    def content(self, column):
+    @property
+    def raw(self) ->'RuleList':
+        return cast('RuleList', super().raw)
+
+    def content(self, column:int) ->str:
         """return content stored in this item"""
         if column == 0:
-            return self.rawContent.name
+            return self.raw.name
         return ''
 
-    def tooltip(self):
+    def tooltip(self) ->str:
         """tooltip for a list item explaining the usage of this list"""
         ruleset = self.ruleset()
         return '<b>' + i18n(ruleset.name) + '</b><br><br>' + \
-            i18n(self.rawContent.description)
+            i18n(self.raw.description)
 
 
 class RuleItem(RuleTreeItem):
 
     """represents a rule in the tree"""
 
-    def __init__(self, content):
+    def __init__(self, content:ParameterRule) ->None:
         RuleTreeItem.__init__(self, content)
 
-    def content(self, column):
+    @property
+    def raw(self) ->Rule:
+        return cast(Rule, super().raw)
+
+    def content(self, column:int) ->str:
         """return the content stored in this node"""
-        colNames = self.parent.parent.parent.rawContent
-        content = self.rawContent
+        colNames = self.parent.parent.parent.raw  # type:ignore
         if column == 0:
-            return content.name
-        if isinstance(content, ParameterRule):
+            return self.raw.name
+        if isinstance(self.raw, ParameterRule):
             if column == 1:
-                return content.parameter
+                return self.raw.parameter
         else:
-            if not hasattr(content.score, str(column)):
-                column = colNames[column]
-            return getattr(content.score, column)
+            _ = str(column)
+            if not hasattr(self.raw.score, _):
+                _ = colNames[column]
+            return getattr(self.raw.score, _)
         return ''
 
-    def tooltip(self):
+    def tooltip(self) ->str:
         """tooltip for rule: just the name of the ruleset"""
         ruleset = self.ruleset()
-        if self.rawContent.description:
+        if self.raw.description:
             return '<b>' + i18n(ruleset.name) + '</b><br><br>' + \
-                i18n(self.rawContent.description)
+                i18n(self.raw.description)
         return i18n(ruleset.name)
 
 
@@ -136,42 +155,43 @@ class RuleModel(TreeModel):
 
     """a model for our rule table"""
 
-    def __init__(self, rulesets, title, parent=None):
+    def __init__(self, rulesets:List[Ruleset], title:str, parent:Optional['QObject']=None) ->None:
         super().__init__(parent)
         self.rulesets = rulesets
         self.loaded = False
-        unitNames = list()
+        unitPairs:List[Tuple[str, int]] = []
+        # unitPairs: int is the priority:show 0 leftmost, 9999 rightmost
         for ruleset in rulesets:
             ruleset.load()
             for rule in ruleset.allRules:
-                unitNames.extend(rule.score.unitNames.items())
-        unitNames = sorted(unitNames, key=lambda x: x[1])
-        unitNames = uniqueList(x[0] for x in unitNames)
+                unitPairs.extend(rule.score.unitNames.items())
+        unitPairs = sorted(unitPairs, key=lambda x: x[1])
+        unitNames = uniqueList(x[0] for x in unitPairs)
         rootData = [title]
         rootData.extend(unitNames)
         self.rootItem = RuleRootItem(rootData)
 
-    def canFetchMore(self, unusedParent=None):
+    def canFetchMore(self, unusedParent:Optional['TreeItem']=None) ->bool:
         """did we already load the rules? We only want to do that
         when the config tab with rulesets is actually shown"""
         return not self.loaded
 
-    def fetchMore(self, unusedParent=None):
+    def fetchMore(self, unusedParent:Optional['TreeItem']=None) ->None:
         """load the rules"""
         for ruleset in self.rulesets:
             self.appendRuleset(ruleset)
         self.loaded = True
 
-    def data(self, index, role):  # pylint: disable=no-self-use
+    def data(self, index:QModelIndex, role:int=Qt.ItemDataRole.DisplayRole) ->Any:
         """get data fom model"""
         # pylint: disable=too-many-branches
         # too many branches
         result = None
         if index.isValid():
             item = index.internalPointer()
-            if role in (Qt.DisplayRole, Qt.EditRole):
+            if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
                 if index.column() == 1:
-                    if isinstance(item, RuleItem) and isinstance(item.rawContent, BoolRule):
+                    if isinstance(item, RuleItem) and isinstance(item.raw, BoolRule):
                         return ''
                 showValue = item.content(index.column())
                 if isinstance(showValue, str) and showValue.endswith('.0'):
@@ -182,65 +202,66 @@ class RuleModel(TreeModel):
                 if showValue == '0':
                     showValue = ''
                 result = showValue
-            elif role == Qt.CheckStateRole:
+            elif role == Qt.ItemDataRole.CheckStateRole:
                 if self.isCheckboxCell(index):
                     bData = item.content(index.column())
-                    result = Qt.Checked if bData else Qt.Unchecked
-            elif role == Qt.TextAlignmentRole:
-                result = int(Qt.AlignLeft | Qt.AlignVCenter)
+                    result = Qt.CheckState.Checked if bData else Qt.CheckState.Unchecked
+            elif role == Qt.ItemDataRole.TextAlignmentRole:
+                result = int(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
                 if index.column() > 0:
-                    result = int(Qt.AlignRight | Qt.AlignVCenter)
-            elif role == Qt.FontRole and index.column() == 0:
+                    result = int(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
+            elif role == Qt.ItemDataRole.FontRole and index.column() == 0:
                 ruleset = item.ruleset()
                 if isinstance(ruleset, PredefinedRuleset):
                     font = QFont()
                     font.setItalic(True)
                     result = font
-            elif role == Qt.ToolTipRole:
-                tip = '<b></b>%s<b></b>' % i18n(
-                    item.tooltip()) if item else ''
+            elif role == Qt.ItemDataRole.ToolTipRole:
+                tip = f'<b></b>{i18n(item.tooltip())}<b></b>' if item else ''
                 result = tip
         return result
 
     @staticmethod
-    def isCheckboxCell(index):
+    def isCheckboxCell(index:QModelIndex) ->bool:
         """are we dealing with a checkbox?"""
         if index.column() != 1:
             return False
         item = index.internalPointer()
-        return isinstance(item, RuleItem) and isinstance(item.rawContent, BoolRule)
+        return isinstance(item, RuleItem) and isinstance(item.raw, BoolRule)
 
-    def headerData(self, section, orientation, role):
+    def headerData(self, section:int, orientation:int, role:int=Qt.ItemDataRole.DisplayRole) ->Optional[Any]:
         """tell the view about the wanted headers"""
         if Qt is None:
             # happens when kajongg exits unexpectedly
             return None
-        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
+        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
+            assert self.rootItem
             if section >= self.rootItem.columnCount():
                 return None
             result = self.rootItem.content(section)
             if result == 'doubles':
                 return 'x2'
             return i18nc('kajongg', result)
-        if role == Qt.TextAlignmentRole:
-            leftRight = Qt.AlignLeft if section == 0 else Qt.AlignRight
-            return int(leftRight | Qt.AlignVCenter)
+        if role == Qt.ItemDataRole.TextAlignmentRole:
+            leftRight = Qt.AlignmentFlag.AlignLeft if section == 0 else Qt.AlignmentFlag.AlignRight
+            return int(leftRight | Qt.AlignmentFlag.AlignVCenter)
         return None
 
-    def appendRuleset(self, ruleset):
+    def appendRuleset(self, ruleset:Ruleset) ->None:
         """append ruleset to the model"""
         if not ruleset:
             return
         ruleset.load()
         parent = QModelIndex()
+        assert self.rootItem
         row = self.rootItem.childCount()
         rulesetItems = list([RulesetItem(ruleset)])
         self.insertRows(row, rulesetItems, parent)
         rulesetIndex = self.index(row, 0, parent)
-        ruleLists = [x for x in ruleset.ruleLists if len(x)]
-        ruleListItems = [RuleListItem(x) for x in ruleLists]
+        ruleLists:List[List[ParameterRule]] = [x for x in ruleset.ruleLists if len(x)]
+        ruleListItems:List[RuleListItem] = [RuleListItem(x) for x in ruleLists]
         for item in ruleListItems:
-            item.colCount = self.rootItem.columnCount()
+            item.colCount = self.rootItem.columnCount()  # type:ignore
         self.insertRows(0, ruleListItems, rulesetIndex)
         for ridx, ruleList in enumerate(ruleLists):
             listIndex = self.index(ridx, 0, rulesetIndex)
@@ -252,17 +273,14 @@ class EditableRuleModel(RuleModel):
 
     """add methods needed for editing"""
 
-    def __init__(self, rulesets, title, parent=None):
+    def __init__(self, rulesets:List[Ruleset], title:str, parent:Optional[QModelIndex]=None) ->None:
         RuleModel.__init__(self, rulesets, title, parent)
 
-    def __setRuleData(self, column, content, value):
+    def __setRuleData(self, column:int, content:Union[Rule, ParameterRule],
+        value:Union[str, int, bool]) ->Tuple[bool, Optional[str]]:
         """change rule data in the model"""
         dirty, message = False, None
-        if column == 0:
-            if content.name != english(value):
-                dirty = True
-                content.name = english(value)
-        elif column == 1 and isinstance(content, ParameterRule):
+        if column == 1 and isinstance(content, ParameterRule):
             oldParameter = content.parameter
             if isinstance(content, BoolRule):
                 return False, ''
@@ -274,11 +292,13 @@ class EditableRuleModel(RuleModel):
                 content.parameter = oldParameter
                 dirty = False
         else:
+            assert self.rootItem
             unitName = self.rootItem.content(column)
+            assert isinstance(value, (int, float))
             dirty, message = content.score.change(unitName, value)
         return dirty, message
 
-    def setData(self, index, value, role=Qt.EditRole):
+    def setData(self, index:QModelIndex, value:str, role:int=Qt.ItemDataRole.EditRole) ->bool:
         """change data in the model"""
         # pylint:  disable=too-many-branches
         if not index.isValid():
@@ -287,64 +307,66 @@ class EditableRuleModel(RuleModel):
         column = index.column()
         item = index.internalPointer()
         ruleset = item.ruleset()
-        content = item.rawContent
-        if role == Qt.EditRole:
+        content = item.raw
+        if role == Qt.ItemDataRole.EditRole:
             if isinstance(content, Ruleset) and column == 0:
                 oldName = content.name
                 content.rename(english(value))
                 dirty = oldName != content.name
-            elif isinstance(content, RuleBase):
+            elif isinstance(content, (Rule, ParameterRule)):
                 dirty, message = self.__setRuleData(column, content, value)
                 if message:
                     Sorry(message)
                     return False
             else:
                 return False
-        elif role == Qt.CheckStateRole:
+        elif role == Qt.ItemDataRole.CheckStateRole:
             if isinstance(content, BoolRule) and column == 1:
                 if not isinstance(ruleset, PredefinedRuleset):
-                    newValue = value == Qt.Checked
+                    newValue = value == Qt.CheckState.Checked
                     if content.parameter != newValue:
                         dirty = True
                         content.parameter = newValue
             else:
                 return False
         if dirty:
-            if isinstance(content, RuleBase):
+            if isinstance(content, (Rule, ParameterRule)):
                 ruleset.updateRule(content)
             self.dataChanged.emit(index, index)
         return True
 
-    def flags(self, index):  # pylint: disable=no-self-use
+    def flags(self, index:QModelIndex) ->Qt.ItemFlags:
         """tell the view what it can do with this item"""
         if not index.isValid():
-            return Qt.ItemIsEnabled
+            return cast(Qt.ItemFlags, Qt.ItemFlag.ItemIsEnabled)
         column = index.column()
         item = index.internalPointer()
-        content = item.rawContent
+        content = item.raw
         checkable = False
         if isinstance(content, Ruleset) and column == 0:
             mayEdit = True
-        elif isinstance(content, RuleBase):
+        elif isinstance(content, (Rule, ParameterRule)):
             checkable = column == 1 and isinstance(content, BoolRule)
             mayEdit = bool(column)
         else:
             mayEdit = False
         mayEdit = mayEdit and not isinstance(item.ruleset(), PredefinedRuleset)
-        result = Qt.ItemIsEnabled | Qt.ItemIsSelectable
+        result = Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsSelectable
         if mayEdit:
-            result |= Qt.ItemIsEditable
+            result |= Qt.ItemFlag.ItemIsEditable
         if checkable:
-            result |= Qt.ItemIsUserCheckable
-        return result
+            result |= Qt.ItemFlag.ItemIsUserCheckable
+        return cast(Qt.ItemFlags, result)
 
 
 class RuleTreeView(QTreeView):
 
     """Tree view for our rulesets"""
 
-    def __init__(self, name, btnCopy=None,
-                 btnRemove=None, btnCompare=None, parent=None):
+    def __init__(self, name:str,
+                 btnCopy:Optional[QPushButton]=None,
+                 btnRemove:Optional[QPushButton]=None,
+                 btnCompare:Optional[QPushButton]=None, parent:Optional[QWidget]=None) ->None:
         QTreeView.__init__(self, parent)
         self.name = name
         self.setObjectName('RuleTreeView')
@@ -356,43 +378,42 @@ class RuleTreeView(QTreeView):
                 button.setEnabled(False)
         self.header().setObjectName('RuleTreeViewHeader')
         self.setSelectionMode(QAbstractItemView.SingleSelection)
-        self.ruleModel = None
+        self.ruleModel:Optional[RuleModel] = None
         self.ruleModelTest = None
-        self.rulesets = []  # nasty: this generates self.ruleModel
-        self.differs = []
+        self.rulesets:List[Ruleset] = []  # nasty: this generates self.ruleModel
+        self.differs:List[RulesetDiffer] = []
 
-    def dataChanged(self, unusedIndex1, unusedIndex2, unusedRoles=None):
+    def dataChanged(self, unusedIndex1:int, unusedIndex2:int, unusedRoles:Optional[Iterable[int]]=None) ->None:
         """get called if the model has changed: Update all differs"""
         for differ in self.differs:
             differ.rulesetChanged()
 
     @property
-    def rulesets(self):
+    def rulesets(self) ->List[Ruleset]:
         """a list of rulesets made available by this model"""
+        assert self.ruleModel
         return self.ruleModel.rulesets
 
     @rulesets.setter
-    def rulesets(self, rulesets):
+    def rulesets(self, rulesets:List[Ruleset]) ->None:
         """a new list: update display"""
         if not self.ruleModel or self.ruleModel.rulesets != rulesets:
             if self.btnRemove and self.btnCopy:
                 self.ruleModel = EditableRuleModel(rulesets, self.name)
             else:
                 self.ruleModel = RuleModel(rulesets, self.name)
-            self.setItemDelegateForColumn(
-                1,
-                RightAlignedCheckboxDelegate(
-                    self,
-                    self.ruleModel.isCheckboxCell))
-            for  col in (1, 2, 3):
+            delegate = RightAlignedCheckboxDelegate(self, self.ruleModel.isCheckboxCell)
+            self.setItemDelegateForColumn(1, delegate)
+            for  col in (2, 3):
                 self.setItemDelegateForColumn(col, ZeroEmptyColumnDelegate(self))
             self.setModel(self.ruleModel)
             if Debug.modelTest:
                 self.ruleModelTest = ModelTest(self.ruleModel, self)
             self.show()
 
-    def selectionChanged(self, selected, unusedDeselected=None):
+    def selectionChanged(self, selected:'QItemSelection', unused_deselected:Optional['QItemSelection']=None) ->None:
         """update editing buttons"""
+        assert self.ruleModel
         enableCopy = enableRemove = enableCompare = False
         if selected.indexes():
             item = selected.indexes()[0].internalPointer()
@@ -410,9 +431,10 @@ class RuleTreeView(QTreeView):
         if self.btnCompare:
             self.btnCompare.setEnabled(enableCompare)
 
-    def showEvent(self, unusedEvent):
+    def showEvent(self, unusedEvent:'QShowEvent') ->None:
         """reload the models when the view comes into sight"""
         # default: make sure the name column is wide enough
+        assert self.ruleModel
         if self.ruleModel.canFetchMore():
             # we want to load all before adjusting column width
             self.ruleModel.fetchMore()
@@ -425,23 +447,23 @@ class RuleTreeView(QTreeView):
         for col in range(header.count()):
             self.resizeColumnToContents(col)
 
-    def selectedRow(self):
+    def selectedRow(self) ->Optional[QModelIndex]:
         """return the currently selected row index (with column 0)"""
         rows = self.selectionModel().selectedRows()
         return rows[0] if len(rows) == 1 else None
 
-    def copyRow(self):
+    def copyRow(self) ->None:
         """copy a ruleset"""
         row = self.selectedRow()
         if row:
             item = row.internalPointer()
             assert isinstance(item, RulesetItem)
-            ruleset = item.rawContent.copyTemplate()
-            self.model().appendRuleset(ruleset)
+            ruleset = item.raw.copyTemplate()
+            cast(RuleModel, self.model()).appendRuleset(ruleset)
             self.rulesets.append(ruleset)
             self.selectionChanged(self.selectionModel().selection())
 
-    def removeRow(self):
+    def removeRow(self) ->None:
         """removes a ruleset or a rule"""
         row = self.selectedRow()
         if row:
@@ -449,16 +471,16 @@ class RuleTreeView(QTreeView):
             assert not isinstance(item.ruleset(), PredefinedRuleset)
             assert isinstance(item, RulesetItem)
             ruleset = item.ruleset()
-            self.model().removeRows(row.row(), parent=row.parent())
+            self.model().removeRow(row.row(), parent=row.parent())
             self.rulesets.remove(ruleset)
             self.selectionChanged(self.selectionModel().selection())
 
-    def compareRow(self):
+    def compareRow(self) ->None:
         """shows the difference between two rulesets"""
         rows = self.selectionModel().selectedRows()
-        ruleset = rows[0].internalPointer().rawContent
+        ruleset = rows[0].internalPointer().raw
         assert isinstance(ruleset, Ruleset)
-        differ = RulesetDiffer(ruleset, self.rulesets)
+        differ = RulesetDiffer([ruleset], self.rulesets)
         differ.show()
         self.differs.append(differ)
 
@@ -467,12 +489,12 @@ class RulesetSelector(QWidget):
 
     """presents all available rulesets with previews"""
 
-    def __init__(self, parent=None):
+    def __init__(self, parent:Optional[QWidget]=None):
         super().__init__(parent)
         self.setContentsMargins(0, 0, 0, 0)
         self.setupUi()
 
-    def setupUi(self):
+    def setupUi(self) ->None:
         """layout the window"""
         decorateWindow(self, i18nc("@title:window", "Customize rulesets"))
         self.setObjectName('Rulesets')
@@ -483,7 +505,8 @@ class RulesetSelector(QWidget):
         v2layout = QVBoxLayout()
         hlayout.addWidget(self.v1widget)
         hlayout.addLayout(v2layout)
-        for widget in [self.v1widget, hlayout, v1layout, v2layout]:
+        widgets:List[Union[QWidget, 'QLayout']] = [self.v1widget, hlayout, v1layout, v2layout]
+        for widget in widgets:
             widget.setContentsMargins(0, 0, 0, 0)
         hlayout.setStretchFactor(self.v1widget, 10)
         self.btnCopy = QPushButton()
@@ -517,7 +540,7 @@ class RulesetSelector(QWidget):
         StateSaver(self)
         self.show()
 
-    def sizeHint(self):
+    def sizeHint(self) ->QSize:
         """we never want a horizontal scrollbar for player names,
         we always want to see them in full"""
         result = QWidget.sizeHint(self)
@@ -526,28 +549,31 @@ class RulesetSelector(QWidget):
         width = max(result.width(), available.width() // 2)
         return QSize(width, height)
 
-    def minimumSizeHint(self):
+    def minimumSizeHint(self) ->QSize:
         """we never want a horizontal scrollbar for player names,
         we always want to see them in full"""
         return self.sizeHint()
 
-    def showEvent(self, unusedEvent):
+    def showEvent(self, unusedEvent:'QShowEvent') ->None:
         """reload the rulesets"""
         self.refresh()
 
-    def refresh(self):
+    def refresh(self) ->None:
         """retranslate and reload rulesets"""
         self.retranslateUi()
         self.rulesetView.rulesets = Ruleset.availableRulesets()
 
-    def hideEvent(self, event):
+    def hideEvent(self, event:'QHideEvent') ->None:
         """close all differ dialogs"""
+        marked:List[RulesetDiffer] = []
         for differ in self.rulesetView.differs:
             differ.hide()
-            del differ
+            marked += differ
         QWidget.hideEvent(self, event)
+        for _ in marked:
+            del _
 
-    def retranslateUi(self):
+    def retranslateUi(self) ->None:
         """translate to current language"""
         self.btnCopy.setText(i18n("C&opy"))
         self.btnCompare.setText(i18nc('Kajongg ruleset comparer', 'Co&mpare'))
