@@ -7,12 +7,14 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
-from twisted.internet.defer import succeed
+from typing import Optional, Union, TYPE_CHECKING, Any, Generator, Literal, cast
 
-from log import logDebug, id4
+from twisted.internet.defer import succeed, Deferred
+
+from log import logDebug, logException
 from mi18n import i18n, i18nc
 from common import LIGHTSOURCES, Internal, isAlive, ZValues, Debug
-from common import StrMixin, Speeds
+from common import ReprMixin, Speeds, id4
 from wind import Wind
 
 from qt import Qt, QMetaObject
@@ -22,15 +24,22 @@ from dialogs import QuestionYesNo
 from guiutil import decorateWindow, sceneRotation
 from board import SelectorBoard, DiscardBoard
 from tileset import Tileset
-from meld import Meld
+from tile import Tile, Meld
 from humanclient import HumanClient
 from uitile import UITile
 from uiwall import UIWall
 from animation import AnimationSpeed, afterQueuedAnimations
 from scoringdialog import ScoringDialog
 
+if TYPE_CHECKING:
+    from qt import QEvent, QKeyEvent, QFocusEvent, QWidget
+    from game import Game
+    from handboard import HandBoard
+    from mainwindow import MainWindow
+    from humanclient import ClientDialog
+    from board import Board
 
-class FocusRect(QGraphicsRectItem, StrMixin):
+class FocusRect(QGraphicsRectItem, ReprMixin):
 
     """show a focusRect with blue border around focused tile or meld.
     We can NOT do this as a child of the item or the focus rect would
@@ -38,36 +47,38 @@ class FocusRect(QGraphicsRectItem, StrMixin):
     focus rect because Z order is only relevant for items having the
     same parent"""
 
-    def __init__(self):
+    def __init__(self) ->None:
         QGraphicsRectItem.__init__(self)
-        pen = QPen(QColor(Qt.blue))
+        pen = QPen(QColor('blue'))
         pen.setWidth(6)
         self.setPen(pen)
         self.setZValue(ZValues.markerZ)
-        self._board = None
+        self._board:Optional[Union[SelectorBoard, DiscardBoard]] = None
         self.hide()
 
     @property
-    def board(self):
+    def board(self) ->Optional[Union[SelectorBoard, DiscardBoard]]:
         """current board the focusrect is on"""
         return self._board
 
     @board.setter
-    def board(self, value):
+    def board(self, value: Optional[Union[SelectorBoard, DiscardBoard]]) ->None:
         """assign and show/hide as needed"""
         if value and not isAlive(value):
             logDebug(
-                'assigning focusRect to a non-alive board %s/%s' %
-                (type(value), value))
+                f'assigning focusRect to a non-alive board {type(value)}/{value}')
             return
         if value:
             self._board = value
             self.refresh()
 
-    @afterQueuedAnimations
-    def refresh(self, unusedDeferredResult=None):
+    @afterQueuedAnimations  # type:ignore[arg-type]
+    def refresh(self, unusedDeferredResult:Optional['Deferred']=None) ->None:
         """show/hide on correct position after queued animations end"""
         board = self.board
+        if not board:
+            # for mypy
+            return
         if not isAlive(board) or not isAlive(self):
             if isAlive(self):
                 self.setVisible(False)
@@ -80,25 +91,28 @@ class FocusRect(QGraphicsRectItem, StrMixin):
         if board.focusTile:
             board.focusTile.setFocus()
             self.setPos(board.focusTile.pos)
+        assert Internal.scene
         game = Internal.scene.game
-        self.setVisible(board.isVisible() and bool(board.focusTile)
-                        and board.isEnabled() and board.hasLogicalFocus and bool(game) and not game.autoPlay)
+        if game is None:
+            self.setVisible(False)
+        else:
+            self.setVisible(board.isVisible() and bool(board.focusTile)
+                        and board.isEnabled() and board.hasLogicalFocus and not game.autoPlay)
 
-
-    def __str__(self):
+    def __str__(self) ->str:
         """for debugging"""
-        return 'FocusRect({} on {})'.format(id4(self), self.board if self.board else 'NOBOARD')
+        return f"FocusRect_{id4(self)}({self.board if self.board else 'NOBOARD'})"
 
 class SceneWithFocusRect(QGraphicsScene):
 
     """our scene with a potential Qt bug fix. FocusRect is a blue frame around a tile or meld"""
 
-    def __init__(self):
+    def __init__(self) ->None:
         QGraphicsScene.__init__(self)
         self.focusRect = FocusRect()
         self.addItem(self.focusRect)
 
-    def focusInEvent(self, event):
+    def focusInEvent(self, event:'QFocusEvent') ->None:
         """
         Work around a qt bug. See U{https://bugreports.qt-project.org/browse/QTBUG-32890}.
         This can be reproduced as follows:
@@ -111,16 +125,16 @@ class SceneWithFocusRect(QGraphicsScene):
         """
         prev = self.focusItem()
         QGraphicsScene.focusInEvent(self, event)
-        if prev and bool(prev.flags() & QGraphicsItem.ItemIsFocusable) and prev != self.focusItem():
+        if prev and bool(prev.flags() & QGraphicsItem.GraphicsItemFlag.ItemIsFocusable) and prev != self.focusItem():
             self.setFocusItem(prev)
 
     @property
-    def focusBoard(self):
+    def focusBoard(self) ->Optional['Board']:
         """get / set the board that has its focusRect shown"""
         return self.focusRect.board
 
     @focusBoard.setter
-    def focusBoard(self, board):
+    def focusBoard(self, board:Optional[Union[DiscardBoard, SelectorBoard]]) ->None:
         """get / set the board that has its focusRect shown"""
         self.focusRect.board = board
 
@@ -128,34 +142,38 @@ class SceneWithFocusRect(QGraphicsScene):
 class GameScene(SceneWithFocusRect):
 
     """the game field"""
-    # pylint: disable=too-many-instance-attributes
 
-    def __init__(self, parent=None):
+    def __init__(self, parent:Optional['QWidget']=None) ->None:
         Internal.scene = self
-        self.mainWindow = parent
-        self._game = None
+        assert parent
+        self.mainWindow:'MainWindow' = parent
+        self._game:Optional['Game'] = None
         super().__init__()
 
         self.scoreTable = None
         self.explainView = None
+        self.clientDialog:Optional['ClientDialog']
         self.setupUi()
+        assert Internal.Preferences
         Internal.Preferences.addWatch('showShadows', self.showShadowsChanged)
 
     @property
-    def game(self):
+    def game(self) ->Optional['Game']:
         """a proxy"""
         return self._game
 
     @game.setter
-    def game(self, value):
+    def game(self, value:Optional['Game']) ->None:
         """if it changes, update GUI"""
         changing = self._game != value
         game = self._game
+        assert self.mainWindow
         self._game = value
         if changing:
             if value:
                 self.mainWindow.updateGUI()
             else:
+                assert game
                 game.close()
                 if self.scoreTable:
                     self.scoreTable.hide()
@@ -165,87 +183,102 @@ class GameScene(SceneWithFocusRect):
         self.mainWindow.updateGUI()
         self.mainWindow.adjustMainView()
 
-    @afterQueuedAnimations
-    def showShadowsChanged(self, deferredResult, unusedOldValue, unusedNewValue): # pylint: disable=unused-argument
+    @afterQueuedAnimations  # type:ignore[arg-type]
+    def showShadowsChanged(self, deferredResult:'Deferred', # pylint: disable=unused-argument
+        unusedOldValue:Any, unusedNewValue:Any) ->None:
         """if the wanted shadow direction changed, apply that change now"""
         for uiTile in self.graphicsTileItems():
             uiTile.setClippingFlags()
         self.applySettings()
 
-    def handSelectorChanged(self, handBoard):
+    def handSelectorChanged(self, handBoard:'HandBoard') ->None:
         """update all relevant dialogs"""
         if self.game and not self.game.finished():
+            assert handBoard.player
             handBoard.player.showInfo()
         # first decorate walls - that will compute player.handBoard for
         # explainView
         if self.explainView:
             self.explainView.refresh()
 
-    def setupUi(self):
+    def setupUi(self) ->None:
         """prepare scene"""
-        # pylint: disable=too-many-statements
+        assert Internal.Preferences
+        assert isinstance(Internal.Preferences.windTilesetName, str)
         self.windTileset = Tileset(Internal.Preferences.windTilesetName)
 
-    def showWall(self):
+    def showWall(self) ->None:
         """shows the wall according to the game rules (length may vary)"""
-        UIWall(self.game)   # sets self.game.wall
+        if self.game:
+            UIWall(self.game)   # sets self.game.wall
 
-    def abort(self):
+    def abort(self) ->Deferred:
         """abort current game"""
         # to be implemented by children
+        return Deferred()
 
-    def adjustSceneView(self):
+    def adjustSceneView(self) ->None:
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with AnimationSpeed():
+            assert self.game.wall
+            with AnimationSpeed(99):
                 self.game.wall.decorate4()
                 for uiTile in self.game.wall.tiles:
-                    if uiTile.board:
-                        uiTile.board.placeTile(uiTile)
+                    _ = cast(UITile, uiTile)
+                    if _.board:
+                        _.board.placeTile(_)
 
-    def applySettings(self):
+    def applySettings(self) ->None:
         """apply preferences"""
+        assert self.mainWindow
+        assert Internal.Preferences
         self.mainWindow.actionAngle.setEnabled(
-            bool(self.game) and Internal.Preferences.showShadows)
+            bool(self.game) and bool(Internal.Preferences.showShadows))
         with AnimationSpeed():
             for item in self.nonTiles():
                 if hasattr(item, 'tileset'):
                     item.tileset = Tileset.current()
 
-    def prepareHand(self):
+    def prepareHand(self) ->None:
         """redecorate wall"""
         self.mainWindow.updateGUI()
         if self.game:
-            with AnimationSpeed(Speeds.windMarker):
+            assert self.game.wall
+            with AnimationSpeed(Speeds.windDisc):
                 self.game.wall.decorate4()
 
-    def updateSceneGUI(self):
+    def updateSceneGUI(self) ->None:
         """update some actions, all auxiliary windows and the statusbar"""
         game = self.game
+        assert Internal.Preferences
         mainWindow = self.mainWindow
         for action in [mainWindow.actionScoreGame, mainWindow.actionPlayGame]:
             action.setEnabled(not bool(game))
         mainWindow.actionAbortGame.setEnabled(bool(game))
         mainWindow.actionAngle.setEnabled(
-            bool(game) and Internal.Preferences.showShadows)
+            bool(game) and bool(Internal.Preferences.showShadows))
         for view in [self.explainView, self.scoreTable]:
             if view:
                 view.refresh()
         self.__showBalance()
 
-    def newLightSource(self):
+    def newLightSource(self) ->Union[Literal['NE'], Literal['NW'], Literal['SW'], Literal['SE']]:
         """next value"""
-        oldIdx = LIGHTSOURCES.index(self.game.wall.lightSource)
-        return LIGHTSOURCES[(oldIdx + 1) % 4]
+        assert self.game
+        assert self.game.wall
+        oldIdx = LIGHTSOURCES.index(cast(UIWall, self.game.wall).lightSource)
+        return cast(Union[Literal['NE'], Literal['NW'], Literal['SW'], Literal['SE']], LIGHTSOURCES[(oldIdx + 1) % 4])
 
-    def changeAngle(self):
+    def changeAngle(self) ->None:
         """change the lightSource"""
-        self.game.wall.lightSource = self.newLightSource()
+        assert self.game
+        assert self.game.wall
+        cast(UIWall, self.game.wall).lightSource = self.newLightSource()
         self.focusRect.refresh()
         self.mainWindow.adjustMainView()
 
-    def __showBalance(self):
+    def __showBalance(self) ->None:
         """show the player balances in the status bar"""
         sBar = self.mainWindow.statusBar()
         if self.game:
@@ -255,26 +288,28 @@ class GameScene(SceneWithFocusRect):
                     sBar.changeItem(sbMessage, idx)
                 else:
                     sBar.insertItem(sbMessage, idx, 1)
-                    sBar.setItemAlignment(idx, Qt.AlignLeft)
+                    sBar.setItemAlignment(idx, Qt.AlignmentFlag.AlignLeft)
         else:
             for idx in range(5):
                 if sBar.hasItem(idx):
                     sBar.removeItem(idx)
 
-    def graphicsTileItems(self):
+    def graphicsTileItems(self) ->Generator[UITile, None, None]:
         """return all UITile in the scene"""
         return (x for x in self.items() if isinstance(x, UITile))
 
-    def nonTiles(self):
+    def nonTiles(self) ->Generator[QGraphicsItem, None, None]:
         """return all other items in the scene"""
         return (x for x in self.items() if not isinstance(x, UITile))
 
-    def removeTiles(self):
+    def removeTiles(self) ->None:
         """remove all tiles from scene"""
         for item in self.graphicsTileItems():
             self.removeItem(item)
         for wind in Wind.all:
-            wind.marker = None
+            if hasattr(wind, 'disc'):
+                wind.disc.hide()
+                delattr(wind, 'disc')
         self.focusRect.hide()
 
 
@@ -282,15 +317,15 @@ class PlayingScene(GameScene):
 
     """scene with a playing game"""
 
-    def __init__(self, parent):
+    def __init__(self, parent:Optional['QWidget']=None) ->None:
         self._game = None
         self.__startingGame = True
-        self._clientDialog = None
+        self._clientDialog:Optional['ClientDialog'] = None
 
         super().__init__(parent)
 
-    @GameScene.game.setter
-    def game(self, value):  # pylint: disable=arguments-differ
+    @GameScene.game.setter  # type: ignore
+    def game(self, value):
         game = self._game
         changing = value != game
         GameScene.game.fset(self, value)
@@ -307,24 +342,19 @@ class PlayingScene(GameScene):
             and bool(value.client.table.chatWindow))
 
     @property
-    def clientDialog(self):
+    def clientDialog(self) ->Optional['ClientDialog']:
         """wrapper: hide dialog when it is set to None"""
         return self._clientDialog
 
     @clientDialog.setter
-    def clientDialog(self, value):
+    def clientDialog(self, value:Optional['ClientDialog']) ->None:
         """wrapper: hide dialog when it is set to None"""
-        if isAlive(self._clientDialog) and not value:
+        if self._clientDialog and isAlive(self._clientDialog) and not value:
             self._clientDialog.timer.stop()
             self._clientDialog.hide()
         self._clientDialog = value
 
-    def resizeEvent(self, unusedEvent):
-        """main window changed size"""
-        if self.clientDialog:
-            self.clientDialog.placeInField()
-
-    def setupUi(self):
+    def setupUi(self) ->None:
         """create all other widgets
         we could make the scene view the central widget but I did
         not figure out how to correctly draw the background with
@@ -332,7 +362,6 @@ class PlayingScene(GameScene):
         QGraphicsView.drawBackground always wants a pixmap
         for a huge rect like 4000x3000 where my screen only has
         1920x1200"""
-        # pylint: disable=too-many-statements
         GameScene.setupUi(self)
         self.setObjectName("PlayingField")
 
@@ -341,19 +370,21 @@ class PlayingScene(GameScene):
 
         self.adjustSceneView()
 
-    def showWall(self):
+    def showWall(self) ->None:
         """shows the wall according to the game rules (length may vary)"""
         GameScene.showWall(self)
         self.discardBoard.maximize()
 
-    def abort(self):
+    def abort(self) ->'Deferred':
         """abort current game"""
-        def gotAnswer(result, autoPlaying):
+        def gotAnswer(gotResult:'Deferred', autoPlaying:bool) ->Union[bool, 'Deferred']:
             """user answered"""
-            if result:
+            result:Union[bool, 'Deferred'] = gotResult
+            if result is True:
                 self.game = None
             else:
                 self.mainWindow.actionAutoPlay.setChecked(autoPlaying)
+                result = False  # this implicitly handles the Cancelled Error
             return result
         if not self.game:
             return succeed(True)
@@ -362,18 +393,18 @@ class PlayingScene(GameScene):
             self.game = None
             return succeed(True)
         autoPlaying = self.mainWindow.actionAutoPlay.isChecked()
-        return QuestionYesNo(i18n("Do you really want to abort this game?"), always=True).addCallback(
+        return QuestionYesNo(i18n("Do you really want to abort this game?"), always=True).addBoth(
             gotAnswer, autoPlaying)
 
-    def keyPressEvent(self, event):
+    def keyPressEvent(self, event:'QKeyEvent') ->None:
         """if we have a clientDialog, pass event to it"""
         mod = event.modifiers()
-        if mod in (Qt.NoModifier, Qt.ShiftModifier):
+        if mod in (Qt.KeyboardModifier.NoModifier, Qt.KeyboardModifier.ShiftModifier):
             if self.clientDialog:
                 self.clientDialog.keyPressEvent(event)
         GameScene.keyPressEvent(self, event)
 
-    def adjustSceneView(self):
+    def adjustSceneView(self) ->None:
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
@@ -382,23 +413,22 @@ class PlayingScene(GameScene):
         GameScene.adjustSceneView(self)
 
     @property
-    def startingGame(self):
+    def startingGame(self) ->bool:
         """are we trying to start a game?"""
         return self.__startingGame
 
     @startingGame.setter
-    def startingGame(self, value):
+    def startingGame(self, value:bool) ->None:
         """are we trying to start a game?"""
         if value != self.__startingGame:
             self.__startingGame = value
             self.mainWindow.updateGUI()
 
-    def applySettings(self):
+    def applySettings(self) ->None:
         """apply preferences"""
         GameScene.applySettings(self)
-        self.discardBoard.showShadows = Internal.Preferences.showShadows
 
-    def toggleDemoMode(self, checked):
+    def toggleDemoMode(self, checked:bool) ->None:
         """switch on / off for autoPlay"""
         if self.game:
             self.focusRect.refresh()  # show/hide it
@@ -410,7 +440,7 @@ class PlayingScene(GameScene):
                 self.clientDialog.selectButton()
                                                # select default, abort timeout
 
-    def updateSceneGUI(self):
+    def updateSceneGUI(self) ->None:
         """update some actions, all auxiliary windows and the statusbar"""
         if not isAlive(self):
             return
@@ -419,7 +449,7 @@ class PlayingScene(GameScene):
         mainWindow = self.mainWindow
         if not game:
             connections = [x.connection for x in HumanClient.humanClients if x.connection]
-            title = ', '.join('{name}/{url}'.format(name=x.username, url=x.url) for x in connections)
+            title = ', '.join(f'{x.username}/{x.url}' for x in connections)
             if title:
                 decorateWindow(mainWindow, title)
         else:
@@ -439,7 +469,7 @@ class PlayingScene(GameScene):
             ) and bool(
                 game.client.table.chatWindow))
 
-    def changeAngle(self):
+    def changeAngle(self) ->None:
         """now that no animation is running, really change"""
         self.discardBoard.lightSource = self.newLightSource()
         GameScene.changeAngle(self)
@@ -448,18 +478,17 @@ class PlayingScene(GameScene):
 class ScoringScene(GameScene):
 
     """a scoring game"""
-    # pylint: disable=too-many-instance-attributes
 
-    def __init__(self, parent=None):
-        self.scoringDialog = None
+    def __init__(self, parent:Optional['QWidget']=None) ->None:
+        self.scoringDialog:Optional[ScoringDialog] = None
         super().__init__(parent)
         self.selectorBoard.hasLogicalFocus = True
 
-    @GameScene.game.setter
-    def game(self, value):  # pylint: disable=arguments-differ
+    @GameScene.game.setter  # type: ignore
+    def game(self, value:'Game') ->None:
         game = self._game
         changing = value != game
-        GameScene.game.fset(self, value)
+        GameScene.game.fset(self, value)  # type:ignore[attr-defined]
         if changing:
             if value is not None:
                 self.scoringDialog = ScoringDialog(scene=self)
@@ -467,13 +496,13 @@ class ScoringScene(GameScene):
                 self.scoringDialog.hide()
                 self.scoringDialog = None
 
-    def handSelectorChanged(self, handBoard):
+    def handSelectorChanged(self, handBoard:'HandBoard') ->None:
         """update all relevant dialogs"""
         GameScene.handSelectorChanged(self, handBoard)
         if self.scoringDialog:
             self.scoringDialog.slotInputChanged()
 
-    def setupUi(self):
+    def setupUi(self) ->None:
         """create all other widgets"""
         GameScene.setupUi(self)
         self.setObjectName("ScoringScene")
@@ -481,9 +510,9 @@ class ScoringScene(GameScene):
         self.addItem(self.selectorBoard)
         QMetaObject.connectSlotsByName(self)
 
-    def abort(self):
+    def abort(self) ->'Deferred':
         """abort current game"""
-        def answered(result):
+        def answered(result:'Deferred') ->'Deferred':
             """got answer"""
             if result:
                 self.game = None
@@ -495,27 +524,30 @@ class ScoringScene(GameScene):
         if self.game.finished():
             self.game = None
             return succeed(True)
-        return QuestionYesNo(i18n("Do you really want to abort this game?"), always=True).addCallback(answered)
+        return QuestionYesNo(i18n("Do you really want to abort this game?"), always=True).addCallback(
+            answered).addErrback(logException)
 
-    def __moveTile(self, uiTile, wind, toConcealed):
+    def __moveTile(self, uiTile:UITile, wind:str, toConcealed:bool) ->None:
         """the user pressed a wind letter or X for center, wanting to move a uiTile there"""
         # this tells the receiving board that this is keyboard, not mouse navigation>
         # needed for useful placement of the popup menu
-        currentBoard = uiTile.board
         if wind == 'X':
             receiver = self.selectorBoard
         else:
-            receiver = self.game.players[wind].handBoard
+            receiver = self.game.players[Wind(wind)].handBoard
+        currentBoard = uiTile.board
+        assert currentBoard
         movingMeld = currentBoard.uiMeldWithTile(uiTile)
         if receiver != currentBoard or toConcealed != movingMeld.meld.isConcealed:
             movingLastMeld = movingMeld.meld == self.computeLastMeld()
+            assert self.scoringDialog
             if movingLastMeld:
                 self.scoringDialog.clearLastTileCombo()
             receiver.dropTile(uiTile, toConcealed)
             if movingLastMeld and receiver == currentBoard:
                 self.scoringDialog.fillLastTileCombo()
 
-    def __navigateScoringGame(self, event):
+    def __navigateScoringGame(self, event:'QKeyEvent') ->bool:
         """keyboard navigation in a scoring game"""
         mod = event.modifiers()
         key = event.key()
@@ -523,17 +555,17 @@ class ScoringScene(GameScene):
         windsX = ''.join(x.char for x in Wind.all)
         moveCommands = i18nc('kajongg:keyboard commands for moving tiles to the players '
                              'with wind ESWN or to the central tile selector (X)', windsX)
-        uiTile = self.focusItem()
+        uiTile = cast(UITile, self.focusItem())
         if wind in moveCommands:
             # translate i18n wind key to ESWN:
-            wind = windsX[moveCommands.index(wind)]
-            self.__moveTile(uiTile, wind, bool(mod & Qt.ShiftModifier))
+            wind_chr = windsX[moveCommands.index(wind)]
+            self.__moveTile(uiTile, wind_chr, bool(mod & Qt.KeyboardModifier.ShiftModifier))
             return True
-        if key == Qt.Key_Tab and self.game:
+        if key == Qt.Key.Key_Tab and self.game:
             tabItems = [self.selectorBoard]
             tabItems.extend(p.handBoard for p in self.game.players if p.handBoard.uiTiles)
             tabItems.append(tabItems[0])
-            currentBoard = uiTile.board
+            currentBoard = uiTile.board  # type: ignore[attr-defined]
             currIdx = 0
             while tabItems[currIdx] != currentBoard and currIdx < len(tabItems) - 2:
                 currIdx += 1
@@ -541,16 +573,16 @@ class ScoringScene(GameScene):
             return True
         return False
 
-    def keyPressEvent(self, event):
+    def keyPressEvent(self, event:'QKeyEvent') ->None:
         """navigate in the selectorboard"""
         mod = event.modifiers()
-        if mod in (Qt.NoModifier, Qt.ShiftModifier):
+        if mod in (Qt.KeyboardModifier.NoModifier, Qt.KeyboardModifier.ShiftModifier):
             if self.game:
                 if self.__navigateScoringGame(event):
                     return
         GameScene.keyPressEvent(self, event)
 
-    def adjustSceneView(self):
+    def adjustSceneView(self) ->None:
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
@@ -558,13 +590,13 @@ class ScoringScene(GameScene):
                 self.selectorBoard.maximize()
         GameScene.adjustSceneView(self)
 
-    def prepareHand(self):
+    def prepareHand(self) ->None:
         """redecorate wall"""
         GameScene.prepareHand(self)
         if self.scoringDialog:
             self.scoringDialog.clearLastTileCombo()
 
-    def updateSceneGUI(self):
+    def updateSceneGUI(self) ->None:
         """update some actions, all auxiliary windows and the statusbar"""
         if not isAlive(self):
             return
@@ -577,19 +609,19 @@ class ScoringScene(GameScene):
         self.selectorBoard.setVisible(bool(game))
         self.selectorBoard.setEnabled(bool(game))
 
-    def changeAngle(self):
+    def changeAngle(self) ->None:
         """now that no animation is running, really change"""
         self.selectorBoard.lightSource = self.newLightSource()
         GameScene.changeAngle(self)
 
-    def computeLastTile(self):
+    def computeLastTile(self) ->'Tile':
         """compile hand info into a string as needed by the scoring engine"""
         if self.scoringDialog:
             # is None while ScoringGame is created
             return self.scoringDialog.computeLastTile()
-        return None
+        return Tile.none
 
-    def computeLastMeld(self):
+    def computeLastMeld(self) ->Meld:
         """compile hand info into a string as needed by the scoring engine"""
         if self.scoringDialog:
             # is None while ScoringGame is created
