@@ -8,6 +8,8 @@ SPDX-License-Identifier: GPL-2.0
 """
 
 import datetime
+from itertools import chain
+from typing import Tuple, Optional, TYPE_CHECKING, List, cast, Generator
 
 from qt import QPointF, QRectF, QDialogButtonBox
 from qt import QGraphicsRectItem, QGraphicsSimpleTextItem
@@ -15,12 +17,13 @@ from qt import QPushButton, QMessageBox, QComboBox
 
 
 from common import Internal, isAlive, Debug
+from tile import MeldList
 from wind import Wind
 from tilesource import TileSource
 from animation import animate
 from log import logError, logDebug, logWarning, i18n
 from query import Query
-from uitile import UITile
+from uitile import UITile, UIMeld
 from board import WindLabel, Board
 from game import Game
 from games import Games
@@ -33,13 +36,27 @@ from uiwall import UIWall, SideText
 from guiutil import decorateWindow, BlockSignals, rotateCenter, sceneRotation
 from mi18n import i18nc
 
+if TYPE_CHECKING:
+    from qt import QWidget, QEvent, QGraphicsSceneDragDropEvent, QGraphicsItem, QGraphicsItemGroup, QObject
+    from tile import Tile, Meld
+    from board import MimeData, SelectorBoard
+    from scoringdialog import RuleBox
+    from scene import ScoringScene
+    from rule import Score, UsedRule, Ruleset, Rule
+    from client import Client
+
+def scoringScene() ->'ScoringScene':
+    """shortcut"""
+    result = cast('ScoringScene', Internal.scene)
+    assert result
+    return result
 
 class SwapDialog(QMessageBox):
 
     """ask the user if two players should change seats"""
 
-    def __init__(self, swappers):
-        QMessageBox.__init__(self)
+    def __init__(self, swappers:List[Player], parent:Optional['QWidget']=None) ->None:
+        QMessageBox.__init__(self, parent)
         decorateWindow(self, i18nc("@title:window", "Swap Seats"))
         self.setText(
             i18n("By the rules, %1 and %2 should now exchange their seats. ",
@@ -54,12 +71,12 @@ class SelectPlayers(SelectRuleset):
 
     """a dialog for selecting four players. Used only for scoring game."""
 
-    def __init__(self):
+    def __init__(self) ->None:
         SelectRuleset.__init__(self)
         Players.load()
         decorateWindow(self, i18nc("@title:window", "Select four players"))
-        self.names = None
-        self.nameWidgets = []
+        self.names:List[str]
+        self.nameWidgets:List['QWidget'] = []
         for idx, wind in enumerate(Wind.all4):
             cbName = QComboBox()
             cbName.manualSelect = False
@@ -82,19 +99,19 @@ class SelectPlayers(SelectRuleset):
                         if playerIdx >= 0:
                             cbName.setCurrentIndex(playerIdx)
                     except KeyError:
-                        logError('database is inconsistent: player with id %d is in game but not in player'
-                                 % playerId)
+                        logError(f'database is inconsistent: player with id {int(playerId)} '
+                                 f'is in game but not in player')
         self.slotValidate()
 
-    def showEvent(self, unusedEvent):
+    def showEvent(self, unusedEvent:'QEvent') ->None:
         """start with player 0"""
         self.nameWidgets[0].setFocus()
 
-    def __selectedNames(self):
+    def __selectedNames(self) ->set[str]:
         """A set with the currently selected names"""
         return {cbName.currentText() for cbName in self.nameWidgets}
 
-    def slotValidate(self):
+    def slotValidate(self) ->None:
         """try to find 4 different players and update status of the Ok button"""
         changedCombo = self.sender()
         if not isinstance(changedCombo, QComboBox):
@@ -127,11 +144,11 @@ class ScoringTileAttr(TileAttr):
 
     """Tile appearance is different in a ScoringHandBoard"""
 
-    def setDark(self):
+    def setDark(self) ->bool:
         """should the tile appear darker?"""
-        return self.yoffset or self.tile.isConcealed
+        return bool(self.yoffset) or self.tile.isConcealed
 
-    def setFocusable(self, hand, meld, idx):
+    def setFocusable(self, hand:'HandBoard', meld:'Meld', idx:Optional[int]) ->bool:
         """in a scoring handboard, only the first tile of a meld is focusable"""
         return idx == 0
 
@@ -139,55 +156,65 @@ class ScoringTileAttr(TileAttr):
 class ScoringHandBoard(HandBoard):
 
     """a board showing the tiles a player holds"""
-    # pylint: disable=too-many-public-methods,too-many-instance-attributes
     tileAttrClass = ScoringTileAttr
 
-    def __init__(self, player):
-        self.__moveHelper = None
-        self.uiMelds = []
+    def __init__(self, player:'ScoringPlayer') ->None:
+        self.__moveHelper:Optional['QGraphicsItemGroup'] = None
+        self.uiMelds:List[UIMeld] = []
         HandBoard.__init__(self, player)
+        self.player:'ScoringPlayer'
 
-    def meldVariants(self, tile, lowerHalf):
+    def meldVariants(self, tile:UITile, forLowerHalf:bool) ->MeldList:
         """Kong might have variants"""
-        result = []
-        meld = self.uiMeldWithTile(tile).meld  # pylint: disable=no-member
-        result.append(meld.concealed if lowerHalf else meld.exposed)
+        result = MeldList()
+        uimeld = self.uiMeldWithTile(tile)
+        if not uimeld:
+            logWarning(f'ScoringHandBoard.meldVariants: no meld found for {tile}')
+            return result
+        meld = uimeld.meld
+        result.append(meld.concealed if forLowerHalf else meld.exposed)
         if len(meld) == 4:
-            if lowerHalf:
-                result = [meld.declared]
+            if forLowerHalf:
+                result.append(meld.declared)
             else:
                 result.append(meld.exposedClaimed)
         return result
 
-    def mapMouseTile(self, uiTile):
+    def mapMouseTile(self, uiTile:UITile) ->UITile:
         """map the pressed tile to the wanted tile. For melds, this would
         be the first tile no matter which one is pressed"""
-        return self.uiMeldWithTile(uiTile)[0]
+        uiMeld = self.uiMeldWithTile(uiTile)
+        assert uiMeld
+        return uiMeld[0]
 
-    def uiMeldWithTile(self, uiTile):
+    def uiMeldWithTile(self, uiTile:UITile) ->UIMeld:
         """return the meld with uiTile"""
         for myMeld in self.uiMelds:
             if uiTile in myMeld:
                 return myMeld
-        return None
+        logWarning(f'ScoringBoard: found no meld with {uiTile}')
+        return UIMeld(uiTile)
 
-    def findUIMeld(self, meld):
+    def findUIMeld(self, meld:Optional['Meld']) ->UIMeld:
         """find the first UIMeld matching the logical meld"""
         for result in self.uiMelds:
             if result.meld == meld:
                 return result
-        return None
+        logWarning(f'Scoring game: cannot find UIMeld for {meld}')
+        if self.uiMelds:
+            return self.uiMelds[1]
+        raise ValueError('Scoring Game: findUIMeld() in absence of any uiMelds')
 
-    def assignUITiles(self, uiTile, meld):  # pylint: disable=unused-argument
+    def assignUITiles(self, uiTile:UITile, meld:'Meld') ->Optional[UIMeld]:  # pylint: disable=unused-argument
         """generate a UIMeld. First uiTile is given, the rest should be as defined by meld"""
         assert isinstance(uiTile, UITile), uiTile
         return self.uiMeldWithTile(uiTile)
 
-    def sync(self, adding=None):  # pylint: disable=unused-argument
+    def sync(self, adding:Optional[List[UITile]]=None) ->None:
         """place all tiles in ScoringHandBoard"""
-        self.placeTiles(sum(self.uiMelds, []))
+        self.placeTiles(cast(List[UITile], list(chain(*self.uiMelds))))
 
-    def deselect(self, meld):
+    def deselect(self, meld:UIMeld) ->None:
         """remove meld from old board"""
         for idx, uiMeld in enumerate(self.uiMelds):
             if all(id(meld[x]) == id(uiMeld[x]) for x in range(len(meld))):
@@ -196,9 +223,9 @@ class ScoringHandBoard(HandBoard):
                 break                 # identical melds, it removes the wrong one
         self.player.removeMeld(meld)  # uiMeld must already be deleted
 
-    def dragMoveEvent(self, event):
+    def dragMoveEvent(self, event:'QGraphicsSceneDragDropEvent') ->None:
         """allow dropping of uiTile from ourself only to other state (open/concealed)"""
-        uiTile = event.mimeData().uiTile
+        uiTile = cast('MimeData', event.mimeData()).uiTile
         localY = self.mapFromScene(QPointF(event.scenePos())).y()
         centerY = self.rect().height() / 2.0
         newLowerHalf = localY >= centerY
@@ -210,38 +237,41 @@ class ScoringHandBoard(HandBoard):
         elif uiTile.isBonus:
             doAccept = False
         else:
-            oldLowerHalf = uiTile.board.isHandBoard and uiTile in uiTile.board.lowerHalfTiles(
-            )
+            oldLowerHalf = False
+            if uiTile.board.isHandBoard:
+                oldLowerHalf = uiTile in cast('HandBoard', uiTile.board).lowerHalfTiles()
             doAccept = oldLowerHalf != newLowerHalf
         event.setAccepted(doAccept)
 
-    def dropEvent(self, event):
+    def dropEvent(self, event:'QGraphicsSceneDragDropEvent') ->None:
         """drop into this handboard"""
-        uiTile = event.mimeData().uiTile
-        lowerHalf = self.mapFromScene(
+        uiTile = cast('MimeData', event.mimeData()).uiTile
+        forLowerHalf = self.mapFromScene(
             QPointF(event.scenePos())).y() >= self.rect().height() / 2.0
-        if self.dropTile(uiTile, lowerHalf):
+        if self.dropTile(uiTile, forLowerHalf):
             event.accept()
         else:
             event.ignore()
         self._noPen()
 
-    def dropTile(self, uiTile, lowerHalf):
+    def dropTile(self, uiTile:UITile, forLowerHalf:bool) ->bool:
         """drop uiTile into lower or upper half of our hand"""
-        senderBoard = uiTile.board
-        newMeld = senderBoard.chooseVariant(uiTile, lowerHalf)
+        senderBoard = cast('SelectorBoard', uiTile.board)
+        assert senderBoard
+        newMeld = senderBoard.chooseVariant(uiTile, forLowerHalf)
         if not newMeld:
             return False
         uiMeld = senderBoard.assignUITiles(uiTile, newMeld)
         for uitile, tile in zip(uiMeld, newMeld):
-            uitile.tile = tile
+            uitile.change_name(tile)
         return self.dropMeld(uiMeld)
 
-    def dropMeld(self, uiMeld):
+    def dropMeld(self, uiMeld:UIMeld) ->bool:
         """drop uiMeld into our hand"""
         senderBoard = uiMeld[0].board
         senderBoard.deselect(uiMeld)
         self.uiMelds.append(uiMeld)
+        assert self.player
         self.player.addMeld(uiMeld.meld)
         self.sync()
         self.hasLogicalFocus = senderBoard == self or not senderBoard.uiTiles
@@ -249,27 +279,34 @@ class ScoringHandBoard(HandBoard):
         senderBoard.autoSelectTile()
         senderBoard.checkTiles()
         if senderBoard is not self and senderBoard.isHandBoard:
-            Internal.scene.handSelectorChanged(senderBoard)
-        Internal.scene.handSelectorChanged(self)
+            scoringScene().handSelectorChanged(senderBoard)
+        scoringScene().handSelectorChanged(self)
         animate()
         self.checkTiles()
         return True
 
-    def focusRectWidth(self):
+    def focusRectWidth(self) ->int:
         """how many tiles are in focus rect? We want to focus
         the entire meld"""
-        meld = self.uiMeldWithTile(self.focusTile)
-        return len(meld) if meld else 1
+        focus = self.focusTile
+        if not focus:
+            logWarning('ScoringBoard.focusRectWidth: there is no focus tile')
+            return 1
+        meld = self.uiMeldWithTile(focus)
+        if not meld:
+            logWarning(f'ScoringBoard.focusRectWidth: there is no meld with focus tile {focus}')
+            return 1
+        return len(meld)
 
-    def addUITile(self, uiTile):
+    def addUITile(self, uiTile:UITile) ->None:
         Board.addUITile(self, uiTile)
         self.showMoveHelper()
 
-    def removeUITile(self, uiTile):
+    def removeUITile(self, uiTile:UITile) ->None:
         Board.removeUITile(self, uiTile)
         self.showMoveHelper()
 
-    def showMoveHelper(self, visible=None):
+    def showMoveHelper(self, visible:Optional[bool]=None) ->None:
         """show help text In empty HandBoards"""
         if visible is None:
             visible = not self.uiTiles
@@ -284,7 +321,7 @@ class ScoringHandBoard(HandBoard):
                     hbCenter.y(),
                     hbCenter.x() * 1,
                     1)
-                helpItems = [splitter]
+                helpItems:List['QGraphicsItem'] = [splitter]
                 for name, yFactor in [(i18n('Move Exposed Tiles Here'), 0.5),
                                       (i18n('Move Concealed Tiles Here'), 1.5)]:
                     helper = QGraphicsSimpleTextItem(name, self)
@@ -304,58 +341,62 @@ class ScoringHandBoard(HandBoard):
             if self.__moveHelper:
                 self.__moveHelper.setVisible(False)
 
-    def newLowerMelds(self):
+    def newLowerMelds(self) ->MeldList:
         """a list of melds for the hand as it should look after sync"""
-        return list(self.player.concealedMelds)
+        assert self.player
+        return MeldList(self.player.concealedMelds)
 
 
 class ScoringPlayer(VisiblePlayer, Player):
 
     """Player in a scoring game"""
-    # pylint: disable=too-many-public-methods
 
-    def __init__(self, game, name):
+    def __init__(self, game:'ScoringGame', name:str) ->None:
         self.handBoard = None  # because Player.init calls clearHand()
-        self.manualRuleBoxes = []
+        self.manualRuleBoxes:List['RuleBox'] = []
         Player.__init__(self, game, name)
         VisiblePlayer.__init__(self)
         self.handBoard = ScoringHandBoard(self)
+        self.game:'ScoringGame'
 
-    def clearHand(self):
+    def clearHand(self) ->None:
         """clears attributes related to current hand"""
         Player.clearHand(self)
         if self.game and self.game.wall:
+            assert self.game
+            assert self.game.wall
             # is None while __del__
             self.front = self.game.wall[self.idx]
             if hasattr(self, 'sideText'):
                 self.sideText.board = self.front
         if isAlive(self.handBoard):
-            self.handBoard.setEnabled(True)
-            self.handBoard.showMoveHelper()
-            self.handBoard.uiMelds = []
+            assert (_ := cast(ScoringHandBoard, self.handBoard))
+            _.setEnabled(True)
+            _.showMoveHelper()
+            _.uiMelds = []
         self.manualRuleBoxes = []
 
-    def explainHand(self):
+    def explainHand(self) ->'Hand':
         """return the hand to be explained"""
         return self.hand
 
     @property
-    def handTotal(self):
+    def handTotal(self) ->int:
         """the hand total of this player"""
         if self.hasManualScore():
-            spValue = Internal.scene.scoringDialog.spValues[self.idx]
+            assert (_ := scoringScene().scoringDialog)
+            spValue = _.spValues[self.idx]
             return spValue.value()
         return self.hand.total()
 
-    def hasManualScore(self):
+    def hasManualScore(self) ->bool:
         """True if no tiles are assigned to this player"""
-        if Internal.scene.scoringDialog:
-            return Internal.scene.scoringDialog.spValues[self.idx].isEnabled()
+        if _ := scoringScene().scoringDialog:
+            return _.spValues[self.idx].isEnabled()
         return False
 
-    def refreshManualRules(self, sender=None):
+    def refreshManualRules(self, sender:Optional['QObject']=None) ->None:
         """update status of manual rules"""
-        assert Internal.scene
         if not self.handBoard:
             # might happen at program exit
             return
@@ -371,13 +412,13 @@ class ScoringPlayer(VisiblePlayer, Player):
                     box, currentScore)
             box.setApplicable(applicable)
 
-    def __ruleChangesScore(self, box, currentScore):
+    def __ruleChangesScore(self, box:'RuleBox', currentScore:'Score') ->bool:
         """does the rule actually influence the result?
         if the action would only influence the score and the rule does not change the score,
         ignore the rule. If however the action does other things like penalties leave it applicable"""
         if box.rule.hasNonValueAction():
             return True
-        with BlockSignals(box):
+        with BlockSignals([box]):
             try:
                 checked = box.isChecked()
                 box.setChecked(not checked)
@@ -386,7 +427,7 @@ class ScoringPlayer(VisiblePlayer, Player):
                 box.setChecked(checked)
         return newHand.score > currentScore
 
-    def __mjstring(self):
+    def __mjstring(self) ->str:
         """compile hand info into a string as needed by the scoring engine"""
         if self.lastTile and self.lastTile.isConcealed:
             lastSource = TileSource.LivingWall.char
@@ -404,57 +445,60 @@ class ScoringPlayer(VisiblePlayer, Player):
                 announcements |= set(options['announcements'])
         return ''.join(['m', lastSource, ''.join(sorted(announcements))])
 
-    def __lastString(self):
+    def __lastString(self) ->str:
         """compile hand info into a string as needed by the scoring engine"""
         if not self.lastTile:
             return ''
+        assert self.handBoard
         if not self.handBoard.tilesByElement(self.lastTile):
             # this happens if we remove the meld with lastTile from the hand
             # again
             return ''
-        return 'L%s%s' % (self.lastTile, self.lastMeld)
+        return f'L{self.lastTile}{self.lastMeld}'
 
-    def computeHand(self):
+    def computeHand(self) ->Hand:
         """return a Hand object, using a cache"""
-        self.lastTile = Internal.scene.computeLastTile()
-        self.lastMeld = Internal.scene.computeLastMeld()
+        self.lastTile = scoringScene().computeLastTile()
+        self.lastMeld = scoringScene().computeLastMeld()
         string = ' '.join(
             [self.scoringString(),
              self.__mjstring(),
              self.__lastString()])
         return Hand(self, string)
 
-    def sortRulesByX(self, rules):
+    def sortRulesByX(self, rules:List['UsedRule']) ->List['UsedRule']:
         """if this game has a GUI, sort rules by GUI order of the melds they are applied to"""
         withMelds = [x for x in rules if x.meld]
         withoutMelds = [x for x in rules if x not in withMelds]
-        tuples = [tuple([x, self.handBoard.findUIMeld(x.meld)]) for x in withMelds]
-        tuples = sorted(tuples, key=lambda x: x[1][0].sortKey())
-        return [x[0] for x in tuples] + withoutMelds
+        assert (_ := cast('ScoringHandBoard', self.handBoard))
+        tuples = cast(Generator[Tuple['UsedRule', 'UIMeld'], None, None],
+            [tuple([x, _.findUIMeld(x.meld)]) for x in withMelds])
+        sorted_tuples = sorted(tuples, key=lambda x: x[1][0].sortKey())
+        return [x[0] for x in sorted_tuples] + withoutMelds
 
-    def addMeld(self, meld):
+    def addMeld(self, meld:'Meld') ->None:
         """add meld to this hand in a scoring game"""
         if meld.isBonus:
             self._bonusTiles.append(meld[0])
             if Debug.scoring:
-                logDebug('{} gets bonus tile {}'.format(self, meld[0]))
+                logDebug(f'{self} gets bonus tile {meld[0]}')
         elif meld.isConcealed and not meld.isKong:
             self._concealedMelds.append(meld)
             if Debug.scoring:
-                logDebug('{} gets concealed meld {}'.format(self, meld))
+                logDebug(f'{self} gets concealed meld {meld}')
         else:
             self._exposedMelds.append(meld)
             if Debug.scoring:
-                logDebug('{} gets exposed meld {}'.format(self, meld))
+                logDebug(f'{self} gets exposed meld {meld}')
         self._hand = None
 
-    def removeMeld(self, uiMeld):
+    def removeMeld(self, uiMeld:UIMeld) ->None:
         """remove a meld from this hand in a scoring game"""
         meld = uiMeld.meld
         if meld.isBonus:
             self._bonusTiles.remove(meld[0])
             if Debug.scoring:
-                logDebug('{} loses bonus tile {}'.format(self, meld[0]))
+                logDebug(f'{self} loses bonus tile {meld[0]}')
         else:
             popped = None
             for melds in [self._concealedMelds, self._exposedMelds]:
@@ -464,13 +508,12 @@ class ScoringPlayer(VisiblePlayer, Player):
                         break
             if not popped:
                 logDebug(
-                    '%s: %s.removeMeld did not find %s' %
-                    (self.name, self.__class__.__name__, meld), showStack=3)
-                logDebug('    concealed: %s' % self._concealedMelds)
-                logDebug('      exposed: %s' % self._exposedMelds)
+                    f'{self.name}: {self.__class__.__name__}.removeMeld did not find {meld}', showStack=True)
+                logDebug(f'    concealed: {self._concealedMelds}')
+                logDebug(f'      exposed: {self._exposedMelds}')
             else:
                 if Debug.scoring:
-                    logDebug('{} lost meld {}'.format(self, popped))
+                    logDebug(f'{self} lost meld {popped}')
         self._hand = None
 
 
@@ -481,8 +524,8 @@ class ScoringGame(Game):
     playerClass = ScoringPlayer
     wallClass = UIWall
 
-    def __init__(self, names, ruleset,
-                 gameid=None, client=None, wantedGame=None):
+    def __init__(self, names:List[Tuple[Wind, str]], ruleset:'Ruleset', gameid:Optional[int]=None,
+        wantedGame:Optional[str]=None, client:Optional['Client']=None) ->None:
         Game.__init__(
             self,
             names,
@@ -490,35 +533,35 @@ class ScoringGame(Game):
             gameid=gameid,
             client=client,
             wantedGame=wantedGame)
+        self.wall:'UIWall'
         self.shouldSave = True
-        scene = Internal.scene
-        scene.selectorBoard.load(self)
+        scoringScene().selectorBoard.load(self)
         self.prepareHand()
         self.initHand()
-        Internal.scene.mainWindow.adjustMainView()
-        Internal.scene.mainWindow.updateGUI()
+        scoringScene().mainWindow.adjustMainView()
+        scoringScene().mainWindow.updateGUI()
         self.wall.decorate4()
         self.throwDices()
 
-    @Game.seed.getter
-    def seed(self): # looks like a pylint bug pylint: disable=invalid-overridden-method
+    @Game.seed.getter  # type:ignore[attr-defined]
+    def seed(self) ->int: # looks like a pylint bug pylint: disable=invalid-overridden-method
         """a scoring game never has a seed"""
-        return None
+        return 0
 
-    def _setHandSeed(self):
+    def _setHandSeed(self) ->None:
         """a scoring game does not need this"""
         return None
 
-    def prepareHand(self):
+    def prepareHand(self) ->None:
         """prepare a scoring game hand"""
         Game.prepareHand(self)
         if not self.finished():
-            selector = Internal.scene.selectorBoard
+            selector = scoringScene().selectorBoard
             selector.refill()
             selector.hasLogicalFocus = True
             self.wall.build(shuffleFirst=False)
 
-    def nextScoringHand(self):
+    def nextScoringHand(self) ->None:
         """save hand to database, update score table and balance in status line, prepare next hand"""
         if self.winner:
             for player in self.players:
@@ -531,15 +574,15 @@ class ScoringGame(Game):
         self.maybeRotateWinds()
         self.prepareHand()
         self.initHand()
-        Internal.scene.scoringDialog.clear()
+        if _ := scoringScene().scoringDialog:
+            _.clear()
 
-    def close(self):
+    def close(self) ->None:
         """log off from the server and return a Deferred"""
-        scene = Internal.scene
-        scene.selectorBoard.uiTiles = []
-        scene.selectorBoard.allSelectorTiles = []
-        if isAlive(scene):
-            scene.removeTiles()
+        scoringScene().selectorBoard.uiTiles = []
+        scoringScene().selectorBoard.allSelectorTiles = []
+        if isAlive(scoringScene()):
+            scoringScene().removeTiles()
         for player in self.players:
             player.hide()
         if self.wall:
@@ -548,11 +591,11 @@ class ScoringGame(Game):
         return Game.close(self)
 
     @staticmethod
-    def isScoringGame():
+    def isScoringGame() ->bool:
         """are we scoring a manual game?"""
         return True
 
-    def saveStartTime(self):
+    def saveStartTime(self) ->None:
         """write a new entry in the game table with the selected players"""
         Game.saveStartTime(self)
         # for PlayingGame, this one is already done in
@@ -563,35 +606,34 @@ class ScoringGame(Game):
             Query('insert into server(url,lastruleset) values(?,?)',
                   (self.ruleset.rulesetId, Query.localServerName))
 
-    def _setGameId(self):
+    def _setGameId(self) ->None:
         """get a new id"""
         if not self.gameid:
             # a loaded game has gameid already set
             self.gameid = self._newGameId()
 
-    def _mustExchangeSeats(self, pairs):
+    def _mustExchangeSeats(self, pairs:List[List[Player]]) ->List[List[Player]]:
         """filter: which player pairs should really swap places?"""
-        # pylint: disable=no-self-use
         # I do not understand the logic of the exec return value. The yes button returns 0
         # and the no button returns 1. According to the C++ doc, the return value is an
         # opaque value that should not be used."""
         return [x for x in pairs if SwapDialog(x).exec_() == 0]
 
-    def savePenalty(self, player, offense, amount):
+    def savePenalty(self, player:'ScoringPlayer', offense:'Rule', amount:int) ->None:
         """save computed values to database, update score table and balance in status line"""
         scoretime = datetime.datetime.now().replace(microsecond=0).isoformat()
+        assert self.gameid
         Query("INSERT INTO SCORE "
               "(game,penalty,hand,data,manualrules,player,scoretime,"
               "won,prevailing,wind,points,payments, balance,rotated,notrotated) "
-              "VALUES(%d,1,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" %
-              (self.gameid, self.handctr, player.nameid,
-               scoretime, int(player == self.winner),
-               self.roundWind, player.wind, 0,
-               amount, player.balance, self.rotated, self.notRotated),
+              f"VALUES({int(self.gameid)},1,{int(self.handctr)},?,?,{player.nameid},'{scoretime}',"
+              f"{int(player == self.winner)},'{self.roundWind}','{player.wind}',0,{amount},"
+              f"{player.balance},{int(self.rotated)}, {int(self.notRotated)})",
               (player.hand.string, offense.name))
+        assert Internal.mainWindow
         Internal.mainWindow.updateGUI()
 
-def scoreGame():
+def scoreGame() ->Optional[ScoringGame]:
     """show all games, select an existing game or create a new game"""
     Players.load()
     if len(Players.humanNames) < 4:
@@ -605,7 +647,7 @@ def scoreGame():
     selected = gameSelector.selectedGame
     gameSelector.close()
     if selected is not None:
-        return ScoringGame.loadFromDB(selected)
+        return cast(ScoringGame, ScoringGame.loadFromDB(selected))
     selectDialog = SelectPlayers()
     if not selectDialog.exec_():
         return None
