@@ -9,7 +9,9 @@ SPDX-License-Identifier: GPL-2.0
 
 import datetime
 
-from qt import Qt, QAbstractTableModel
+from typing import TYPE_CHECKING, Optional, List, Any, cast
+
+from qt import Qt, QAbstractTableModel, QModelIndex
 from qt import QDialog, QDialogButtonBox, QWidget
 from qt import QHBoxLayout, QVBoxLayout, QAbstractItemView
 from qt import QItemSelectionModel, QGridLayout, QColor, QPalette
@@ -28,28 +30,34 @@ from common import Internal, Debug
 from modeltest import ModelTest
 from chat import ChatMessage, ChatWindow
 
+if TYPE_CHECKING:
+    from client import ClientTable
+    from qt import QObject, QEvent, QItemSelection
+    from login import Url
+    from humanclient import HumanClient
+    from scene import PlayingScene
 
 class TablesModel(QAbstractTableModel):
 
     """a model for our tables"""
 
-    def __init__(self, tables, parent=None):
+    def __init__(self, tables:List['ClientTable'], parent:Optional['QObject']=None) ->None:
         super().__init__(parent)
         self.tables = tables
         assert isinstance(tables, list)
 
-    def headerData(  # pylint: disable=no-self-use
-            self, section,
-            orientation, role=Qt.DisplayRole):
+    def headerData(
+            self, section:int,
+            orientation:Qt.Orientation, role:int=Qt.ItemDataRole.DisplayRole) ->Any:
         """show header"""
-        if role == Qt.TextAlignmentRole:
-            if orientation == Qt.Horizontal:
+        if role == Qt.ItemDataRole.TextAlignmentRole:
+            if orientation == Qt.Orientation.Horizontal:
                 if section in [3, 4]:
-                    return int(Qt.AlignLeft)
-                return int(Qt.AlignHCenter | Qt.AlignVCenter)
-        if role != Qt.DisplayRole:
+                    return int(Qt.AlignmentFlag.AlignLeft)
+                return int(Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignVCenter)
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
-        if orientation != Qt.Horizontal:
+        if orientation != Qt.Orientation.Horizontal:
             return int(section + 1)
         result = ''
         if section < 5:
@@ -61,34 +69,34 @@ class TablesModel(QAbstractTableModel):
                       i18n('Ruleset')][section]
         return result
 
-    def rowCount(self, parent=None):
+    def rowCount(self, parent:QModelIndex=QModelIndex()) ->int:
         """how many tables are in the model?"""
-        if parent and parent.isValid():
+        if parent.isValid():
             # we have only top level items
             return 0
         return len(self.tables)
 
-    def columnCount(self, unusedParent=None):  # pylint: disable=no-self-use
+    def columnCount(self, unusedParent:QModelIndex=QModelIndex()) ->int:
         """for now we only have id (invisible), id (visible), players,
         status, ruleset.name.
         id(invisible) always holds the real id, also 1000 for suspended tables.
         id(visible) is what should be displayed."""
         return 5
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index:QModelIndex, role:int=Qt.ItemDataRole.DisplayRole) ->Any:
         """score table"""
         # pylint: disable=too-many-branches,too-many-locals
-        result = None
-        if role == Qt.TextAlignmentRole:
+        result:Any = None
+        if role == Qt.ItemDataRole.TextAlignmentRole:
             if index.column() == 0:
-                result = int(Qt.AlignHCenter | Qt.AlignVCenter)
+                result = int(Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignVCenter)
             else:
-                result = int(Qt.AlignLeft | Qt.AlignVCenter)
+                result = int(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
         if index.isValid() and (0 <= index.row() < len(self.tables)):
             table = self.tables[index.row()]
-            if role == Qt.DisplayRole and index.column() in (0, 1):
+            if role == Qt.ItemDataRole.DisplayRole and index.column() in (0, 1):
                 result = table.tableid
-            elif role == Qt.DisplayRole and index.column() == 2:
+            elif role == Qt.ItemDataRole.DisplayRole and index.column() == 2:
                 players = []
                 zipped = list(zip(table.playerNames, table.playersOnline))
                 for idx, pair in enumerate(zipped):
@@ -100,23 +108,19 @@ class TablesModel(QAbstractTableModel):
                         color = palette.color(
                             QPalette.Active,
                             QPalette.WindowText).name()
-                        style = ('font-weight:normal;'
-                                 'font-style:normal;color:%s'
-                                 % color)
+                        style = f'font-weight:normal;font-style:normal;color:{color}'
                     else:
                         color = palette.color(
                             QPalette.Disabled,
                             QPalette.WindowText).name()
-                        style = ('font-weight:100;font-style:italic;color:%s'
-                                 % color)
+                        style = f'font-weight:100;font-style:italic;color:{color}'
                     players.append(
-                        '<nobr style="%s">' %
-                        style +
+                        f'<nobr style="{style}">' +
                         name +
                         '</nobr>')
                 names = ''.join(players)
                 result = names
-            elif role == Qt.DisplayRole and index.column() == 3:
+            elif role == Qt.ItemDataRole.DisplayRole and index.column() == 3:
                 status = table.status()
                 if table.suspendedAt:
                     dateVal = ' ' + datetime.datetime.strptime(
@@ -127,12 +131,11 @@ class TablesModel(QAbstractTableModel):
                     dateVal = ''
                 result = i18nc('table status', status) + dateVal
             elif index.column() == 4:
-                if role == Qt.DisplayRole:
+                if role == Qt.ItemDataRole.DisplayRole:
                     result = i18n((table.myRuleset if table.myRuleset else table.ruleset).name)
-                elif role == Qt.ForegroundRole:
+                elif role == Qt.ItemDataRole.ForegroundRole:
                     palette = KApplication.palette()
-                    color = palette.windowText().color() if table.myRuleset else 'red'
-                    result = QColor(color)
+                    result = palette.windowText().color() if table.myRuleset else QColor('red')
         return result
 
 
@@ -140,7 +143,7 @@ class SelectRuleset(QDialog):
 
     """a dialog for selecting a ruleset"""
 
-    def __init__(self, server=None):
+    def __init__(self, server:Optional['Url']=None) ->None:
         QDialog.__init__(self, None)
         decorateWindow(self, i18n('Select a ruleset'))
         self.buttonBox = KDialogButtonBox(self)
@@ -163,15 +166,15 @@ class TableList(QWidget):
     """a widget for viewing, joining, leaving tables"""
     # pylint: disable=too-many-instance-attributes
 
-    def __init__(self, client):
+    def __init__(self, client:'HumanClient'):
         super().__init__(None)
         self.autoStarted = False
         self.client = client
         self.setObjectName('TableList')
         self.resize(700, 400)
         self.view = MJTableView(self)
-        self.differ = None
-        self.debugModelTest = None
+        self.__differ:RulesetDiffer
+        self.debugModelTest:ModelTest
         self.requestedNewTable = False
         self.view.setItemDelegateForColumn(
             2,
@@ -229,14 +232,14 @@ class TableList(QWidget):
 
         self.view.doubleClicked.connect(client.joinTable)
         StateSaver(self, self.view.horizontalHeader())
-        self.updateButtonsForTable(None)
+        self.__updateButtonsForNoTable()
 
-    def hideEvent(self, unusedEvent):  # pylint: disable=no-self-use
+    def hideEvent(self, unusedEvent:'QEvent') ->None:
         """table window hides"""
         scene = Internal.scene
         if scene:
-            scene.startingGame = False
-        model = self.view.model()
+            cast('PlayingScene', scene).startingGame = False
+        model = cast(TablesModel, self.view.model())
         if model:
             for table in model.tables:
                 if table.chatWindow:
@@ -247,16 +250,19 @@ class TableList(QWidget):
                 # do we still need this connection?
                 self.client.logout()
 
-    def chat(self):
+    def chat(self) ->None:
         """chat. Only generate ChatWindow after the
         message has successfully been sent to the server.
         Because the server might have gone away."""
-        def initChat(_):
+        msg:ChatMessage
+        def initChat(_:Any) ->None:
             """now that we were able to send the message to the server
             instantiate the chat window"""
-            table.chatWindow = ChatWindow(table=table)
+            assert table
+            table.chatWindow = ChatWindow(table)
             table.chatWindow.receiveLine(msg)
         table = self.selectedTable()
+        assert table
         if not table.chatWindow:
             line = i18nE('opens a chat window')
             msg = ChatMessage(
@@ -264,7 +270,7 @@ class TableList(QWidget):
                 table.client.name,
                 line,
                 isStatusMessage=True)
-            table.client.sendChat(
+            cast('HumanClient', table.client).sendChat(
                 msg).addCallback(
                     initChat).addErrback(
                         self.client.tableError)
@@ -273,18 +279,19 @@ class TableList(QWidget):
         else:
             table.chatWindow.show()
 
-    def show(self):
+    def show(self) ->None:
         """prepare the view and show it"""
         if self.client.hasLocalServer():
             title = i18n(
                 'Local Games with Ruleset %1',
                 self.client.ruleset.name)
         else:
+            assert self.client.connection
             title = i18n('Tables at %1', self.client.connection.url)
+        assert self.client.name
         decorateWindow(self, ' - '.join([self.client.name, title]))
         self.view.hideColumn(1)
-        tableCount = self.view.model().rowCount(
-            None) if self.view.model() else 0
+        tableCount = self.view.model().rowCount() if self.view.model() else 0
         self.view.showColumn(0)
         self.view.showColumn(2)
         self.view.showColumn(4)
@@ -295,21 +302,36 @@ class TableList(QWidget):
                 self.view.hideColumn(2)
                 self.view.hideColumn(4)
 
-    def selectTable(self, idx):
+    def selectTable(self, idx:int) ->None:
         """select table by idx"""
         self.view.selectRow(idx)
         self.updateButtonsForTable(self.selectedTable())
 
-    def updateButtonsForTable(self, table):
+    def __updateButtonsForNoTable(self) ->None:
+        """update button status when no table is selected"""
+        self.joinButton.setEnabled(False)
+        self.leaveButton.setVisible(False)
+        self.compareButton.setVisible(False)
+        self.startButton.setVisible(False)
+        self.newButton.setToolTip(i18n("Allocate a new table"))
+        self.joinButton.setText(i18n('&Join'))
+        self.joinButton.setToolTip(i18n("Join a table"))
+        self.startButton.setEnabled(False)
+        self.compareButton.setEnabled(False)
+        self.chatButton.setVisible(False)
+
+    def updateButtonsForTable(self, table:Optional['ClientTable']=None) ->None:
         """update button status for the currently selected table"""
-        hasTable = bool(table)
-        suspended = hasTable and bool(table.suspendedAt)
-        running = hasTable and table.running
+        if table is None:
+            self.__updateButtonsForNoTable()
+            return
+        assert table
+        suspended = bool(table.suspendedAt)
+        running = table.running
         suspendedLocalGame = (
             suspended and table.gameid
             and self.client.hasLocalServer())
         self.joinButton.setEnabled(
-            hasTable and
             not running and
             not table.isOnline(self.client.name) and
             (self.client.name in table.playerNames) == suspended)
@@ -327,14 +349,14 @@ class TableList(QWidget):
             self.joinButton.setText(i18n('&Join'))
             self.joinButton.setToolTip(i18n("Join a table"))
         self.leaveButton.setEnabled(
-            hasTable and not running and not self.joinButton.isEnabled())
+            not running and not self.joinButton.isEnabled())
         self.startButton.setEnabled(
-            not running and not suspendedLocalGame and hasTable
+            not running and not suspendedLocalGame
             and self.client.name == table.playerNames[0])
-        self.compareButton.setEnabled(hasTable and table.myRuleset is None)
+        self.compareButton.setEnabled(table.myRuleset is None)
         self.chatButton.setVisible(not self.client.hasLocalServer())
         self.chatButton.setEnabled(
-            not running and hasTable
+            not running
             and self.client.name in table.playerNames
             and sum(x.startswith('Robot ') for x in table.playerNames) < 3)
         if self.chatButton.isEnabled():
@@ -345,48 +367,53 @@ class TableList(QWidget):
                 "For chatting with others on this table, "
                 "please first take a seat"))
 
-    def selectionChanged(self, selected, unusedDeselected):
+    def selectionChanged(self, selected:'QItemSelection', unusedDeselected:'QItemSelection') ->None:
         """update button states according to selection"""
         if selected.indexes():
             self.selectTable(selected.indexes()[0].row())
 
-    def selectedTable(self):
+    def selectedTable(self) ->Optional['ClientTable']:
         """return the selected table"""
         if self.view.selectionModel():
             index = self.view.selectionModel().currentIndex()
-            if index.isValid() and self.view.model():
-                return self.view.model().tables[index.row()]
+            model = cast(TablesModel, self.view.model())
+            if index.isValid() and model:
+                return model.tables[index.row()]
         return None
 
-    def compareRuleset(self):
+    def compareRuleset(self) ->None:
         """compare the ruleset of this table against ours"""
         table = self.selectedTable()
-        self.differ = RulesetDiffer(table.ruleset, Ruleset.availableRulesets())
-        self.differ.show()
+        if table:
+            self.__differ = RulesetDiffer([table.ruleset], Ruleset.availableRulesets())
+            self.__differ.show()
 
-    def startGame(self):
+    def startGame(self) ->None:
         """start playing at the selected table"""
         table = self.selectedTable()
-        self.startButton.setEnabled(False)
-        self.client.callServer(
-            'startGame',
-            table.tableid).addErrback(
-                self.client.tableError)
+        if table:
+            self.startButton.setEnabled(False)
+            self.client.callServer(
+                'startGame',
+                table.tableid).addErrback(
+                    self.client.tableError)
 
-    def leaveTable(self):
+    def leaveTable(self) ->None:
         """leave a table"""
         table = self.selectedTable()
-        self.client.callServer(
-            'leaveTable',
-            table.tableid).addErrback(
-                self.client.tableError)
+        if table:
+            self.client.callServer(
+                'leaveTable',
+                table.tableid).addErrback(
+                    self.client.tableError)
 
-    def __keepChatWindows(self, tables):
+    def __keepChatWindows(self, tables:List['ClientTable']) ->None:
         """copy chatWindows from the old table list which will be
         thrown away"""
-        if self.view.model():
-            chatWindows = {x.tableid: x.chatWindow for x in self.view.model().tables}
-            unusedWindows = {x.chatWindow for x in self.view.model().tables}
+        model = cast(TablesModel, self.view.model())
+        if model:
+            chatWindows = {x.tableid: x.chatWindow for x in model.tables}
+            unusedWindows = {x.chatWindow for x in model.tables}
             for table in tables:
                 table.chatWindow = chatWindows.get(table.tableid, None)
                 unusedWindows -= {table.chatWindow}
@@ -394,7 +421,7 @@ class TableList(QWidget):
                 if unusedWindow:
                     unusedWindow.hide()
 
-    def __preselectTableId(self, tables):
+    def __preselectTableId(self, tables:List['ClientTable']) ->int:
         """which table should be preselected?
         If we just requested a new table:
           select first new table.
@@ -407,17 +434,18 @@ class TableList(QWidget):
           select first table"""
         if self.requestedNewTable:
             self.requestedNewTable = False
-            model = self.view.model()
+            model = cast(TablesModel, self.view.model())
             if model:
                 oldIds = {x.tableid for x in model.tables}
                 newIds = sorted({x.tableid for x in tables} - oldIds)
                 if newIds:
                     return newIds[0]
-        if self.selectedTable():
-            return self.selectedTable().tableid
+        _ = self.selectedTable()
+        if _:
+            return _.tableid
         return 0
 
-    def loadTables(self, tables):
+    def loadTables(self, tables:List['ClientTable']) ->None:
         """build and use a model around the tables.
         Show all new tables (no gameid given yet) and all suspended
         tables that also exist locally. In theory all suspended games should
@@ -428,7 +456,7 @@ class TableList(QWidget):
         if Debug.table:
             for table in tables:
                 if table.gameid and not table.gameExistsLocally():
-                    logDebug('Table %s does not exist locally' % table)
+                    logDebug(f'Table {table} does not exist locally')
         tables = [x for x in tables if not x.gameid or x.gameExistsLocally()]
         tables.sort(key=lambda x: x.tableid)
         preselectTableId = self.__preselectTableId(tables)
