@@ -8,7 +8,10 @@ SPDX-License-Identifier: GPL-2.0
 
 """
 
+from typing import cast
+
 from qt import QWidget, QHBoxLayout, QLineEdit
+from tile import Tile
 from tileset import Tileset
 from uitile import UITile
 from board import Board, FittingView
@@ -23,9 +26,10 @@ class TilesetSelector(QWidget):
 
     """presents all available tiles with previews"""
 
-    def __init__(self, parent):
+    def __init__(self, parent:QWidget) ->None:
         super().__init__(parent)
 
+        assert Internal.Preferences
         loadUi(self)
         self.kcfg_tilesetName = QLineEdit(self)
         self.kcfg_tilesetName.setVisible(False)
@@ -34,10 +38,11 @@ class TilesetSelector(QWidget):
         self.tileScene = SceneWithFocusRect()
         self.tileView = FittingView()
         self.tileView.setScene(self.tileScene)
-        self.tileset = Tileset(Internal.Preferences.tilesetName)
-        self.uiTiles = [UITile('w' + s.char.lower()) for s in Wind.all4]
+        _ = Internal.Preferences.tilesetName
+        assert isinstance(_, str)
+        self.tileset = Tileset(_)
+        self.uiTiles = [UITile(Tile('w' + s.char.lower())) for s in Wind.all4]
         self.board = Board(2, 2, self.tileset)
-        self.board.showShadows = True
         self.tileScene.addItem(self.board)
         self.tileView.setParent(self.tilesetPreview)
         layout = QHBoxLayout(self.tilesetPreview)
@@ -49,9 +54,10 @@ class TilesetSelector(QWidget):
             self.uiTiles[idx].focusable = False
         self.setUp()
 
-    def setUp(self):
+    def setUp(self) ->None:
         """set-up the selector"""
 
+        assert Internal.Preferences
         # The lineEdit widget holds our tileset path, but the user does
         # not manipulate it directly
         self.kcfg_tilesetName.hide()
@@ -64,9 +70,9 @@ class TilesetSelector(QWidget):
         self.tilesetList = Tileset.available()
         for aset in self.tilesetList:
             self.tilesetNameList.addItem(aset.name)
-        self.kcfg_tilesetName.setText(Internal.Preferences.tilesetName)
+        self.kcfg_tilesetName.setText(cast(str, Internal.Preferences.tilesetName))
 
-    def tilesetNameChanged(self, name):
+    def tilesetNameChanged(self, name:str) ->None:
         """the name changed: update the current row"""
         igrindex = 0
         for idx, aset in enumerate(self.tilesetList):
@@ -75,7 +81,7 @@ class TilesetSelector(QWidget):
                 break
         self.tilesetNameList.setCurrentRow(igrindex)
 
-    def tilesetRowChanged(self):
+    def tilesetRowChanged(self) ->None:
         """user selected a new tileset, update our information about it and
         paint preview"""
         selTileset = self.tilesetList[self.tilesetNameList.currentRow()]
